#! /bin/bash

export eiffeldoc="EiffelDoc v1.4"
export copyleft="&copy; 2004, Cyril Adrian"

# This script is discibuted under the GPL licence, version 2, the text of
# which may be found at htp://www.gnu.org

# EiffelDoc is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
# details.

# ---------------------------------------------------------------------- #
# Those variables may be changed by command-line options:

export arrow="../images/FlecheRouge.gif"
export background="../images/fond_gw.gif"
export home_page="../images/se-home.gif"
export seline="../images/se-line.gif"

export htmldir=$(pwd)/site

# ---------------------------------------------------------------------- #
# Some tools:

export not_distributed=$(pwd)/not_distributed.txt

filter_distrib() {
    # Filter out the non-distributed clusters
    while read dir; do
	if grep -qs '^'$dir'$' $not_distributed; then
	    :
	else
	    echo $dir
	fi
    done
}

dos2unix() {
    # Emulate the "dos2unix" command (not always available)
    sed 's/\r//g'
}

# ---------------------------------------------------------------------- #
# The script itself:

body_of() {
    html_file=$1
    if grep -qs '<body>' $html_file; then
        awk '/<body>/ {if (i==0) i=1;next} /<\/body>/ {i=2;next} {if (i==1) print $0}' $html_file
    else
        cat $html_file
    fi
}

desc_summary() {
    html_file=$1
    lynx -dump $html_file | awk -F'.' '$0 ~ /./ { printf("%s.\n", $1); exit 0 } { print $0 }'
}

list_clients() {
    source=$1
    {
        echo ANY
        echo NONE
        cat $source | grep -e '^feature' | awk '{print $2}' | sed 's/[{}]//g' | sed 's/--.*$//' | awk -F, '{for (i = NF; i > 0; i--) printf("%s\n", $i) }'
    } | dos2unix | sort -u
}

do_short() {
    source=$1
    url_target=${source%.e}.html
    html_target=$htmldir/$url_target

    /bin/rm -f $tmpfile
    list_clients $source > $tmpfile

    w=$((100 / $(cat $tmpfile | wc -l)))

    for client in $(list_clients $source); do

        if [ $client = ANY ]; then
            url_client_target=$url_target
        else
            url_client_target=_${client}_${url_target}
        fi

        html_client_target=$htmldir/$url_client_target
        tmp_client_target=${html_client_target}~

        if newer $source $html_client_target; then
            short -html2 -client $client $source | sed 's/<a/<a target="content"/g' > $tmp_client_target
            {
                awk '/<BODY/ {print; exit 0} {print}' $tmp_client_target
                echo '<table width="100%" border="0" cellspacing="10"><tr>'
                for c in $(< $tmpfile); do
                    echo -n '<td align="center" width="'$w'%" bgcolor="'
                    if [ $c = $client ]; then
                        echo '#3366FF"><b><font color="#FFFFFF">'
                        echo -n $c
                        echo '</font></b>'
                    else
                        echo '#000099">'
                        echo -n '<a href="'
                        if [ $c = ANY ]; then
                            echo -n $url_target
                        else
                            echo -n _${c}_$url_target
                        fi
                        echo -n '"><b><font color="#66FF66">'
                        echo -n $c
                        echo '</font></b></a>'
                    fi
                    echo '</td>'
                done
                echo '</tr></table>'
                awk '/<BODY/ {i=1; next} {if (i==1) print}' $tmp_client_target
            } > $html_client_target
            /bin/rm -f $tmp_client_target
        fi
    done
}

do_generate() {
    root_cluster="$1"
    shift
    for dir in $@; do
        (
            export cluster
            if [ $root_cluster = root ]; then
                cluster=$dir
            else
                cluster=${root_cluster}@${dir}
            fi
            clustername=$(echo $cluster | tr '@' '/')

            echo "Entering cluster "$clustername
            cd $dir

            export url_cluster_summary=summary_$cluster".html"
            export url_cluster_classes=cluster_$cluster".html"

            export html_cluster_summary=$htmldir/$url_cluster_summary
            export html_cluster_classes=$htmldir/$url_cluster_classes

            /bin/rm -f $html_cluster_summary $html_cluster_classes


            # Are there any Eiffel files in this cluster?

            if ls *.e > /dev/null 2>&1; then

                # ----- Preparing cluster-wide files:

                # The cluster classes list
                {
                    echo -n '<br><a href="'
                    echo -n $url_cluster_classes
                    echo -n '" target="cluster">'
                    echo -n '<img src="'$arrow'" width="14" height="12" border="0" alt="-&gt;"><font color="#66FF66"><b>'
                    echo -n $clustername
                    echo "</b></font></a>"
                } >> $html_clusters

                # The cluster summary
                {
                    echo -n '<tr><td valign="top"><a href="'
                    echo -n $url_cluster_summary
                    echo -n '">'
                    echo -n $clustername
                    echo '</a></td><td align="justify">'
                    if [ -e cluster.html ]; then
                        desc_summary cluster.html
                    else
                        echo -n "&nbsp;"
                    fi
                    echo "<br>&nbsp;</td></tr>"
                } >> $html_summary

                # The link to the cluster description
                {
                    echo '<html><head></head>'
                    echo '<!-- Generated by '$eiffeldoc' '$copyleft' -->'
                    echo '<body background="'$background'">'
                    echo -n '<table bgcolor="#0080C0" width="100%" nosave><tr><td valign="center" align="center">'
                    echo -n '<a href="'
                    echo -n $url_cluster_summary
                    echo '" target="content">'
		    echo -n '<font color="#66FF66" size="+1"><b>Cluster '
                    echo -n $clustername
                    echo '</b></font></a></td></tr></table><br>'
                    echo '<table border="0" cellspacing="3" cellspacing="3" bgcolor="#000099" width="100%">'
                    echo '<tr>'
                    echo '<td nowrap>'
                } > $html_cluster_classes

                # The description target link
                {
                    echo '<html><head></head>'
                    echo '<!-- Generated by '$eiffeldoc' '$copyleft' -->'
                    echo '<body bgcolor="#FFFFFF">'
                    echo -n '<table bgcolor="#3366FF" width="100%"><tr><td valign="center" align="center"><b><font color="#FFFFFF" size="+4">'
                    echo -n $clustername
                    echo "</font></b></td></tr></table>"
                    if [ -e cluster.html ]; then
                        echo '<table width="100%" border="0"><tr><td width="60%"></td><td>'
                        echo '<table width="100%" border="0" bgcolor="#FFF3BB"><tr><td align="justify">'
                        desc_summary cluster.html
                        echo '</td></tr><tr><td align="right"><a href="#description"><img src="'$arrow'" alt="-&gt;" border="0"><b>See description</b></a></td></tr></table><br>&nbsp;'
                        echo '</td></tr></table>'
                    fi
                    echo '<br><table border="0">'
                } > $html_cluster_summary


                # ----- Read the Eiffel files in this cluster:

                export source
                for source in *.e; do
                    echo "    Generating $source"

                    export url_target=${source%.e}.html
                    export html_target=$htmldir/$url_target

                    # Create the HTML files thanks to "short"

                    do_short $source

                    export classname=$(echo ${source%.e} | tr '[a-z]' '[A-Z]')

                    # Add the class name in the global table
                    {
                        echo -n '<a name="'
                        echo -n $classname
                        echo -n '"></a><a href="'
                        echo -n $url_target
                        echo -n '" target="content">'
                        echo -n '<img src="'$arrow'" alt="-&gt;" width="14" height="12" border="0" alt="-&gt;"><font color="#66FF66"><b>'
                        echo -n $classname
                        echo "</b></font></a><br>"
                    } >> $html_classes

                    # Add the class name in the cluster table
                    {
                        echo -n '<a href="'
                        echo -n $url_target
                        echo -n '" target="content">'
                        echo -n '<img src="'$arrow'" alt="-&gt;" width="14" height="12" border="0" alt="-&gt;"><font color="#66FF66"><b>'
                        echo -n $classname
                        echo "</b></font></a><br>"
                    } >> $html_cluster_classes

                    # Add the class description in the cluster summary
                    {
                        awk '$1 ~ /^--/ {if (i==1) print $0; next} /class/ {if (i==0) i=1; next} /(creation|inherit|feature|invariant|end)/ {i=2; next}' $source | sed 's/^[ \t]*--//' | awk '$0 ~ /^[ \t\r]*$/ { if (p==1) cr++; next } { if (cr > 0) for (; cr--; cr >= 0) printf("\n"); print $0; p=1; cr=0 }' > $tmpfile

                        echo -n '<tr><td valign="top"><a href="'
                        echo -n $url_target
                        echo -n '">'
                        echo -n $classname
                        echo -n '</a></td><td align="justify" nowrap><pre>'
                        cat $tmpfile
                        if [ $(cat $tmpfile | wc -l) -eq 0 ]; then
                            echo "&nbsp;"
                        fi
                        echo ""
                        echo "</pre></td></tr>"

                        /bin/rm -f $tmpfile
                    } >> $html_cluster_summary

               done

                {
                    echo "</table>"
                    if [ -e cluster.html ]; then
                        echo '<br>&nbsp;<br>'
                        echo '<center><img src="'$seline'"></center><p>'
                        echo -n '<a name="description"></a><h1>Cluster '
                        echo -n $clustername
                        echo ' description</h1>'
                        body_of cluster.html
                        echo '</p>'
                    fi
                    echo '<p>&nbsp;</p></body></html>'
                } >> $html_cluster_summary

                echo "</td></tr></table></body></html>" >> $html_cluster_classes

            fi


            # ----- Go recursively down clusters:

            ls -l | grep -q '^d' && do_generate $cluster $(ls -l | grep '^d' | awk '{print $9}' | grep -v '^CVS$' | filter_distrib)


            echo "Exitting cluster "$clustername
        )
    done
}


start() {

    {
        echo '<html><head></head>'
        echo '<!-- Generated by '$eiffeldoc' '$copyleft' -->'
        echo '<body background="'$background'">'
        cat <<EOF
  <table border="0" cellspacing="3" cellpadding="3" cols="1" width="220" bgcolor="#000099">
    <tr><td>
EOF
        echo '<a href="http://SmartEiffel.loria.fr/" target="_top"><center><img src="'$home_page'" alt="Return home page" border="0"></center></a>'
      cat <<EOF
    </td></tr>
  </table>
<br><br><br><br>
<!-- Begin Nedstat Basic code -->
<!-- Title: Library Interface -->
<!-- URL: http://SmartEiffel.loria.fr -->
<script language="JavaScript" src="http://m1.nedstatbasic.net/basic.js">
</script>
<script language="JavaScript">
<!--
  nedstatbasic("ABrwYQSwIQR24pPJcDEqqA0Xj7qA", 0);
// -->
</script>
<noscript>
<a target="_blank" href="http://v1.nedstatbasic.net/stats?ABrwYQSwIQR24pPJcDEqqA0Xj7qA"><img
src="http://m1.nedstatbasic.net/n?id=ABrwYQSwIQR24pPJcDEqqA0Xj7qA"
border="0" nosave width="18" height="18"></a>
</noscript>
<!-- End Nedstat Basic code -->
</body>
</html>
EOF
    } >$htmldir/return_home.html

    {
        echo '<html><head></head>'
        echo '<!-- Generated by '$eiffeldoc' '$copyleft' -->'
        echo '<body background="'$background'">'
        echo '<table border="0"><tr><td>&nbsp;</td></tr></table>'
        echo '</body></html>'
    } >$htmldir/separator.html

    {
        echo '<html><head><title>EiffelDoc</title></head>'
        echo '<!-- Generated by '$eiffeldoc' '$copyleft' -->'
cat <<EOF
  <frameset cols="250px,*" border="0" frameborder="no">
    <frameset rows="56px,30%,5px,*" border="0" frameborder="no" framespacing="0">
      <frame src="return_home.html" scrolling="no" frameborder="no">
      <frame src="clusters.html" frameborder="no">
      <frame src="separator.html" scrolling="no" frameborder="no">
      <frame src="all_classes.html" frameborder="no" name="cluster">
    </frameset>
    <frame src="summary.html" name="content">
  </frameset>
<noframes>
  <h2>Frame Alert</h2>
  <p>This document is designed to be viewed using the frames feature.</p>
  <p>You may browse the <a href="summary.html">no-frames site</a>.
</noframes>
</html>
EOF
    } >$htmldir/index.html

    {
        echo '<html><head></head>'
        echo '<!-- Generated by '$eiffeldoc' '$copyleft' -->'
        echo '<body background="'$background'"><table border="0">'
        echo '<table border="0" cellspacing="3" cellpadding="3" bgcolor="#000099" width="100%">'
        echo '<tr>'
        echo '<td nowrap>'
        echo '<a href="all_classes.html" target="cluster">'
	echo -n '<img src="'$arrow'" width="14" height="12" border="0" alt="-&gt;">'
	echo '<font color="#66FF66"><b>All clusters</b></font></a>'
    } > $html_clusters

    {
        echo '<html><head></head>'
        echo '<!-- Generated by '$eiffeldoc' '$copyleft' -->'
        echo '<body bgcolor="#FFFFFF"><table border="0">'
        echo '<table bgcolor="#3366FF" width="100%"><tr><td valign="center" align="center"><b><font color="#FFFFFF" size="+4">All Clusters</font></b></td></tr></table><br><table border="0">'
    } > $html_summary
}

finish() {
    /bin/mv $html_classes ${html_classes}~
    {
        echo '<html><head></head><body background="'$background'"><table>'
        echo '<table bgcolor="#0080C0" width="100%"><tr><td valign="center" align="center">'
        echo -n '<a href="summary.html" target="content">'
	echo -n '<font color="#66FF66" size="+1"><b>All Clusters'
	echo '</b></font></a></td></tr></table><br>'
        echo '<table border="0" cellspacing="3" cellspacing="3" bgcolor="#000099" width="100%">'
        echo '<tr>'
        echo '<td nowrap>'
        sort ${html_classes}~
        echo '</td></tr></table>'
        echo "</table></body></html>"
    } > $html_classes
    /bin/rm -f ${html_classes}~

    {
        echo "</table>"
        echo '<center><img src="'$seline'"></center><p><font size="-1"><pre>'
        short -version
        echo '</pre></font></p>'
        echo '<p><font size="-1"<pre align="center">'
        echo '<a href="eiffeldoc.sh">'$eiffeldoc'</a> '$copyleft
        echo '</pre></font></p>'
        echo "</table></body></html>"
    } >> $html_summary

    {
        echo '</td></tr></table>'
        echo "</table></body></html>"
    } >> $html_clusters

    /bin/rm -f $tmpfile
}

abort() {
    echo "Aborted by user" >&2
    finish
    exit 1
}

usage() {
    {
        echo "Usage:"
        echo "  $0 [-h|--help]"
        echo ""
        echo "  $0 [-a|--arrow=] <arrow image> [-b|--background=] <background image>"
        echo "     [-l|--line] <ruler image>"
        echo "     [-s|--site=] <site root directory>"
        echo "     [-- package ...]"
        echo ""
        echo "   -a   arrow image; default is "$arrow
        echo "   -b   background image; default is "$background
        echo "   -l   ruler image; default is "$seline
        echo "   -s   site root directory; default is "$htmldir
        echo ""
    } >&2
}

# ---------------------------------------------------------------------- #
# Parameters:

again=true;
while [ $# -gt 0 ] && $again; do
    case $1 in
        (-a)
        if [ x$2 = x ]; then
            usage
            exit 1
        fi
        arrow=$2
        shift
        ;;

        (-b)
        if [ x$2 = x ]; then
            usage
            exit 1
        fi
        background=$2
        shift
        ;;

        (-l)
        if [ x$2 = x ]; then
            usage
            exit 1
        fi
        seline=$2
        shift
        ;;

        (-s)
        if [ x$2 = x ]; then
            usage
            exit 1
        fi
        htmldir=$2
        shift
        ;;

        (-d)
        if [ x$2 = x ]; then
            usage
            exit 1
        fi
        SE=$2
        shift
        ;;

        (--arrow=*)
        arrow=$(echo $1 | awk -F= '{print $2}')
        ;;

        (--background=*)
        background=$(echo $1 | awk -F= '{print $2}')
        ;;

        (--line=*)
            seline=$(echo $1 | awk -F= '{print $2}')
            ;;

        (--site=*)
        htmldir=$(echo $1 | awk -F= '{print $2}')
        ;;

        (--dir=*)
        SE=$(echo $1 | awk -F= '{print $2}')
        ;;

        (--)
        # The following arguments will be passed to do_generate (see below)
        again=false;
        ;;

        (*)
        usage
        exit 1
        ;;

    esac
    shift
done

if [ $htmldir != /${htmldir#/} ]; then
    htmldir=$(pwd)/$htmldir
fi

while [ ${htmldir%/}/ = ${htmldir} ]; do
    htmldir=${htmldir%/}
done

echo "Generating in directory "$htmldir

# ---------------------------------------------------------------------- #
# MAIN

if [ ${SE:-unset} = unset ]; then
    if [ ${SmartEiffelDirectory:+set} = set -a -d ${SmartEiffelDirectory:=/usr/local/SmartEiffel} ]; then
        SE=${SmartEiffelDirectory}
    else
        if [ ${SmartEiffel:-unset} = unset ]; then
            echo "Please set the environment Variable 'SmartEiffel' with"
            echo "the absolute path of file SmartEiffel/sys/system.se"
            exit 1
        fi
        if [ ! -f ${SmartEiffel} ]; then
            echo "File ${SmartEiffel} not found."
            echo "Please check the value of the environment Variable"
            echo "'SmartEiffel' which must be the absolute path of "
            echo "the file SmartEiffel/sys/system.se"
            exit 1
        fi
        if [ $(basename $SmartEiffel) != system.se ]; then
            echo "Please check the value of the environment Variable"
            echo "'SmartEiffel' which must be the absolute path of "
            echo "the file SmartEiffel/sys/system.se"
            exit 1
        fi
        SE=$(dirname $SmartEiffel)
        SE=$(dirname $SE)
        if [ ! -d ${SE} ]; then
            echo "SmartEiffel home directory ${SE} not found."
            echo "Please check the value of the environment Variable"
            echo "'SmartEiffel' which must be the absolute path of "
            echo "the file SmartEiffel/sys/system.se"
            exit 1
        fi
    fi
fi

# Conventions:
#  - $html_* variables are filenames containing generated HTML.
#  - $url_* are filenames when used as URLs.

[ ! -d $htmldir ] && mkdir $htmldir

/bin/cp $0 $htmldir >/dev/null 2>&1

export html_clusters=$htmldir/clusters.html
export html_classes=$htmldir/all_classes.html
export html_summary=$htmldir/summary.html

export tmpfile=/tmp/eiffeldoc_$$

/bin/rm -f $html_index $html_clusters $html_classes $html_summary

cd $SE

start
trap abort HUP QUIT INT TERM

do_generate root lib "$@"

finish
