-- See the Copyright notice at the end of this file.
--
deferred class STREAM
	--
	-- A stream of characters.
	--
	-- There are two kinds of streams:
	-- + input streams (see INPUT_STREAM)
	-- + output_streams (see OUTPUT_STREAM)
	--
	-- Streams can:
	-- + be connected (e.g. to some system object)
	-- + be used ot read or write characters, only if they are connected
	-- + be filtered (see FILTER)
	--
	--*** Note that currently STREAM is inserted (not inherited) by INPUT_STREAM and OUTPUT_STREAM (due to
	-- limitations of the new Eiffel way, or bugs of the compiler, I don't know)
	--

feature {ANY}
	is_connected: BOOLEAN is
		deferred
		end

	disconnect is
		require
			is_connected
		deferred
		ensure
			filter = Void
		end

	is_filtered: BOOLEAN is
		do
			Result := filter /= Void
		end

	detach is
		deferred
		ensure
			not is_filtered
		end

feature {FILTER}
	set_filter (a_filter: like filter) is
		require
			a_filter /= Void
		do
			filter := a_filter
		ensure
			filter = a_filter
		end

	filter: FILTER

end -- class STREAM
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is free software, which comes along with SmartEiffel. This software is distributed in the hope that it will be
-- useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- You can modify it as you want, provided this footer is kept unaltered, and a notification of the changes is added.
-- You are allowed to redistribute it and sell it, alone or as a part of another product.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
