-- See the Copyright notice at the end of this file.
--
deferred class FRACTION_GENERAL_NUMBER
	--
	-- To implement NUMBER (do not use this class, see NUMBER).
	--

inherit
	NUMBER

feature {ANY}
	is_zero: BOOLEAN is False

	is_one: BOOLEAN is False

	factorial: NUMBER is
		do
			check
				False
			end
		end

	infix "@=" (other: INTEGER_64): BOOLEAN is
		do
		end

	infix "//" (other: NUMBER): NUMBER is
		do
			check
				False
			end
		end

	infix "@//" (other: INTEGER_64): NUMBER is
		do
			check
				False
			end
		end

	infix "\\" (other: NUMBER): NUMBER is
		do
			check
				False
			end
		end

	infix "@\\" (other: INTEGER_64): NUMBER is
		do
			check
				False
			end
		end

feature {ANY} -- Misc:

	hash_code: INTEGER is
		do
			Result := numerator.hash_code #+ denominator.hash_code
			if Result < 0 then
				Result := ~ Result
			end
		end
	
	gcd (other: NUMBER): INTEGER_GENERAL_NUMBER is
		do
			check False end
		end

feature {NUMBER}
	from_two_integer (n, d: INTEGER_64): NUMBER is
		require
			n /= 0
			d /= 0
			n \\ d /= 0
		local
			n_number, d_number: INTEGER_64_NUMBER
		do
			create n_number.make(n)
			create d_number.make(d)
			Result := from_two_integer_general_number(n_number, d_number)
		end

	from_two_integer_general_number (n, d: INTEGER_GENERAL_NUMBER): NUMBER is
		require
			n /= Void
			d /= Void
			not d.is_zero
		local
			neg: BOOLEAN; num, den: INTEGER_GENERAL_NUMBER; n_abs, d_abs, g: INTEGER_GENERAL_NUMBER
		do
			if n.is_zero then
				create {INTEGER_64_NUMBER} Result.make(0)
			else
				n_abs := n.abs
				d_abs := d.abs
				g := n_abs.gcd(d_abs)
				if g @= 1 then
					num := n_abs
					den := d_abs
				else
					num ?= n_abs // g
					den ?= d_abs // g
				end
				if n.is_positive = d.is_positive then
				else
					neg := True
				end
				if den @= 1 then
					if neg then
						Result := - num
					else
						Result := num
					end
				else
					create {FRACTION_WITH_BIG_INTEGER_NUMBER} Result.make_simply(num, den, neg)
				end
			end
		end

	from_integer_and_integer_general_number (n: INTEGER_64; d: INTEGER_GENERAL_NUMBER): NUMBER is
		require
			n /= 0
			d /= Void
			not d.is_zero
			not ((n.to_number \\ d) @= 0)
		local
			n_number: INTEGER_64_NUMBER
		do
			create n_number.make(n)
			Result := from_two_integer_general_number(n_number, d)
		end

	from_abstract_integer_and_integer (n: INTEGER_GENERAL_NUMBER; d: INTEGER_64): NUMBER is
		require
			d /= 0
			n /= Void
			not n.is_zero
			not ((n \\ d.to_number) @= 0)
		local
			d_number: INTEGER_64_NUMBER
		do
			create d_number.make(d)
			Result := from_two_integer_general_number(n, d_number)
		end

feature {}
	decimal_in (buffer: STRING; num, denom: NUMBER; negative: BOOLEAN; digits: INTEGER
		all_digits: BOOLEAN) is
		local
			n, div, remainder: NUMBER; counter: INTEGER
		do
			n := num
			if negative then
				buffer.extend('-')
			end
			div := n // denom
			div.append_in(buffer)
			remainder := n \\ denom
			if digits > 0 then
				if all_digits or else not (remainder @= 0) then
					buffer.extend('.')
					from
						counter := 1
					until
						counter > digits or else remainder @= 0
					loop
						n := remainder @* 10
						;
						(n // denom).append_in(buffer)
						remainder := n \\ denom
						counter := counter + 1
					end
					if all_digits then
						from
						until
							counter > digits
						loop
							buffer.extend('0')
							counter := counter + 1
						end
					end
				end
			end
		end

feature {NUMBER} -- Implementation:
	gcd_with_integer_64_number (other: INTEGER_64_NUMBER) : INTEGER_GENERAL_NUMBER is
		do
			check False end
		end

	gcd_with_big_integer_number (other: BIG_INTEGER_NUMBER) : INTEGER_GENERAL_NUMBER is
		do
			check False end
		end
	
end -- class FRACTION_GENERAL_NUMBER
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is free software, which comes along with SmartEiffel. This software is distributed in the hope that it will be
-- useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- You can modify it as you want, provided this footer is kept unaltered, and a notification of the changes is added.
-- You are allowed to redistribute it and sell it, alone or as a part of another product.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
