-- See the Copyright notice at the end of this file.
--
class FRACTION_WITH_BIG_INTEGER_NUMBER
	--
	-- To implement NUMBER (do not use this class, see NUMBER).
	--

inherit
	FRACTION_GENERAL_NUMBER

creation {ANY}
	make, make_simply

feature {ANY}
	-- *** Include the sign of the fraction in the numerator ?
	-- (Vincent Croizier, 27/07/04) ***
	numerator: INTEGER_GENERAL_NUMBER

	denominator: INTEGER_GENERAL_NUMBER

feature {NUMBER}
	make (n, d: INTEGER_GENERAL_NUMBER; s: BOOLEAN) is
			-- Create a simplified large_fraction
		require
			n.is_positive
			d.is_positive
			not ((n \\ d) @= 0)
		local
			gcd_frac, num, den: INTEGER_GENERAL_NUMBER
		do
			gcd_frac := n.gcd(d)
			num ?= n // gcd_frac
			den ?= d // gcd_frac
			numerator := num
			denominator := den
			is_negative := s
		end

	make_simply (n, d: INTEGER_GENERAL_NUMBER; s: BOOLEAN) is
			-- create a large_fraction without simplify it
		require
			n.is_positive
			d.is_positive
			not ((n \\ d) @= 0)
		do
			numerator := n
			denominator := d
			is_negative := s
		end

feature {ANY}
	is_negative: BOOLEAN

	inverse: NUMBER is
		do
			if numerator.is_one then
				if is_negative then
					Result := - denominator
				else
					Result := denominator
				end
			else
				create {FRACTION_WITH_BIG_INTEGER_NUMBER} Result.make_simply(denominator, numerator, is_negative)
			end
		end

	prefix "-": NUMBER is
		do
			create {FRACTION_WITH_BIG_INTEGER_NUMBER} Result.make_simply(numerator, denominator, not is_negative)
		end

	infix "+" (other: NUMBER): NUMBER is
		do
			Result := other.add_with_fraction_with_big_integer_number(Current)
		end

	is_positive: BOOLEAN is
		do
			Result := not is_negative
		end

	append_in (string: STRING) is
		do
			if is_negative then
				string.extend('-')
			end
			numerator.append_in(string)
			string.extend('/')
			denominator.append_in(string)
		end

	append_decimal_in (buffer: STRING; digits: INTEGER; all_digits: BOOLEAN) is
		do
			decimal_in(buffer, numerator, denominator, is_negative, digits, all_digits)
		end

	is_equal (other: NUMBER): BOOLEAN is
		local
			n2: like Current
		do
			n2 ?= other
			if n2 /= Void then
				if is_negative then
					if n2.is_negative then
						Result := denominator.is_equal(n2.denominator) and then numerator.is_equal(n2.numerator)
					end
				elseif n2.is_positive then
					Result := denominator.is_equal(n2.denominator) and then numerator.is_equal(n2.numerator)
				end
			end
		end

	to_double: DOUBLE is
			-- *** This is not very good, bad precision on big numbers
			-- (Vincent Croizier, 05/07/04) ***
		do
			Result := numerator.to_double / denominator.to_double
			if is_negative then
				Result := - Result
			end
		end

	infix "*" (other: NUMBER): NUMBER is
		do
			Result := other.multiply_with_fraction_with_big_integer_number(Current)
		end

	infix "@+" (other: INTEGER_64): NUMBER is
			-- Sum of 'Current' and 'other'.
			-- *** Can be very faster !! (Vincent Croizier, 04/07/04) ***
		local
			tmp: INTEGER_GENERAL_NUMBER
		do
			if is_negative then
				tmp ?= denominator @* other - numerator
			else
				tmp ?= denominator @* other + numerator
			end
			check
				tmp /= Void
			end
			create {FRACTION_WITH_BIG_INTEGER_NUMBER} Result.make(tmp.abs, denominator, tmp.is_negative)
		end

feature {NUMBER, NUMBER_TOOLS}
	add_with_big_integer_number (other: BIG_INTEGER_NUMBER): NUMBER is
		local
			num: INTEGER_GENERAL_NUMBER
		do
			if is_negative then
				num ?= other * denominator - numerator
				check
					num /= Void
				end
			else
				num ?= other * denominator + numerator
				check
					num /= Void
				end
			end
			Result := from_two_integer_general_number(num, denominator)
		end

	add_with_fraction_with_big_integer_number (other: FRACTION_WITH_BIG_INTEGER_NUMBER): NUMBER is
		local
			new_num, new_den: INTEGER_GENERAL_NUMBER
		do
			new_den ?= denominator * other.denominator
			if is_negative then
				if other.is_negative then
					new_num ?= - numerator * other.denominator + - other.numerator * denominator
				else
					new_num ?= - numerator * other.denominator + other.numerator * denominator
				end
			else
				if other.is_negative then
					new_num ?= numerator * other.denominator + - other.numerator * denominator
				else
					new_num ?= numerator * other.denominator + other.numerator * denominator
				end
			end
			check
				new_num /= Void
				new_den /= Void
			end
			Result := from_two_integer_general_number(new_num, new_den)
		end

	multiply_with_big_integer_number (other: BIG_INTEGER_NUMBER): NUMBER is
		local
			num: INTEGER_GENERAL_NUMBER
		do
			num ?= numerator.multiply_with_big_integer_number(other)
			check
				num /= Void
			end
			if is_negative then
				num ?= - num
			end
			Result := from_two_integer_general_number(num, denominator)
		end

	multiply_with_fraction_with_big_integer_number (other: FRACTION_WITH_BIG_INTEGER_NUMBER): NUMBER is
		local
			prod1, prod2: INTEGER_GENERAL_NUMBER
		do
			if is_negative = other.is_negative then
				prod1 ?= numerator * other.numerator
			else
				prod1 ?= - numerator * other.numerator
			end
			prod2 ?= denominator * other.denominator
			check
				prod1 /= Void
				prod2 /= Void
			end
			Result := from_two_integer_general_number(prod1, prod2)
		end

feature {ANY}
	infix "@*" (other: INTEGER_64): NUMBER is
			-- *** Can be very faster !! (Vincent Croizier, 04/07/04) ***
		local
			tmp: INTEGER_GENERAL_NUMBER
		do
			if is_negative then
				tmp ?= - numerator @* other
				check
					tmp /= Void
				end
				Result := from_two_integer_general_number(tmp, denominator)
			else
				tmp ?= numerator @* other
				check
					tmp /= Void
				end
				Result := from_two_integer_general_number(tmp, denominator)
			end
		end

	infix "@/" (other: INTEGER_64): NUMBER is
			-- *** Can be very faster !! (Vincent Croizier, 04/07/04) ***
		do
			if other = 1 then
				Result := Current
			else
				Result := Current * other.to_number.inverse
			end
		end

	infix "@<" (other: INTEGER_64): BOOLEAN is
		do
			if is_negative then
				if other < 0 then
					Result := - numerator < denominator @* other
				else
					Result := True
				end
			else
				if other > 0 then
					Result := numerator < denominator @* other
				end
			end
		end

	infix "@>" (other: INTEGER_64): BOOLEAN is
		do
			Result := not (Current @< other)
		end

	infix "@<=" (other: INTEGER_64): BOOLEAN is
		do
			Result := Current @< other
		end

	infix "@>=" (other: INTEGER_64): BOOLEAN is
		do
			Result := not (Current @< other)
		end

	infix "<" (other: NUMBER): BOOLEAN is
		do
			Result := other.greater_with_fraction_with_big_integer_number(Current)
		end

	infix "#=" (other: DOUBLE): BOOLEAN is
		do
			if is_negative then
				if other >= 0 then
				elseif Current < min_double then
				else
					Result := to_double = other
				end
			elseif other < 0 then
			elseif Current > max_double then
			else
				Result := to_double = other
			end
		end

	infix "#<" (other: DOUBLE): BOOLEAN is
		do
			if is_negative then
				if other >= 0 then
					Result := True
				elseif Current < min_double then
					Result := True
				else
					Result := to_double < other
				end
			elseif other < 0 then
			elseif Current > max_double then
			else
				Result := to_double < other
			end
		end

	infix "#<=" (other: DOUBLE): BOOLEAN is
		do
			if is_negative then
				if other >= 0 then
					Result := True
				elseif Current <= - max_double then
					Result := True
				else
					Result := to_double <= other
				end
			elseif other <= 0 then
			elseif Current >= max_double then
			else
				Result := to_double <= other
			end
		end

	infix "#>=" (other: DOUBLE): BOOLEAN is
		do
			if is_negative then
				if other >= 0 then
				elseif Current <= min_double then
				else
					Result := to_double >= other
				end
			elseif other <= 0 then
				Result := True
			elseif Current >= max_double then
				Result := True
			else
				Result := to_double >= other
			end
		end

	infix "#>" (other: DOUBLE): BOOLEAN is
		do
			if is_negative then
				if other >= 0 then
				elseif Current < min_double then
				else
					Result := to_double > other
				end
			elseif other < 0 then
				Result := True
			elseif Current > max_double then
				Result := True
			else
				Result := to_double > other
			end
		end

feature {NUMBER, NUMBER_TOOLS}
	greater_with_big_integer_number (other: BIG_INTEGER_NUMBER): BOOLEAN is
		do
			if is_negative then
				Result := denominator.multiply_with_big_integer_number(other) < - numerator
			else
				Result := denominator.multiply_with_big_integer_number(other) < numerator
			end
		end

	greater_with_fraction_with_big_integer_number (other: FRACTION_WITH_BIG_INTEGER_NUMBER): BOOLEAN is
		do
			if is_negative and then other.is_negative then
				Result := numerator * other.denominator < other.numerator * denominator
			elseif is_negative then
			elseif other.is_negative then
				Result := True
			else
				Result := other.numerator * denominator < numerator * other.denominator
			end
		end

invariant
	denominator @>= 2

	numerator @> 0

	numerator.gcd(denominator) @= 1

end -- class FRACTION_WITH_BIG_INTEGER_NUMBER
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is free software, which comes along with SmartEiffel. This software is distributed in the hope that it will be
-- useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- You can modify it as you want, provided this footer is kept unaltered, and a notification of the changes is added.
-- You are allowed to redistribute it and sell it, alone or as a part of another product.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
