-- See the Copyright notice at the end of this file.
--
class INTEGER_64_NUMBER
	--
	-- To implement NUMBER (do not use this class, see NUMBER).
	--
	-- This is an INTEGER_64

inherit
	INTEGER_GENERAL_NUMBER

creation {ANY}
	make

feature {ANY}
	is_zero: BOOLEAN is
		do
			Result := value = 0
		end

	is_one: BOOLEAN is
		do
			Result := value = 1
		end

	is_positive: BOOLEAN is
		do
			Result := value >= 0
		end

	is_negative: BOOLEAN is
		do
			Result := value < 0
		end

	is_integer_value: BOOLEAN is
		do
			Result := True
		end

	to_double: DOUBLE is
		do
			Result := value
		end

	append_in (str: STRING) is
		do
			str.append(value.to_string)
		end

	prefix "-": NUMBER is
		do
			if value = Minimum_integer_64 then
				mutable_register1.from_integer_64(Minimum_integer_64)
				mutable_register1.negate
				Result := mutable_register1.to_integer_general_number
			else
				create {INTEGER_64_NUMBER} Result.make(- value)
			end
		end

	infix "+" (other: NUMBER): NUMBER is
		do
			Result := other @+ value
		end

	infix "@+" (other: INTEGER_64): NUMBER is
		local
			sum: INTEGER_64
		do
			sum := value #+ other
			if value > 0 = (other > 0) implies sum > 0 = (value > 0) then
				-- no overflow
				create {INTEGER_64_NUMBER} Result.make(sum)
			else
				mutable_register1.from_integer_64(value)
				mutable_register1.add_integer_64(other)
				Result := mutable_register1.to_integer_general_number
			end
		end

	infix "*" (other: NUMBER): NUMBER is
		do
			if is_zero then
				Result := zero
			else
				Result := other @* value
			end
		end

	infix "@*" (other: INTEGER_64): NUMBER is
		do
			if other = 0 or else value = 0 then
				create {INTEGER_64_NUMBER} Result.make(0)
			elseif other = 1 then
				Result := Current
			elseif value = 1 then
				create {INTEGER_64_NUMBER} Result.make(other)
			else
				mutable_register1.from_integer_64(value)
				mutable_register2.from_integer_64(other)
				mutable_register1.multiply_to(mutable_register2, mutable_register3)
				Result := mutable_register3.to_integer_general_number
			end
		end

	infix "@/" (other: INTEGER_64): NUMBER is
		local
			other_number: INTEGER_64_NUMBER
		do
			if other = 1 then
				Result := Current
			elseif other = -1 then
				if value = Minimum_integer_64 then
					mutable_register1.from_integer_64(value)
					mutable_register1.negate
				else
					create {INTEGER_64_NUMBER} Result.make(- value)
				end
			elseif value #\\ other = 0 then
				create {INTEGER_64_NUMBER} Result.make(value #// other)
			else
				create other_number.make(other)
				create {FRACTION_WITH_BIG_INTEGER_NUMBER} Result.make(Current.abs, other_number.abs, is_negative = (other > 0))
			end
		end

	infix "//" (other: NUMBER): NUMBER is
		local
			oth: INTEGER_GENERAL_NUMBER
		do
			oth ?= other
			check
				oth /= Void
			end
			Result := oth.integer_divide_integer_64_number(Current)
		end

	infix "@//" (other: INTEGER_64): NUMBER is
		do
			-- *** Must be rewrited directly with integer_64 (Vincent Croizier, 04/07/04) ***
			put_into_mutable_big_integer(mutable_register1)
			mutable_register2.from_integer_64(other)
			mutable_register1.divide_to(mutable_register2, mutable_register3, mutable_register4)
			Result := mutable_register3.to_integer_general_number
		end

	infix "\\" (other: NUMBER): NUMBER is
		local
			oth: INTEGER_GENERAL_NUMBER
		do
			oth ?= other
			Result := oth.remainder_of_divide_integer_64_number(Current)
		end

	infix "@\\" (other: INTEGER_64): NUMBER is
		do
			-- *** Must be rewrited directly with integer_64 (Vincent Croizier, 04/07/04) ***
			put_into_mutable_big_integer(mutable_register1)
			mutable_register2.from_integer_64(other)
			mutable_register1.divide_to(mutable_register2, mutable_register3, mutable_register4)
			Result := mutable_register4.to_integer_general_number
		end

feature {ANY} -- Misc:
	hash_code: INTEGER is
		do
			Result := value.hash_code
		end

	gcd (other: NUMBER): INTEGER_GENERAL_NUMBER is
		do
			Result := other.gcd_with_integer_64_number(Current)
		end

	infix "@=" (other: INTEGER_64): BOOLEAN is
		do
			Result := value = other
		end

	infix "@<" (other: INTEGER_64): BOOLEAN is
		do
			Result := value < other
		end

	infix "@<=" (other: INTEGER_64): BOOLEAN is
		do
			Result := value <= other
		end

	infix "@>" (other: INTEGER_64): BOOLEAN is
		do
			Result := value > other
		end

	infix "@>=" (other: INTEGER_64): BOOLEAN is
		do
			Result := value >= other
		end

	infix "#=" (other: DOUBLE): BOOLEAN is
		do
			Result := other = value
		end

	infix "#<" (other: DOUBLE): BOOLEAN is
		do
			Result := value < other
		end

	infix "#<=" (other: DOUBLE): BOOLEAN is
		do
			Result := value <= other
		end

	infix "#>" (other: DOUBLE): BOOLEAN is
		do
			Result := value > other
		end

	infix "#>=" (other: DOUBLE): BOOLEAN is
		do
			Result := value >= other
		end

	infix "<" (other: NUMBER): BOOLEAN is
		do
			Result := other @> value
		end

	is_equal (other: NUMBER): BOOLEAN is
		local
			n2: like Current
		do
			n2 ?= other
			if n2 /= Void then
				Result := value = n2.value
			end
		end

	inverse: NUMBER is
		do
			if is_one or else Current @= -1 then
				Result := Current
			else
				create {FRACTION_WITH_BIG_INTEGER_NUMBER} Result.make_simply(integer_general_number_one, Current.abs, is_negative)
			end
		end

feature {NUMBER} -- Implementation:
	value: INTEGER_64

	add_with_big_integer_number (other: BIG_INTEGER_NUMBER): NUMBER is
		do
			Result := other @+ value
		end

	add_with_fraction_with_big_integer_number (other: FRACTION_WITH_BIG_INTEGER_NUMBER): NUMBER is
		do
			Result := other @+ value
		end

	multiply_with_big_integer_number (other: BIG_INTEGER_NUMBER): NUMBER is
		do
			Result := other @* value
		end

	multiply_with_fraction_with_big_integer_number (other: FRACTION_WITH_BIG_INTEGER_NUMBER): NUMBER is
		do
			Result := other.multiply_with_integer_64_number(Current)
		end

	integer_divide_integer_64_number (other: INTEGER_64_NUMBER): INTEGER_GENERAL_NUMBER is
		do
			Result ?= other @// value
		end

	integer_divide_big_integer_number (other: BIG_INTEGER_NUMBER): INTEGER_GENERAL_NUMBER is
		do
			Result ?= other @// value
		end

	remainder_of_divide_integer_64_number (other: INTEGER_64_NUMBER): INTEGER_GENERAL_NUMBER is
		do
			Result ?= other @\\ value
		end

	remainder_of_divide_big_integer_number (other: BIG_INTEGER_NUMBER): INTEGER_GENERAL_NUMBER is
		do
			Result ?= other @\\ value
		end

	greater_with_big_integer_number (other: BIG_INTEGER_NUMBER): BOOLEAN is
		do
			Result := other.is_negative
		end

	greater_with_fraction_with_big_integer_number (other: FRACTION_WITH_BIG_INTEGER_NUMBER): BOOLEAN is
		do
			if other.is_negative then
				Result := other.denominator * Current > - other.numerator
			else
				Result := other.denominator * Current > other.numerator
			end
		end

feature {NUMBER} -- Implementation:
	gcd_with_integer_64_number (other: INTEGER_64_NUMBER) : INTEGER_64_NUMBER is
		do
			create Result.make(value.gcd(other.value))
		end

feature {NUMBER}
	put_into_mutable_big_integer (mut: MUTABLE_BIG_INTEGER) is
		do
			mut.from_integer_64(value)
		end

feature {}
	make (val: INTEGER_64) is
		do
			value := val
		ensure
			Current.to_integer_64 = val
		end

end -- class INTEGER_64_NUMBER
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is free software, which comes along with SmartEiffel. This software is distributed in the hope that it will be
-- useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- You can modify it as you want, provided this footer is kept unaltered, and a notification of the changes is added.
-- You are allowed to redistribute it and sell it, alone or as a part of another product.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
