-- See the Copyright notice at the end of this file.
--
deferred class AVL_HELPER[E -> COMPARABLE]

feature {} -- Interface used by descendants:
	new_node (e: E): like a_new_node is
		require
			e /= Void
		do
			if lost_nodes = Void or else lost_nodes.is_empty then
				Result := a_new_node(e)
			else
				Result := lost_nodes.last
				lost_nodes.remove_last
				Result.set_item(e)
			end
		ensure
		--old lost_nodes.count > 0 implies
		--   (lost_nodes.count = old lost_nodes.count - 1
		--    and then Result = old lost_nodes.last)
		end

	discard_node (n: like a_new_node) is
		require
			n /= Void
		do
			if lost_nodes /= Void then
            lost_nodes.add_last(n)
         end
		ensure
			discarded: lost_nodes /= Void implies (lost_nodes.count = old lost_nodes.count + 1)
		end

feature {} -- Internals:
	lost_nodes: COLLECTION[AVL_NODE[E]] is
			-- Keeps discarded nodes to avoid memory leaks. Should be a once function. May return Void if lost
			-- nodes are not kept.
		deferred
		end

	a_new_node (e: E): AVL_NODE[E] is
		require
			e /= Void
		deferred
		ensure
			Result /= Void
			Result.item = e -- to put back in place when the new lookup is OK
		end

end -- class AVL_HELPER
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is free software, which comes along with SmartEiffel. This software is distributed in the hope that it will be
-- useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- You can modify it as you want, provided this footer is kept unaltered, and a notification of the changes is added.
-- You are allowed to redistribute it and sell it, alone or as a part of another product.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
