-- See the Copyright notice at the end of this file.
--
class AVL_SET_NODE[E -> COMPARABLE]
	--
	-- Auxiliary class to implement AVL_SET[E].
	--
	-- This a classic implementation of an AVL tree (balanced tree first
	-- designed by Adelson-Velskii and Landis, 1960)
	--

inherit
	AVL_NODE[E]

insert
	AVL_HELPER[E]
		undefine out_in_tagged_out_memory
		end

creation {AVL_HELPER}
	make

feature {AVL_HELPER}
	has (e: like item): BOOLEAN is
			-- Is element `e' in the tree?
		local
			safe_equal: SAFE_EQUAL[E]
		do
			Result := safe_equal.test(item, e)
			if not Result then
				if e < item then
					Result := left /= Void and then left.has(e)
				else
					Result := right /= Void and then right.has(e)
				end
			end
		end

	fast_has (e: like item): BOOLEAN is
			-- Is element `e' in the tree?
		local
			safe_equal: SAFE_EQUAL[E]
		do
			Result := item = e
			if not Result and then not safe_equal.test(item, e) then
				if e < item then
					Result := left /= Void and then left.fast_has(e)
				else
					Result := right /= Void and then right.fast_has(e)
				end
			end
		ensure
			Result implies count > 0
		end

	at (e: like item): AVL_SET_NODE[E] is
			-- Is element `e' in the tree?
		local
			safe_equal: SAFE_EQUAL[E]
		do
			if safe_equal.test(item, e) then
				Result := Current
			elseif e < item then
				if left /= Void then
					Result := left.at(e)
				end
			else
				if right /= Void then
					Result := right.at(e)
				end
			end
		end

feature {}
	node_memo: MEMO[AVL_SET_NODE[E]] is
      once
         create Result
		end

	inserted_node_memo: MEMO[AVL_SET_NODE[E]] is
      once
         create Result
		end

	lost_nodes: COLLECTION[AVL_SET_NODE[E]] is
      local
         m: MEMORY
		once
         if not m.smart_eiffel_collector then
            create {FAST_ARRAY[AVL_SET_NODE[E]]} Result.make(0)
         end
		end

	a_new_node (e: E): AVL_SET_NODE[E] is
		do
			create Result.make(e)
		end

	last_inserted_memory: MEMO[like Current] is
		once
			create Result
		end

end -- class AVL_SET_NODE
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- This file is free software, which comes along with SmartEiffel. This software is distributed in the hope that it will be
-- useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- You can modify it as you want, provided this footer is kept unaltered, and a notification of the changes is added.
-- You are allowed to redistribute it and sell it, alone or as a part of another product.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
