-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class AGENT_POOL
	--
	-- Singleton object in charge of agents.
	-- This singleton is shared via the GLOBALS.`agent_pool' once function.
	--

inherit
	ANY
insert
	GLOBALS

feature {AGENT_LAUNCHER}
	agent_creation_collected_flag: BOOLEAN
	
feature {AGENT_CREATION}
	register_for_c_define(mold_id: STRING; agent_creation: AGENT_CREATION) is
		require
			smart_eiffel.status.collecting_done
		do
			if not defined_agent_creation.has(mold_id) then
				defined_agent_creation.add_last(mold_id.twin)
				pending_agent_creation.add_last(agent_creation)
			end
		end
	
	mold_id_index_of(mold_id: STRING): INTEGER is
		require
			smart_eiffel.status.collecting_done
		do
			Result := defined_agent_creation.index_of(mold_id)
		ensure
			defined_agent_creation.valid_index(Result)
		end
	
	agent_creation_collect (agent_creation: AGENT_CREATION; agent_type: TYPE) is
		require
			agent_type.is_agent
		local
			i: INTEGER; agent_args: AGENT_ARGS; launcher_type: TYPE; model: AGENT_CREATION
		do
			agent_creation_collected_flag := True
			model := creation_collected_memory.reference_at(agent_creation)
			if model = Void then
				creation_collected_memory.add(agent_creation)
				smart_eiffel.magic_count_increment
			end
			-- Visit all already collected compatible launcher to update the
			-- assignment_handler accordingly:
			from
				i := launcher_collected_memory.lower
			until
				i > launcher_collected_memory.upper
			loop
				agent_args := launcher_collected_memory.item(i)
				launcher_type := agent_args.agent_type
				if agent_type /= launcher_type and then agent_type.is_a_test(launcher_type) then
					notify_assignment_handler_with(agent_type, launcher_type)
				end
				i := i + 1
			end
		end

	open_close_mold_id_in (buffer: STRING; open_flag: BOOLEAN; type: TYPE) is
			-- Unique place for the normalized id strings.
		do
			-- Do _NOT_ change the following mangling because it is used by `mold_id_is_a' in AGENT_POOL.
			buffer.extend('_')
			if open_flag then
				buffer.append(once "Open")
			else
				buffer.append(once "Clos")
			end
			if type.is_reference then
				buffer.append(once "T0")
			else
				buffer.extend('T')
				type.live_type.id.append_in(buffer)
			end
		end
	
	mold_id_is_a (launcher_id, mold_id, result_id: STRING; function_flag: BOOLEAN): BOOLEAN is
		require
			not launcher_id.is_empty
			not mold_id.is_empty
		local
			i, launcher_idx, id1, id2: INTEGER
		do
			Result := True
			result_id.copy(mold_id)
			-- Getting in `result_id' only the useful part of `mold_id':
			from
				i := result_id.count
				i := result_id.reverse_index_of('_', i)
			until
				result_id.item(i - 1) = '_'
			loop
				i := result_id.reverse_index_of('_', i - 1)
			end
			inspect
				result_id.item(i - 2)
			when 'P' then
				-- A procedure cannot match with a function.
				Result := not function_flag
			when 'F' then
				-- The function Result can be dropped when unused.
			end
			if Result then
				result_id.remove_first(i - 1)
				-- Now performing lazy is_a:
				from
					i := 2
					launcher_idx := 2
				until
					not Result or else i > result_id.count or else launcher_idx > launcher_id.count
				loop
					if result_id.item(i) = launcher_id.item(launcher_idx) then
						i := i + 1
						launcher_idx := launcher_idx + 1
					elseif result_id.item(i) = 'C' then
						from
						until
							i > result_id.count or else result_id.item(i) = '_'
						loop
							result_id.remove(i)
						end
						result_id.remove(i)
					else
						check
							result_id.item(i).is_digit
							launcher_id.item(launcher_idx).is_digit
						end
						from
							id1 := 0
						until
							launcher_idx > launcher_id.count or else not launcher_id.item(launcher_idx).is_digit
						loop
							id1 := id1 * 10 + launcher_id.item(launcher_idx).decimal_value
							launcher_idx := launcher_idx + 1
						end
						from
							id2 := 0
						until
							i > result_id.count or else not result_id.item(i).is_digit
						loop
							id2 := id2 * 10 + result_id.item(i).decimal_value
							i := i + 1
						end
						-- *** Beuurk Warning Hard code ahead *** (Dom. sept 25th 2004) ***
						inspect
							id1
						when 1, 2, 4, 5, 10, 11 then
							inspect
								id2
							when 1, 2, 4, 5, 10, 11 then
								-- INTEGER_* and REAL_* *** Hard code *** (Dom. sept 25th 2004) ***
							else
								Result := False
							end
						else
							Result := False
						end
					end
				end
				if Result and then i < result_id.count then
					from
					until
						not Result or else i > result_id.count
					loop
						inspect
							result_id.item(i)
						when '_' then
							result_id.remove(i)
						when 'C' then
							from
							until
								i > result_id.count or else result_id.item(i) = '_'
							loop
								result_id.remove(i)
							end
						when 'O' then
							-- (Cannot launch with missing Open.)
							Result := False
						end
					end
				end
			end
		end
	
feature {AGENT_LAUNCHER}
	agent_launcher_collect (launcher_type: TYPE): AGENT_ARGS is
		require
			launcher_type.canonical_type_mark.is_agent
		local
			i: INTEGER; signature: STRING; new_flag: BOOLEAN; formal, ar, agent_type: TYPE; open: ARRAY[TYPE]
			unknown_position: POSITION
		do
			-- Computation of the `signature' first:
			signature := once "..... unique buffer ....."
			signature.clear_count
			ar := launcher_type.agent_result
			if ar /= Void then
				signature_add_last(signature, ar)
				signature.extend('_')
			end
			signature.append(once "agent_launcher")
			open := launcher_type.open_arguments
			if open /= Void then
				from
					i := 1
				until
					i > open.count
				loop
					formal := open.item(i)
					signature.extend('_')
					signature_add_last(signature, formal)
					i := i + 1
				end
			end
			Result := launcher_memory.reference_at(signature)
			if Result = Void then
				new_flag := True
				signature := signature.twin
				create Result.make(signature, launcher_type)
				launcher_memory.put(Result, signature)
				launcher_collected_memory.add(Result)
			elseif not launcher_collected_memory.has(Result) then
				new_flag := True
				launcher_collected_memory.add(Result)
			end
			if new_flag then
				-- To notify the `assignment_handler':
				from
					i := creation_collected_memory.lower
				until
					i > creation_collected_memory.upper
				loop
					agent_type := creation_collected_memory.item(i).resolve_in_memory
					if agent_type /= launcher_type and then agent_type.is_a_test(launcher_type) then
						notify_assignment_handler_with(agent_type, launcher_type)
						-- *** Why is it necessary to do this ?	-- *** (Dom. sept 25th 2004) ***
					end
					i := i + 1
				end
			end
		ensure
			Result /= Void
		end

feature {LIVE_TYPE}
	c_define_pending_agent_creation(live_type: LIVE_TYPE) is
		local
			i: INTEGER
		do
			from
				i := pending_agent_creation.upper
			until
				i < pending_agent_creation.lower
			loop
				pending_agent_creation.item(i).c_define_function(live_type.type)
				i := i - 1
			end
			pending_agent_creation.clear_count
		end
	
feature {SMART_EIFFEL}	
	reset is
		do
			agent_creation_collected_flag := False
			creation_collected_memory.clear_count
			launcher_collected_memory.clear_count
		end

	c_header_pass1 is
		require
			cpp.on_h
		do
			if not creation_collected_memory.is_empty then
				cpp.put_string(once "[
               typedef union _se_agent se_agent;
               typedef struct _se_agent0 se_agent0;

                                 ]")
			end
		ensure
			cpp.on_h
		end

	customize_jvm_runtime is
		do
			-- *** ???? ***
		end

	no_pending_agent_creation: BOOLEAN is
		do
			Result := pending_agent_creation.is_empty
		ensure
			Result
		end

feature {JVM}
	jvm_define_deferred_methods is
		local
			i: INTEGER
		do
			method_info.add_init(fz_java_lang_object)
			from
				i := 1
			until
				i > launcher_collected_memory.count
			loop
				launcher_collected_memory.item(i).jvm_define(False)
				i := i + 1
			end
			method_info.finish
		end

	ajout_call_full (i: INTEGER) is
			-- *** NOM MOCHE ***
		do
			method_info.add_init(fz_java_lang_object)
			launcher_collected_memory.item(i).jvm_define(True)
			method_info.finish
		end

feature {C_PRETTY_PRINTER}
	customize_c_runtime_1 is
		require
			cpp.on_h
		local
			boost: BOOLEAN
		do
			boost := ace.boost
			if agent_creation_collected_flag then
				cpp.put_string(once "[
               /*The generic se_agent0 definition:*/
               struct _se_agent0{
               Tid id;
               Tid creation_mold_id;
               void(*afp)(
				 ]")
				if not boost then
					cpp.put_string(once "se_dump_stack*,")
				end
				cpp.put_string(once "[
				 se_agent*);
				 void(*gc_mark_agent_mold)(se_agent*);
				 };

				 ]")
				cpp.swap_on_h
			end
		ensure
			cpp.on_h
		end

feature {SMART_EIFFEL}
	customize_c_runtime_2 is
		require
			cpp.on_c
		local
			i: INTEGER; mold_id: STRING; agent_args: AGENT_ARGS
		do
			if agent_creation_collected_flag then
				echo.print_count(once "Agent call wrapper", launcher_collected_memory.count)
				from
					i := 1
				until
					i > launcher_collected_memory.count
				loop
					agent_args := launcher_collected_memory.item(i)
					agent_definition_set.add(agent_args.signature)
					agent_args.c_define_agent_launcher
					i := i + 1
				end

				cpp.swap_on_h
				cpp.put_string(once "union _se_agent{T0 s0;se_agent0 u0;%N")
				from
					i := defined_agent_creation.upper
				until
					i < defined_agent_creation.lower
				loop
					mold_id := defined_agent_creation.item(i)
					cpp.put_string(once "se_")
					cpp.put_string(mold_id)
					cpp.put_string(once " u")
					cpp.put_string(mold_id)
					cpp.put_string(once ";%N")
					i := i - 1
				end
				cpp.put_string(once "};%N")
				cpp.swap_on_c

				cpp.sys_runtime_h_and_c(once "agents")
			end
		ensure
			cpp.on_c			
		end

feature {GC_HANDLER}
	gc_info_in (body: STRING) is
			-- Produce C code to print GC information.
		do
			if not creation_collected_memory.is_empty then
				body.append(once "[
               if(gc_info_nb_agent)
                  fprintf(SE_GCINFO,
                  "%d\tagent(s) created. (store_left=%d).\n",
                  gc_info_nb_agent,store_left_agent);

                              ]")
			end
		end

feature {AGENT_ARGS}
	c_switch_in (buffer: STRING; launcher_type, agent_result: TYPE) is
		require
			agent_result = launcher_type.agent_result
		local
			launcher_id, mold_id, open_id: STRING; i, argi, open_count: INTEGER; launcher_open: ARRAY[TYPE]
		do
			open_id := once "......................"
			launcher_open := launcher_type.open_arguments
			-- First, compute `launcher_id':
			launcher_id := once "...................."
			launcher_id.clear_count
			if launcher_open /= Void then
				from
					i := launcher_open.lower
				until
					i > launcher_open.upper
				loop
					open_close_mold_id_in(launcher_id, True, launcher_open.item(i))
					i := i + 1
				end
			end
			from
				i := defined_agent_creation.upper
			until
				i < defined_agent_creation.lower
			loop
				mold_id := defined_agent_creation.item(i)
				if mold_id_is_a(launcher_id, mold_id, open_id, agent_result /= Void) then
					buffer.append(once "case ")
					i.append_in(buffer)
					buffer.append(once ":{%N")
					if agent_result /= Void then
						buffer.append(once "return(")
						agent_result.canonical_type_mark.c_type_for_result_in(buffer)
						buffer.append(once ")(")
					end
					buffer.append(once "((se_")
					buffer.append(mold_id)
					buffer.append(once "*)a)->afp(")
					if not ace.boost then
						buffer.append(once "&ds,")
					end
					buffer.append(once "((/*agent*/void*)a)")
				   from
						argi := 1
						open_count := open_id.occurrences('T')
					until
						argi > open_count
					loop
						buffer.append(once ",a")
						argi.append_in(buffer)
						argi := argi + 1
					end
					buffer.extend(')')
					if agent_result /= Void then
						buffer.extend(')')
					else
						buffer.append(once ";%Nbreak")
					end
					buffer.append(once ";%N}%N")
				else
					buffer.extend('%N')
				end
				i := i - 1
			end
		end

feature {}
	notify_assignment_handler_with (agent_type, launcher_type: TYPE) is
		require
			agent_type.is_agent
			launcher_type.is_agent
			agent_type.is_a_test(launcher_type)
		local
			argi, open_agent_count, open_launcher_count: INTEGER
			agent_oa, launcher_oa, agent_result, launcher_result: TYPE; open_agent, open_launcher: ARRAY[TYPE]
			unknown_position: POSITION
		do
			open_agent := agent_type.open_arguments
			if open_agent /= Void then
				open_agent_count := open_agent.count
			end
			open_launcher := launcher_type.open_arguments
			if open_launcher /= Void then
				open_launcher_count := open_launcher.count
			end
			if open_agent_count >= open_launcher_count then
				-- (Because we allow to drop some actual args.)
				from
					argi := open_launcher_count
				until
					argi <= 0
				loop
					agent_oa := open_agent.item(argi)
					launcher_oa := open_launcher.item(argi)
					assignment_handler.collect(unknown_position, launcher_oa, agent_oa)
					argi := argi - 1
				end
			end
			-- Finally the results:
			agent_result := agent_type.agent_result
			launcher_result := launcher_type.agent_result
			if agent_result /= Void and then launcher_result /= Void then
				assignment_handler.collect(unknown_position, agent_result, launcher_result)
			end
		end

	creation_collected_memory: SET[AGENT_CREATION] is
			-- Those which are collected.
		once
			create {HASHED_SET[AGENT_CREATION]} Result.make
		end

	launcher_memory: HASHED_DICTIONARY[AGENT_ARGS, STRING] is
			-- All ever created signatures for all encountered agent launcher.
		once
			create Result.make
		end

	launcher_collected_memory: SET[AGENT_ARGS] is
			-- A subset of `launcher_memory' (collected ones).
		once
			create {HASHED_SET[AGENT_ARGS]} Result.make
		end

	agent_definition_set: SET[STRING] is
		once
			create {HASHED_SET[STRING]} Result.make
		end

	signature_add_last (signature: STRING; type: TYPE) is
		require
			type /= Void
		local
			s: STRING; i: INTEGER; c: CHARACTER
		do
			from
				s := type.name.to_string
				i := 1
			until
				i > s.count
			loop
				c := s.item(i)
				inspect
					c
				when '[', ']', ',', ' ' then
					signature.append(once "__")
					-- It is not allowed to have 2 _ in the same
					-- identifier in Eiffel, but in C.
				else
					signature.extend(c)
				end
				i := i + 1
			end
		end

	defined_agent_creation: FAST_ARRAY[STRING] is
		once
			create Result.with_capacity(32)
		end

	pending_agent_creation: FAST_ARRAY[AGENT_CREATION] is
		once
			create Result.with_capacity(32)
		end
	
	singleton_memory: AGENT_POOL is
		once
			Result := Current
		end

invariant
	is_real_singleton: Current = singleton_memory

end -- class AGENT_POOL
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
