-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class TYPE_BIT_1
	--
	-- Handling for example of "BIT 32", or "BIT 8", etc. type mark
	--

inherit
	TYPE_BIT
		redefine set_start_position
		end

insert
	STATIC_TYPE_MARK
		redefine set_start_position
		end

creation {ANY}
	make

feature {ANY}
	integer_constant: INTEGER_CONSTANT

	bit_count: INTEGER is
		do
			Result := integer_constant.value_memory.to_integer
		end

	class_text_name: CLASS_NAME is
		local
			fake_class_text: CLASS_TEXT
		do
			if class_text_name_memory = Void then
				create class_text_name_memory.make(written_name, start_position)
				fake_class_text := smart_eiffel.fake_bit_n_class_text(class_text_name_memory)
			end
			Result := class_text_name_memory
		end

	accept (visitor: TYPE_BIT_1_VISITOR) is
		do
			visitor.visit_type_bit_1(Current)
		end

feature {TYPE_MARK}
	set_start_position (sp: like start_position) is
		do
			start_position := sp
		end

	short_ (shorted_type: TYPE) is
		do
			short_printer.put_class_name_without_link(class_text_name)
		end

feature {}
	class_text_name_memory: CLASS_NAME

	make (sp: like start_position; ic: like integer_constant) is
		require
			not sp.is_unknown
			ic.value_memory > 0
		local
			buffer: STRING
		do
			start_position := sp
			integer_constant := ic
			buffer := once "....... once unique buffer ......."
			buffer.copy(as_bit)
			buffer.extend(' ')
			bit_count.append_in(buffer)
			written_name := string_aliaser.hashed_string(buffer)
		ensure
			start_position = sp
			integer_constant = ic
		end

invariant
	integer_constant /= Void

end -- class TYPE_BIT_1
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
