-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class TYPE_BIT_2
	--
	-- Handling for example of "BIT Foo" type mark
	--

inherit
	TYPE_BIT

creation {ANY}
	make

feature {ANY}
	is_static: BOOLEAN is False

	feature_name: FEATURE_NAME
			-- The one which is supposed to gives the number of bits.

	bit_count: INTEGER is
		do
			Result := resolved_mark.bit_count
		end

	declaration_type: TYPE_BIT is
		local
			type_like_current: TYPE
		do
			if declaration_type_memory = Void then
				type_like_current := start_position.class_text.declaration_type_of_like_current
				declaration_type_memory := compute_resolved_mark(type_like_current)
				if resolved_mark = Void then
					resolved_mark := declaration_type_memory
				end
			end
			Result := declaration_type_memory
		end

	constrained_type: TYPE_MARK is
		do
			Result := declaration_type
		end

	specialize_in (new_type: TYPE): like Current is
		local
			fn: like feature_name; rm: like resolved_mark
		do
			if resolved_mark = Void then
				resolved_mark := declaration_type_memory
			end
			fn := feature_name.specialize_in(new_type)
			rm := compute_resolved_mark(new_type)
			if resolved_mark = Void then
				resolved_mark := rm
			end
			Result := current_or_twin_init(fn, rm)
		end

	specialize_thru (parent_type: TYPE; inherit_item: INHERIT_TEXT_ITEM; new_type: TYPE): like Current is
		local
			fn: like feature_name; rm: like resolved_mark
		do
			check
				resolved_mark /= Void
			end
			fn := feature_name.specialize_thru(parent_type, inherit_item, new_type)
			rm := compute_resolved_mark(new_type)
			Result := current_or_twin_init(fn, rm)
		end

	has_been_specialized: BOOLEAN is
		do
			Result := feature_name.has_been_specialized and then resolved_mark /= Void
		end

	to_static (new_type: TYPE): TYPE_BIT is
		do
			Result := compute_resolved_mark(new_type)
		end

	signature_resolve_in (new_type: TYPE): TYPE is
		do
			Result := resolve_in(new_type)
		end

	accept (visitor: TYPE_BIT_2_VISITOR) is
		do
			visitor.visit_type_bit_2(Current)
		end

	class_text_name: CLASS_NAME is
		do
			Result := declaration_type.class_text_name
		end

feature {TYPE_BIT_2}
	resolved_mark: TYPE_BIT_1
			-- The latest resolved one (which is most often `declaration_type_memory').

	init (fn: like feature_name; rm: like resolved_mark) is
		require
			fn /= Void
			rm /= Void
		do
			feature_name := fn
			resolved_mark := rm
		ensure
			feature_name = fn
			resolved_mark = rm
		end

feature {TYPE_MARK}
	short_ (shorted_type: TYPE) is
		do
			resolved_mark.short_(shorted_type)
		end

feature {}
	compute_resolved_mark (new_type: TYPE): like resolved_mark is
			-- (This is not really optimized, but this TYPE_BIT_* stuff will be soon obsolete.)
		require
			new_type /= Void
		local
			fs: FEATURE_STAMP; af: ANONYMOUS_FEATURE; cst_att_integer: CST_ATT_INTEGER; bc: like bit_count
			ic: INTEGER_CONSTANT; tlc: TYPE; external_name: STRING; external_function: EXTERNAL_FUNCTION
			fn: like feature_name; sp: POSITION
		do
			-- Compute first the original FEATURE_STAMP:
			tlc := feature_name.start_position.class_text.declaration_type_of_like_current
			fs := tlc.search(feature_name)
			if fs = Void then
				error_handler.append("Feature ")
				error_handler.add_feature_name(feature_name)
				error_handler.append(" not found.")
				error_handler.print_as_fatal_error
			end
			-- Search now the good FEATURE_STAMP for `new_type':
			if tlc = new_type or else tlc.is_expanded -- *** Bug if this test is not here ??
				then
				fn := feature_name
			else
				fs := fs.resolve_dynamic_binding_for(new_type)
				if fs = Void then
					error_handler.append("Feature ")
					error_handler.add_feature_name(feature_name)
					error_handler.append(" not found in ")
					error_handler.append(new_type.name.to_string)
					error_handler.append(".")
					error_handler.print_as_fatal_error
				end
				-- Now computing the possible new name here:
				fn := new_type.get_feature_name(fs)
			end
			check
				fn /= Void
			end
			af := new_type.find_anonymous_feature_for(fs, fn)
			cst_att_integer ?= af
			if cst_att_integer = Void then
				external_function ?= af
				if external_function = Void then
					error_handler.add_position(feature_name.start_position)
					error_handler.add_position(af.start_position)
					error_handler.append(fz_iinaiv)
					error_handler.print_as_fatal_error
				else
					external_name := external_function.first_name.to_string
					if as_integer_bits = external_name then
						bc := Integer_bits
					elseif as_character_bits = external_name then
						bc := Character_bits
					end
				end
			else
				bc := cst_att_integer.value_memory.value_memory.to_integer
			end
			if bc <= 0 then
				error_handler.add_position(feature_name.start_position)
				error_handler.add_position(cst_att_integer.start_position)
				error_handler.append("Constant found is not a positive INTEGER. (Wrong constant for BIT definition.)")
				error_handler.print_as_fatal_error
			end
			-- Now, we finish the work:
			if resolved_mark /= Void and then resolved_mark.bit_count = bc then
				Result := resolved_mark -- is recycled.
			else
				sp := feature_name.start_position
				create ic.make(bc, sp, create {TYPE_INTEGER}.integer_32(sp))
				create Result.make(start_position, ic)
			end
		ensure
			Result /= Void
		end

	declaration_type_memory: TYPE_BIT_1
			-- To cache `declaration_type'.

	current_or_twin_init (fn: like feature_name; rt: like resolved_mark): like Current is
		require
			fn /= Void
		do
			if fn = feature_name then
				Result := Current
			else
				Result := twin
				Result.init(fn, rt)
			end
		ensure
			Result.feature_name = fn
			Result.resolved_mark = rt
		end

	make (sp: like start_position; fn: like feature_name) is
		require
			not sp.is_unknown
			fn /= Void
		local
			buffer: STRING
		do
			buffer := once "....... once unique buffer ......."
			buffer.copy(as_bit)
			buffer.extend(' ')
			buffer.append(fn.to_string)
			written_name := string_aliaser.hashed_string(buffer)
			start_position := sp
			feature_name := fn
		ensure
			start_position = sp
			feature_name = fn
		end

invariant
	feature_name /= Void

end -- class TYPE_BIT_2
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
