-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class TYPE_CLASS

inherit
	STATIC_TYPE_MARK

creation {ANY}
	make

feature {ANY}
	class_text_name: CLASS_NAME

	is_generic: BOOLEAN is False

	jvm_method_flags: INTEGER is 17

	written_name: HASHED_STRING is
		do
			Result := class_text_name.hashed_name
		end

	actual_reference (destination: TYPE_MARK): TYPE_MARK is
		local
			sp: POSITION
		do
			sp := start_position
			if sp.is_unknown then
				if destination /= Void then
					sp := destination.start_position
				end
			end
			create {TYPE_REFERENCE} Result.make(sp, Current)
		end

	actual_separate (destination: TYPE_MARK): TYPE_MARK is
		local
			sp: POSITION
		do
			sp := destination.start_position
			Result := separate_tools.create_type_separate(sp, Current)
		end

	is_expanded: BOOLEAN is
		do
			Result := class_text.is_expanded
		end

	is_separate: BOOLEAN is
		do
			Result := class_text.is_separate
		end

	is_reference: BOOLEAN is
		do
			Result := not class_text.is_expanded
		end

	is_user_expanded: BOOLEAN is
		do
			Result := is_expanded
		end

	is_dummy_expanded: BOOLEAN is
		do
			if is_expanded then
				Result := live_type.writable_attributes = Void
			end
		end

	generic_list: ARRAY[TYPE_MARK] is
		do
			crash -- *** TO BE REMODED IN A FEW DAYS (Dom. july 19th 2004) ***
		   check
		      False -- Because of the inherited require.
		   end
		end

	local_from_separate: TYPE_MARK is
		do
			not_yet_implemented
		end

	id: INTEGER is
		do
			Result := class_text.id
		end

	c_sizeof: INTEGER is
		do
			if is_reference then
				Result := c_sizeof_pointer
			else
				Result := live_type.c_sizeof
			end
		end

	smallest_ancestor (other: TYPE_MARK): TYPE_MARK is
		local
			bc, bc2: CLASS_TEXT
		do
			if other.is_none then
				Result := Current
			elseif other.is_any then
				Result := other
			else
				bc := class_text
				bc2 := other.class_text
				if bc2 = bc then
					Result := Current
				elseif bc2.is_subclass_of(bc) then
					Result := Current
				elseif bc.is_subclass_of(bc2) then
					Result := other
				elseif other.is_expanded and then not is_expanded then
					Result := other.smallest_ancestor(Current)
				else
					Result := bc2.smallest_ancestor(other, Current)
				end
			end
		end

	jvm_descriptor_in (str: STRING) is
		do
			if is_reference then
				str.append(jvm_root_descriptor)
			else
				live_type.jvm_type_descriptor_in(str)
			end
		end

	jvm_target_descriptor_in (str: STRING) is
		do
		end

	jvm_return_code is
		do
			code_attribute.opcode_areturn
		end

	jvm_check_class_invariant is
		do
			standard_jvm_check_class_invariant
		end

	jvm_push_local (offset: INTEGER) is
		do
			code_attribute.opcode_aload(offset)
		end

	jvm_push_default: INTEGER is
		do
			Result := 1
			if is_reference then
				code_attribute.opcode_aconst_null
			else
				live_type.jvm_expanded_push_default
			end
		end

	jvm_write_local_creation (offset: INTEGER) is
		do
			code_attribute.opcode_astore(offset)
		end

	jvm_write_local (offset: INTEGER) is
		do
			if not is_expanded then
				code_attribute.opcode_astore(offset)
			else
				jvm_write_local_expanded(offset)
			end
		end

	jvm_write_local_expanded (offset: INTEGER) is
		local
			rc: LIVE_TYPE; wa: ARRAY[RUN_FEATURE_2]
		do
			rc := live_type
			wa := rc.writable_attributes
			code_attribute.opcode_aload(offset)
			code_attribute.opcode_swap
			jvm.fields_by_fields_expanded_copy(wa)
			code_attribute.opcode_pop2
		end

	jvm_xnewarray is
		local
			idx: INTEGER
		do
			if is_reference then
				idx := constant_pool.idx_jvm_root_class
			else
				check
					is_user_expanded
				end
				idx := live_type.jvm_constant_pool_index
			end
			code_attribute.opcode_anewarray(idx)
		end

	jvm_xastore is
		do
			code_attribute.opcode_aastore
		end

	jvm_xaload is
		do
			code_attribute.opcode_aaload
		end

	jvm_if_x_eq: INTEGER is
		do
			Result := code_attribute.opcode_if_acmpeq
		end

	jvm_if_x_ne: INTEGER is
		do
			Result := code_attribute.opcode_if_acmpne
		end

	jvm_to_reference is
		do
			if is_expanded then
				live_type.jvm_to_reference
			end
		end

	jvm_expanded_from_reference (other: TYPE_MARK): INTEGER is
		do
			check
				False
			end
		end

	jvm_convert_to (destination: TYPE_MARK): INTEGER is
		do
			if is_reference then
				if destination.is_reference then
					Result := 1
				else
					Result := destination.jvm_expanded_from_reference(Current)
				end
			elseif destination.is_reference then
				jvm_to_reference
				Result := 1
			else
				Result := 1
			end
		end

	jvm_standard_is_equal is
		local
			rc: LIVE_TYPE; wa: ARRAY[RUN_FEATURE_2]
		do
			rc := live_type
			wa := rc.writable_attributes
			jvm.std_is_equal(rc, wa)
		end

	is_a (other: TYPE_MARK): BOOLEAN is
		local
			bcn, obcn: CLASS_NAME
		do
			if other.is_separate then
				Result := is_a(other.local_from_separate)
			elseif is_separate then
				error_handler.add_position(start_position)
				error_handler.add_position(other.start_position)
				fatal_error(fz_sc1)
			end
			if not Result then
				bcn := class_text_name
				obcn := other.class_text_name
				if bcn.to_string = obcn.to_string then
					Result := True
				elseif is_reference and then other.is_expanded then
				elseif bcn.is_subclass_of(obcn) then
					if other.is_generic then
						Result := bcn.class_text.is_a_vncg(Current, other)
					else
						Result := True
					end
				end
			end
			if not Result then
				error_handler.type_error(Current, other)
			end
		end

	can_be_assigned_to (other: TYPE_MARK; t, o: TYPE; tp, op: POSITION): BOOLEAN is
		local
			bcn, obcn: CLASS_NAME
		do
			bcn := class_text_name
			obcn := other.class_text_name
			if bcn.to_string = obcn.to_string then
				Result := True
			elseif is_expanded and then other.is_expanded then
			elseif is_reference and then other.is_expanded then
			elseif other.is_any or else other.class_text.name.to_string = as_general then
				Result := True
			elseif other.is_generic then
				Result := t.is_conform_to_vncg(o)
			else
				Result := t.is_conform_to(o)
			end
			if not Result then
				error_handler.conformance_error(t, o, tp, op)
			end
		end

	accept (visitor: TYPE_CLASS_VISITOR) is
		do
			visitor.visit_type_class(Current)
		end

	c_header_pass1 is
		do
			standard_c_typedef
		end

	c_header_pass2 is
		do
		end

	c_header_pass3 is
		do
			if is_expanded then
				if need_c_struct then
					standard_c_struct
				end
			end
		end

	c_header_pass4 is
		do
			if is_reference then
				if need_c_struct then
					standard_c_struct
				end
			end
			standard_c_object_model
			standard_c_print_function
		end

	c_type_for_argument_in (buffer: STRING) is
		do
			if is_reference then
				buffer.append(fz_t0_star)
			elseif is_dummy_expanded then
				buffer.append(fz_int)
			else
				buffer.extend('T')
				id.append_in(buffer)
			end
		end

	c_type_for_target_in (buffer: STRING) is
		do
			if is_dummy_expanded then
				buffer.append(fz_int)
			else
				buffer.extend('T')
				id.append_in(buffer)
				buffer.extend('*')
			end
		end

	c_type_for_result_in (buffer: STRING) is
		do
			if is_reference then
				buffer.append(fz_t0_star)
			elseif is_dummy_expanded then
				buffer.append(fz_int)
			else
				buffer.extend('T')
				id.append_in(buffer)
			end
		end

	need_c_struct: BOOLEAN is
		do
			if is_dummy_expanded then
			elseif is_expanded then
				Result := True
			elseif live_type.is_tagged then
				Result := True
			else
				Result := live_type.writable_attributes /= Void
			end
		end

	start_position: POSITION is
		do
			Result := class_text_name.start_position
		end

	c_initialize is
		do
			if is_expanded then
				c_initialize_expanded
			else
				cpp.put_string(fz_null)
			end
		end

	c_initialize_in (str: STRING) is
		do
			if is_expanded then
				if need_c_struct then
					live_type.c_object_model_in(str)
				else
					str.extend('0')
				end
			else
				str.append(fz_null)
			end
		end

	need_gc_mark_function: BOOLEAN is
		do
			if is_reference then
				Result := True
			else
				Result := live_type.gc_mark_to_follow
			end
		end

feature {LIVE_TYPE}
	just_before_gc_mark_in (str: STRING) is
		do
			if is_reference then
				standard_just_before_gc_mark_in(str)
			end
		end

	gc_info_in (str: STRING) is
		do
			if is_reference then
				standard_gc_info_in(str)
			end
		end

	gc_define1 is
		do
			if is_reference then
				standard_gc_define1
			end
		end

	gc_define2 is
		do
			if is_reference then
				standard_gc_define2
			else
				standard_gc_define2_for_expanded
			end
		end

feature {TYPE_MARK}
	frozen short_ (shorted_type: TYPE) is
		do
			short_printer.put_class_name(class_text_name)
		end

feature {}
	make (bcn: like class_text_name) is
		require
			not bcn.predefined
			not bcn.is_tuple_related
		do
			class_text_name := bcn
			debug
				debug_info := bcn.to_string
			end
		ensure
			class_text_name = bcn
		end

	debug_info: STRING -- To view more under sedb.

end -- class TYPE_CLASS
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
