-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class TYPE_REFERENCE
	--
	-- Handling of the "reference FOO" type mark.
	--

inherit
	TYPE_MARK
		redefine set_start_position
		end

creation {ANY}
	make

feature {ANY}
	is_reference, need_gc_mark_function: BOOLEAN is True

	is_expanded, is_separate, is_user_expanded, is_dummy_expanded: BOOLEAN is False

	jvm_method_flags: INTEGER is 17

	start_position: POSITION
			-- Of the first letter of the reference keyword.

	mapped: TYPE_MARK
			-- The corresponding usually expanded one.

	written_name: HASHED_STRING is
		local
			buffer, mapped_wm: STRING
		do
			Result := written_name_memory
			if Result = Void then
				mapped_wm := mapped.written_mark
				buffer := once ".....         local unique buffer          ....."
				buffer.copy(once "reference ")
				buffer.append(mapped_wm)
				Result := string_aliaser.hashed_string(buffer)
				written_name_memory := Result
			end
		end

	is_static: BOOLEAN is
		do
			Result := mapped.is_static
			if Result and then run_type = Void then
				run_type := Current
			end
		end

	has_been_specialized: BOOLEAN is True

	specialize_in (new_type: TYPE): like Current is
		do
			Result := Current
		end

	specialize_thru (parent_type: TYPE; inherit_item: INHERIT_TEXT_ITEM; new_type: TYPE): like Current is
		do
			Result := Current
		end

	run_time_name: HASHED_STRING is
		do
			if run_type = Void then
				if mapped.is_static then
					run_type := Current
				else
					check
						False
					end
				end
			end
			if run_type = Current then
				Result := written_name
			else
				Result := run_type.written_name
			end
		end

	to_static (t: TYPE): like Current is
		local
			m: like mapped
		do
			m := mapped.to_static(t)
			if m = mapped then
				Result := Current
				if run_type = Void then
					run_type := Current
				end
			else
				Result := twin
				Result.init(m, Void, Void, Result)
			end
		end

	signature_resolve_in (new_type: TYPE): TYPE is
		local
			m: TYPE; res: like Current
		do
			m := mapped.signature_resolve_in(new_type) --*** memory leak
			res := twin
			res.init(m.canonical_type_mark, Void, Void, res)
			Result := res.type
		end

	actual_reference (destination: TYPE_MARK): TYPE_MARK is
		do
			check
				False
			end
		end

	actual_separate (destination: TYPE_MARK): TYPE_MARK is
		local
			sp: POSITION
		do
			sp := destination.start_position
			Result := separate_tools.create_type_separate(sp, Current)
		end

	id: INTEGER is
		do
			Result := live_type.id
		end

	is_generic: BOOLEAN is
		do
			Result := mapped.is_generic
		end

	generic_list: ARRAY[TYPE_MARK] is
		do
			Result := mapped.generic_list
		end

	local_from_separate: TYPE_MARK is
		do
			check
				False
			end
		end

	class_text_name: CLASS_NAME is
		do
			Result := mapped.class_text_name
		end

	to_runnable (ct: TYPE_MARK): like Current is
		local
			rtm: STRING
		do
			if rtm = as_integer_general then
				error_handler.add_position(start_position)
				error_handler.append("Type %"reference INTEGER_GENERAL%" is not valid (CTCR).")
				error_handler.print_as_fatal_error
			end
			if rtm = as_real_general then
				error_handler.add_position(start_position)
				error_handler.append("Type reference %"REAL_GENERAL%" is not valid (CTCR).")
				error_handler.print_as_fatal_error
			end
		end

	is_a (other: TYPE_MARK): BOOLEAN is
		do
			if written_name = other.written_name then
				Result := True
			else
				Result := mapped.is_a(other)
			end
			if not Result then
				error_handler.type_error(Current, other)
			end
		end

	can_be_assigned_to (other: TYPE_MARK; t, o: TYPE; tp, op: POSITION): BOOLEAN is
		do
			Result := mapped.type.is_a(o, tp, op) --*** perf: type lookup
			if not Result then
				error_handler.conformance_error(t, o, tp, op)
			end
		end

	smallest_ancestor (other: TYPE_MARK): TYPE_MARK is
		do
			if written_name = other.written_name then
				Result := Current
			else
				Result := mapped.smallest_ancestor(other)
			end
		end

	just_before_gc_mark_in (str: STRING) is
		do
			standard_just_before_gc_mark_in(str)
		end

	gc_info_in (str: STRING) is
		do
			standard_gc_info_in(str)
		end

	gc_define1 is
		do
			standard_gc_define1
		end

	gc_define2 is
		do
			standard_gc_define2
		end

	c_sizeof: INTEGER is
		do
			Result := c_sizeof_pointer
		end

	c_type_for_argument_in (str: STRING) is
		do
			str.append(fz_t0_star)
		end

	c_type_for_target_in (str: STRING) is
		do
			str.extend('T')
			id.append_in(str)
			str.extend('*')
		end

	c_type_for_result_in (str: STRING) is
		do
			str.append(fz_t0_star)
		end

	need_c_struct: BOOLEAN is
		do
			if live_type.is_tagged then
				Result := True
			else
				Result := live_type.writable_attributes /= Void
			end
		end

	c_header_pass1 is
		do
			standard_c_typedef
		end

	c_header_pass2 is
		do
		end

	c_header_pass3 is
		do
		end

	c_header_pass4 is
		do
			if need_c_struct then
				standard_c_struct
			end
			standard_c_object_model
			standard_c_print_function
		end

	c_initialize is
		do
			cpp.put_string(fz_null)
		end

	c_initialize_in (str: STRING) is
		do
			str.append(fz_null)
		end

	jvm_descriptor_in (str: STRING) is
		do
			str.append(jvm_root_descriptor)
		end

	jvm_target_descriptor_in (str: STRING) is
		do
		end

	jvm_return_code is
		do
			code_attribute.opcode_areturn
		end

	jvm_check_class_invariant is
		do
			standard_jvm_check_class_invariant
		end

	jvm_push_local (offset: INTEGER) is
		do
			code_attribute.opcode_aload(offset)
		end

	jvm_push_default: INTEGER is
		do
			Result := 1
			code_attribute.opcode_aconst_null
		end

	jvm_write_local_creation, jvm_write_local (offset: INTEGER) is
		do
			code_attribute.opcode_astore(offset)
		end

	jvm_xnewarray is
		local
			idx: INTEGER
		do
			idx := constant_pool.idx_jvm_root_class
			code_attribute.opcode_anewarray(idx)
		end

	jvm_xastore is
		do
			code_attribute.opcode_aastore
		end

	jvm_xaload is
		do
			code_attribute.opcode_aaload
		end

	jvm_if_x_eq: INTEGER is
		do
			Result := code_attribute.opcode_if_acmpeq
		end

	jvm_if_x_ne: INTEGER is
		do
			Result := code_attribute.opcode_if_acmpne
		end

	jvm_to_reference is
		do
		end

	jvm_expanded_from_reference (other: TYPE_MARK): INTEGER is
		do
			check
				False
			end
		end

	jvm_convert_to (destination: TYPE_MARK): INTEGER is
		do
			check
				destination.is_reference
			end
			Result := 1
		end

	jvm_standard_is_equal is
		local
			ca: like code_attribute; cp: like constant_pool; idx: INTEGER
		do
			ca := code_attribute
			cp := constant_pool
			idx := jvm_item_field_idx
			live_type.opcode_checkcast
			ca.opcode_getfield(idx, 0)
			ca.opcode_swap
			live_type.opcode_checkcast
			ca.opcode_getfield(idx, 0)
			mapped.jvm_standard_is_equal
		end

	short_ (shorted_type: TYPE) is
		do
			short_printer.hook_or(fz_reference, "reference ")
			mapped.short_(shorted_type)
		end

	declaration_type: like Current is
		do
			Result := declaration_type_memory
			if Result = Void then
				if mapped.is_static then
					check
						mapped.declaration_type = mapped
					end
					Result := Current
				else
					check
						mapped.declaration_type /= mapped
					end
					create Result.make(start_position, mapped.declaration_type)
				end
				declaration_type_memory := Result
			end
		end

	constrained_type: TYPE_MARK is
		local
			static_type: TYPE_REFERENCE
		do
			--*** Could use a `constrained_type_memory' ? <FM-3/8/2004>
			if mapped.is_static then
				check
					mapped.constrained_type = mapped
				end
				Result := Current
			else
				check
					mapped.constrained_type /= mapped
				end
				create static_type.make(start_position, mapped.constrained_type)
				Result := static_type
			end
		end

	accept (visitor: TYPE_REFERENCE_VISITOR) is
		do
			visitor.visit_type_reference(Current)
		end

feature {TYPE_MARK}
	set_start_position (sp: like start_position) is
		do
			start_position := sp
		end

feature {LIVE_TYPE}
	jvm_prepare_item_field is
		local
			name_idx, descriptor: INTEGER; cp: like constant_pool
		do
			cp := constant_pool
			name_idx := cp.idx_utf8(as_item)
			tmp_string.clear_count
			mapped.jvm_descriptor_in(tmp_string)
			descriptor := cp.idx_utf8(tmp_string)
			field_info.add(1, name_idx, descriptor)
		end

feature {TYPE_REFERENCE}
	init (m: like mapped; wnm: like written_name_memory; dtm: like declaration_type_memory; rt: like run_type) is
		require
			m /= Void
		do
			mapped := m
			written_name_memory := wnm
			declaration_type_memory := dtm
			run_type := rt
		ensure
			mapped = m
			written_name_memory = wnm
			run_type = rt
		end

feature {ANY}
	run_type: TYPE_MARK
			-- To cache `run_type' value.

feature {}
	declaration_type_memory: TYPE_REFERENCE

	written_name_memory: HASHED_STRING

	make (sp: like start_position; m: like mapped) is
		require
			m /= Void
		do
			start_position := sp
			mapped := m
			if m.written_mark = as_integer_general and then sp.class_text.name.to_string /= as_integer then
				error_handler.append("Type %"reference INTEGER_GENERAL%" is not valid (CTCR).")
				error_handler.add_position(sp)
				error_handler.print_as_fatal_error
			end
		ensure
			start_position = sp
			mapped = m
		end

	jvm_item_field_idx: INTEGER is
		local
			c, n, t: INTEGER; cp: like constant_pool
		do
			cp := constant_pool
			c := live_type.jvm_constant_pool_index
			n := cp.idx_utf8(as_item)
			tmp_string.clear_count
			mapped.jvm_descriptor_in(tmp_string)
			t := cp.idx_utf8(tmp_string)
			Result := cp.idx_fieldref5(c, n, t)
		end

end -- class TYPE_REFERENCE
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
