-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class BIT_CONSTANT
	--
	-- For Manifest Constant BIT_N. Example:   0101B
	--

inherit
	EXPRESSION

creation {ANY}
	make

feature {ANY}
	start_position: POSITION
			-- Of the first bit.

	value: STRING
			-- Only '0' and '1'.

	is_writable, is_current, is_implicit_current, is_manifest_string, is_result, is_void: BOOLEAN is False

	use_current, once_pre_computable: BOOLEAN is False

	is_static, extra_bracket_flag, has_been_specialized: BOOLEAN is True

	specialize_in (type: TYPE): like Current is
		do
			Result := Current
		end

	specialize_thru (parent_type: TYPE; inherit_item: INHERIT_TEXT_ITEM; new_type: TYPE): like Current is
		do
			Result := Current
		end

	specialize_2 (type: TYPE): like Current is
		do
			Result := Current
		end

	resolve_in (type: TYPE): TYPE is
		do
			if type_memory = Void then
				type_memory := type_mark_memory.resolve_in(type)
			end
			Result := type_memory
		end

	collect (type: TYPE): TYPE is
		do
			Result := resolve_in(type)
			smart_eiffel.collect_constant(Result)
		end

	side_effect_free (type: TYPE): BOOLEAN is
		do
			Result := True
		end

	adapt_for (type: TYPE): like Current is
		do
			Result := Current
		end

	declaration_type: TYPE is
		do
			if type_memory = Void then
				type_memory := smart_eiffel.get_type(type_mark_memory)
			end
			Result := type_memory
		end

	safety_check is
		do
		end

	mapping_c_target (type: TYPE; target_type: TYPE_MARK) is
		do
			if type_mark_memory.is_c_unsigned_ptr then
				cpp.put_character('&')
			end
			mapping_c_arg(type, target_type)
		end

	mapping_c_arg (type: TYPE; formal_arg_type: TYPE_MARK) is
		do
			compile_to_c(type)
		end

	compile_to_c (type: TYPE) is
		do
			if type_mark_memory.is_c_unsigned_ptr then
				cpp.register_bit_constant(Current)
			else
				cpp.put_string(fz_0x)
				cpp.put_string(hexa)
			end
		end

	compile_target_to_jvm, compile_to_jvm (type: TYPE) is
		local
			i, idx: INTEGER; ca: like code_attribute; cp: like constant_pool
		do
			ca := code_attribute
			cp := constant_pool
			idx := cp.idx_class2(fz_java_util_bitset)
			ca.opcode_new(idx)
			ca.opcode_dup
			ca.opcode_push_integer(value.count)
			idx := cp.idx_methodref3(fz_java_util_bitset, fz_35, fz_27)
			ca.opcode_invokespecial(idx, 0)
			from
				i := value.count
			until
				i = 0
			loop
				if value.item(i) = '1' then
					ca.opcode_dup
					ca.opcode_push_integer(i - 1)
					idx := cp.idx_methodref3(fz_java_util_bitset, fz_set, fz_27)
					ca.opcode_invokevirtual(idx, -2)
				end
				i := i - 1
			end
		end

	jvm_branch_if_false, jvm_branch_if_true (type: TYPE): INTEGER is
		do
			check
				False
			end
		end

	compile_to_jvm_into (type: TYPE; dest: TYPE_MARK): INTEGER is
		do
			Result := standard_compile_to_jvm_into(type, dest)
		end

	jvm_assign_creation, jvm_assign (type: TYPE) is
		do
			check
				False
			end
		end

	non_void_no_dispatch_type (type: TYPE): TYPE is
		do
			Result := resolve_in(type)
		end

	simplify (type: TYPE): BIT_CONSTANT is
		do
			Result := Current
		end

	precedence: INTEGER is
		do
			Result := atomic_precedence
		end

	bracketed_pretty, pretty (indent_level: INTEGER) is
		do
			pretty_printer.put_string(value)
			pretty_printer.put_character('B')
		end

	pretty_target (indent_level: INTEGER) is
		do
			pretty_printer.put_character('(')
			pretty(indent_level)
			pretty_printer.put_character(')')
			pretty_printer.put_character('.')
		end

	short (type: TYPE) is
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > value.count
			loop
				short_printer.put_character(value.item(i))
				i := i + 1
			end
			short_printer.put_character('B')
		end

	short_target (type: TYPE) is
		do
			bracketed_short(type)
			short_printer.put_dot
		end

	result_type: TYPE_BIT_1 is
		do
			Result := type_mark_memory
		end

	accept (visitor: BIT_CONSTANT_VISITOR) is
		do
			visitor.visit_bit_constant(Current)
		end

feature {C_PRETTY_PRINTER}
	c_define is
		local
			i, k, digit_nb: INTEGER
		do
			digit_nb := digit_nb.object_size * 2
			k := hexa.count #\\ digit_nb
			i := 1
			if k > 0 then
				from
					cpp.put_string(fz_0x)
				until
					k = 0
				loop
					cpp.put_character(hexa.item(i))
					i := i + 1
					k := k - 1
				end
				cpp.put_character(',')
			end
			from
			until
				i > hexa.count
			loop
				cpp.put_string(fz_0x)
				from
					k := digit_nb
				until
					k = 0
				loop
					cpp.put_character(hexa.item(i))
					i := i + 1
					k := k - 1
				end
				if i <= hexa.count then
					cpp.put_character(',')
				end
			end
		end

feature {}
	id: INTEGER

	hexa: STRING

	type_mark_memory: TYPE_BIT_1
			-- Of the corresponding manifest bit constant.

	type_memory: TYPE
			-- To avoid recomputation from `type_mark_memory'.

	make (sp: like start_position; s: STRING) is
			-- Where `s' contains only '0' and '1'.
		require
			not sp.is_unknown
			s.count > 0
			s.occurrences('0') + s.occurrences('1') = s.count
		local
			ic: INTEGER_CONSTANT
		do
			start_position := sp
			value := s
			create ic.make(value.count, start_position, create {TYPE_INTEGER}.integer_32(sp))
			create type_mark_memory.make(start_position, ic)
			hexa := value.twin
			hexa.to_hexadecimal
		ensure
			value.count = s.count
		end

	fz_0x: STRING is "0x"

invariant
	type_mark_memory.is_static

end -- class BIT_CONSTANT
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
