-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
deferred class CALL_PROC_CALL
	--
	-- Common root for CALL and PROC_CALL.
	--

insert
	GLOBALS

feature {}
	collect_exp (t: TYPE): TYPE is
		require
			t.feature_collection_done
			specialize_2_done: feature_stamp /= Void
		local
			target_type, sub_type, covariant_type: TYPE; rts: RUN_TIME_SET; af: ANONYMOUS_FEATURE
			i: INTEGER
		do
			target_type := target.collect(t)
			debug
				echo.put_string(once "   collecting feature call: ")
				echo.put_string(target_type.canonical_type_mark.written_mark)
				echo.put_character('.')
				echo.put_string(feature_name.complete_name)
				echo.put_new_line
			end
			-- register the feature as being alive:
			Result := smart_eiffel.collect(target_type, feature_stamp, False) -- now the arguments:
			if Result /= Void and then target_type.live_type /= Void then
				from
					rts := target_type.live_type.run_time_set
					i := 1
				until
					i > rts.count
				loop
					sub_type := rts.item(i).type
					if sub_type /= target_type then
						af := feature_stamp.resolve_dynamic_binding_for(sub_type).anonymous_feature(sub_type)
						covariant_type := af.result_type.resolve_in(sub_type)
						if covariant_type /= Result then
							assignment_handler.collect(start_position, covariant_type, Result)
						end
					end
					i := i + 1
				end
			end
			if arguments /= Void then
				--*** (PR 06/06/04) if the target is current, it's simple
				--collect. Do it anywhere for call and proc_call, unless
				--direct_call is implemented.
				arguments.collect(t, feature_stamp, target_type)
			else
				check
					feature_stamp.anonymous_feature(target_type).arguments = Void
				end
			end
			check
				set_collected_for(t)
			end
		end

feature {CALL_PROC_CALL}
	creation_type: TYPE_MARK is
			-- Must be set to avoid the standard check. (Only used by CREATE_TOOLS.)
		deferred
		end

	frozen standard_check_export (t: TYPE) is
			-- Called by `specialize_2' when `feature_stamp' has been defined
		local
			exported: BOOLEAN; ct: CLASS_TEXT; af: ANONYMOUS_FEATURE; cl: CLIENT_LIST; target_type: TYPE
		do
			--*** CAD: pas d'accord avec ce "if" (cf. BAD_EXPORT5 et 6)
			--if start_position.is_the_validity_check_site_of(t) then
			if creation_type = Void then
				if target.is_current then
					if not target.is_implicit_current then
						-- only test export clauses when the target is not an implicit Current
						target_type := t
					end
				else
					target_type := target.resolve_in(t)
				end
				if target_type /= Void then
					af := feature_stamp.anonymous_feature(target_type)
					ct := target_type.class_text
					cl := af.permissions
					if cl /= Void then
						exported := cl.gives_permission_to(t.class_text.name)
						if not exported then
							error_handler.add_position(start_position)
							error_handler.append("Forbidden call (i.e. exportation rules violated) when the type of Current is ")
							error_handler.append(t.canonical_type_mark.written_mark)
							error_handler.append(". (See the next error report for details.)")
							error_handler.print_as_error
							error_handler.add_position(af.start_position)
							error_handler.append("This feature is only exported to ")
							error_handler.append(cl.eiffel_view)
							cl.locate_in_error_handler
							error_handler.extend('.')
							error_handler.print_as_fatal_error
						end
					end
				end
			end
			--end
		end

feature {}
	collect_memory: FAST_ARRAY[TYPE]

	set_collected_for (t: TYPE): BOOLEAN is
			-- Debugging purpose only (used in check only)
		do
			if collect_memory = Void then
				create collect_memory.with_capacity(4)
			end
			if not collect_memory.fast_has(t) then
				check
					smart_eiffel.status.is_collecting
				end
				collect_memory.add_last(t)
			end
			Result := True
		end

	collected_for (t: TYPE): BOOLEAN is
			-- Use only in check clause.
		do
			Result := collect_memory /= Void and then collect_memory.fast_has(t)
		end

feature {ANY}
	side_effect_free (type: TYPE): BOOLEAN is
		local
			target_type: TYPE; af: ANONYMOUS_FEATURE
		do
			if arguments = Void then
				Result := True
			else
				Result := arguments.side_effect_free(type)
			end
			if Result then
				Result := False
				if target.side_effect_free(type) then
					target_type := target.resolve_in(type)
					if target_type.direct_non_void_call_flag then
						af := feature_stamp.anonymous_feature(target_type)
						Result := af.side_effect_free(target_type)
					end
				end
			end
		end

	adapt_for (type: TYPE): like Current is
		local
			tgt: like target; a: like arguments; call: CALL; target_type: TYPE
		do
			check
				collected_for(type)
			end
			tgt := target.adapt_for(type) -- Computation of extra local variables for user expanded object used as a target:
			call ?= tgt
			if call /= Void then
				target_type := call.extra_local_expanded(type)
				if target_type /= Void then
					smart_eiffel.register_extra_local_expanded(target_type)
				end
			end
			if arguments /= Void then
				a := arguments.adapt_for(type)
			end
			if tgt = target and then a = arguments then
				Result := Current
				set_run_feature_for(type)
			else
				Result := twin
				Result.set_target(tgt)
				Result.set_arguments(a)
				Result.set_run_feature_for(type)
			end
		end

feature {CALL_PROC_CALL, CREATE_TOOLS}
	frozen set_target (t: like target) is
		require
			t /= Void
		do
			target := t
		ensure
			target = t
		end

	set_arguments (a: like arguments) is
		deferred
		ensure
			arguments = a
		end

	frozen set_run_feature_for (type: TYPE) is
		local
			target_type: TYPE; af: ANONYMOUS_FEATURE
		do
			target_type := target.resolve_in(type) -- Assume that the obsolete mark is actually inherited:
			af := feature_stamp.anonymous_feature(target_type)
			af.obsolete_warning_check(start_position)
			run_feature := feature_stamp.run_feature_for(target_type)
		ensure
			run_feature /= Void
		end

feature {CALL_PROC_CALL, CREATE_TOOLS, E_PROCEDURE}
	frozen set_feature_stamp (fs: like feature_stamp) is
		do
			feature_stamp := fs
		ensure
			feature_stamp = fs
		end

feature {ANY}
	feature_stamp: FEATURE_STAMP
			-- The good feature_stamp for the feature in some type (unrelated to `like_what' due to renamings).

	target: EXPRESSION
			-- Target of the call.

	feature_name: FEATURE_NAME
			-- Written selector name of the call.

	run_feature: RUN_FEATURE
			-- Non Void when runnable and if any `run_feature' exists.

	arguments: EFFECTIVE_ARG_LIST is
			-- Arguments of the call if any.
		deferred
		ensure
			Result = Void or else Result.count > 0
		end

	arg_count: INTEGER is
			-- The `arguments' count or 0.
		deferred
		ensure
			Result >= 0
		end

	frozen start_position: POSITION is
		do
			Result := feature_name.start_position
		end

	use_current, frozen standard_use_current: BOOLEAN is
		do
			if arg_count > 0 then
				Result := arguments.use_current
			end
			if Result then
			elseif target.is_current then
				Result := True
			else
				Result := target.use_current
			end
		end

	frozen safety_check is
		local
			run_time_set: RUN_TIME_SET
		do
			if run_feature /= Void then
				if arguments /= Void then
					run_time_set := run_feature.live_type.run_time_set
					if run_time_set.count >= 1 then
						smart_eiffel.covariance_check(start_position, run_feature, run_time_set)
					end
				end
			end
			target.safety_check
			if arguments /= Void then
				arguments.safety_check
			end
		end

feature {}
	frozen call_proc_call_c2c (type: TYPE) is
		do
			cpp.put_cpc(type, feature_stamp, run_feature, target, arguments)
		end

	frozen call_proc_call_c2jvm (type: TYPE) is
		do
			jvm.put_cpc(type, feature_stamp, run_feature, target, arguments)
		end

invariant
	target /= Void

	feature_name /= Void

end -- class CALL_PROC_CALL
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
