-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class FAKE_TARGET
	--
	-- This pseudo expression is used to simulate some target of some static known TYPE. As an example, this kind of object is used
	-- to simulate the target of a CECIL call or an open target of a delayed call (see AGENT_CREATION objects).
	--

inherit
	EXPRESSION

creation {ANY}
	make

feature {ANY}
	is_current, is_implicit_current: BOOLEAN is False

	is_writable: BOOLEAN is False

	is_static: BOOLEAN is False

	once_pre_computable: BOOLEAN is False

	is_manifest_string: BOOLEAN is False

	is_result: BOOLEAN is False

	is_void: BOOLEAN is False

	use_current: BOOLEAN is False

	extra_bracket_flag: BOOLEAN is False

	precedence: INTEGER is
		do
			Result := atomic_precedence
		end

	start_position: POSITION

	result_type: TYPE_MARK

	safety_check is
		do
		end

	specialize_in (type: TYPE): like Current is
		local
			rt: like result_type
		do
			rt := result_type.specialize_in(type)
			Result := current_or_twin_init(rt)
		end

	specialize_thru (parent_type: TYPE; inherit_item: INHERIT_TEXT_ITEM; new_type: TYPE): like Current is
		local
			rt: like result_type
		do
			rt := result_type.specialize_thru(parent_type, inherit_item, new_type)
			Result := current_or_twin_init(rt)
		end

	specialize_2 (type: TYPE): like Current is
		do
			Result := Current
		end

	side_effect_free (type: TYPE): BOOLEAN is
		do
			Result := True
		end

	adapt_for (type: TYPE): like Current is
		do
			Result := current_or_twin_init(result_type.to_static(type))
		end

	has_been_specialized: BOOLEAN is
		do
			Result := result_type.has_been_specialized
		end

	collect (type: TYPE): TYPE is
		do
			Result := resolve_in(type)
		end

	resolve_in (type: TYPE): TYPE is
		do
			Result := result_type.resolve_in(type)
		end

	declaration_type: TYPE is
		do
			if result_type.is_static then
				Result := result_type.type
			else
				check
					result_type.is_like_current
				end
				Result := start_position.class_text.declaration_type_of_like_current
			end
		end

	non_void_no_dispatch_type (type: TYPE): TYPE is
		do
			-- *** To be done. *** (Dom. june 18th 2004) ***
		end

	simplify (type: TYPE): FAKE_TARGET is
		do
			Result := Current
		end

	compile_to_c (type: TYPE) is
		do
			cpp.put_character('C')
		end

	mapping_c_target (type: TYPE; target_type: TYPE_MARK) is
		do
			target_type.mapping_cast
			compile_to_c(type)
		end

	mapping_c_arg (type: TYPE; formal_arg_type: TYPE_MARK) is
		do
			mapping_c_target(type, result_type)
		end

	compile_to_jvm (type: TYPE) is
		do
			result_type.jvm_push_local(0)
		end

	compile_target_to_jvm (type: TYPE) is
		do
			compile_to_jvm(type)
		end

	jvm_branch_if_false (type: TYPE): INTEGER is
		do
			compile_to_jvm(type)
			Result := code_attribute.opcode_ifeq
		end

	jvm_branch_if_true (type: TYPE): INTEGER is
		do
			compile_to_jvm(type)
			Result := code_attribute.opcode_ifne
		end

	jvm_assign_creation, jvm_assign (type: TYPE) is
		do
			check
				False
			end
		end

	compile_to_jvm_into (type: TYPE; dest: TYPE_MARK): INTEGER is
		do
			check
				False
			end
			-- *** ???
		end

	pretty, bracketed_pretty, pretty_target (indent_level: INTEGER) is
		do
			check
				False
			end
		end

	short (type: TYPE) is
		do
		end

	short_target (type: TYPE) is
		do
		end

	accept (visitor: FAKE_TARGET_VISITOR) is
		do
			visitor.visit_fake_target(Current)
		end

feature {FAKE_TARGET}
	init (rt: like result_type) is
		require
			rt /= Void
		do
			result_type := rt
		ensure
			result_type = rt
		end

feature {}
	make (sp: like start_position; rt: like result_type) is
		require
			rt /= Void
		do
			start_position := sp
			result_type := rt
		ensure
			result_type = rt
		end

	current_or_twin_init (rt: like result_type): like Current is
		require
			rt /= Void
		do
			if rt = result_type then
				Result := Current
			else
				Result := twin
				Result.init(rt)
			end
		ensure
			Result.result_type = rt
		end

invariant
	result_type /= Void

end -- class FAKE_TARGET
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
