-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class ASSIGNMENT_ATTEMPT
	--
	-- For instructions like :
	--                          foo ?= bar
	--                          foo ?= bar + 1
	--

inherit
	INSTRUCTION

creation {ANY}
	make

feature {ANY}
	left_side: EXPRESSION

	right_side: EXPRESSION

	end_mark_comment: BOOLEAN is False

	side_effect_free (type: TYPE): BOOLEAN is
		do
		end

	use_current: BOOLEAN is
		do
			if left_side.use_current then
				Result := True
			else
				Result := right_side.use_current
			end
		end

	simplify (type: TYPE): INSTRUCTION is
		local
			rs: like right_side; lt, rt: TYPE
		do
			lt := left_side.resolve_in(type)
			rt := right_side.resolve_in(type)
			if rt.is_a(lt, right_side.start_position, left_side.start_position) then
				smart_eiffel.magic_count_increment
				-- An ordinary assignment is equivalent:
				create {ASSIGNMENT} Result.make(left_side, right_side.simplify(type))
			else
				error_handler.cancel
				if not lt.is_a(rt, left_side.start_position, right_side.start_position) then
					error_handler.cancel
					smart_eiffel.magic_count_increment
					-- An ordinary assignment with Void is equivalent:
					create {ASSIGNMENT} Result.make(left_side, create {E_VOID}.make(right_side.start_position))
				else
					rs := right_side.simplify(type)
					if rs.is_void then
						-- An ordinary assignment with Void is equivalent:
						smart_eiffel.magic_count_increment
						create {ASSIGNMENT} Result.make(left_side, rs)
					elseif rs /= right_side then
						smart_eiffel.magic_count_increment
						create {ASSIGNMENT_ATTEMPT} Result.make(left_side, rs)
					else
						Result := Current
					end
				end
			end
		end

	compile_to_c (type: TYPE) is
		local
			left_type, right_type: TYPE; run_time_set, run_time_set2: RUN_TIME_SET; i: INTEGER
		do
			left_type := left_side.resolve_in(type)
			right_type := right_side.resolve_in(type)
			if ace.no_check then
				cpp.put_trace_or_sedb_instruction(start_position)
			end
			cpp.put_string(once "/*AA*/")
			if right_type.is_expanded then
				error_handler.add_position(start_position)
				error_handler.append("Right-hand side expanded Not Yet Implemented.")
				error_handler.print_as_fatal_error
			end
			run_time_set := left_type.live_type.run_time_set
			if run_time_set.count = 0 then
				if not right_side.side_effect_free(type) then
					right_side.compile_to_c(type)
					cpp.put_string(fz_00)
				end
				left_side.compile_to_c(type)
				cpp.put_string(fz_30)
			elseif right_side.is_current then
				if run_time_set.has(right_type.live_type) then
					left_side.compile_to_c(type)
					cpp.put_string(once "=((void*)")
					right_side.compile_to_c(type)
					cpp.put_string(fz_14)
				else
					left_side.compile_to_c(type)
					cpp.put_string(fz_30)
				end
			elseif run_time_set.count = 1 and then not run_time_set.first.is_tagged then
				check
					ace.boost
				end
				left_side.compile_to_c(type)
				cpp.put_character('=')
				run_time_set2 := right_type.live_type.run_time_set
				if run_time_set2.count = 1 and then run_time_set2.first = run_time_set.first then
					-- Only one and the good one.
				else
					cpp.put_string(fz_null)
					cpp.put_character(';')
				end
				right_side.compile_to_c(type)
				cpp.put_string(fz_00)
			else
				-- General translation scheme:
				left_side.compile_to_c(type)
				cpp.put_character('=')
				if right_side.is_current then
					cpp.put_string(fz_cast_t0_star)
				end
				right_side.compile_to_c(type)
				cpp.put_string(once ";%Nif(NULL!=(")
				left_side.compile_to_c(type)
				cpp.put_string(once ")){%N")
				if smart_eiffel.scoop and then not left_type.is_separate and then right_type.is_separate then
					cpp.put_string(once "se_subsystem_t* self = se_current_subsystem_thread();%N")
					cpp.put_string(once "if (get_subsystem(")
					left_side.compile_to_c(type)
					cpp.put_string(once ", NULL) == self) {%N")
					left_side.compile_to_c(type)
					cpp.put_string(once "=((sT0*)(")
					left_side.compile_to_c(type)
					cpp.put_string(once "))->ref;%N}%N")
				end
				cpp.put_string(once "switch(((T0*)")
				left_side.compile_to_c(type)
				cpp.put_string(once ")->")
				cpp.put_string(once "id){%N")
				from
					i := 1
				until
					i > run_time_set.count
				loop
					check
						run_time_set.item(i).is_tagged
					end
					cpp.put_string(once "case ")
					cpp.put_integer(run_time_set.item(i).id)
					cpp.put_character(':')
					i := i + 1
				end
				cpp.put_string(once "%Nbreak;%Ndefault:%N")
				left_side.compile_to_c(type)
				cpp.put_string(once "=NULL;%N}%N}")
			end
		end

	compile_to_jvm (type: TYPE) is
		local
			left_type, right_type: TYPE; run_time_set: RUN_TIME_SET; rc: LIVE_TYPE; point1, i: INTEGER
			ca: like code_attribute; branch_index: INTEGER
		do
			left_type := left_side.resolve_in(type)
			right_type := right_side.resolve_in(type)
			ca := code_attribute
			if right_type.is_expanded then
				error_handler.add_position(start_position)
				fatal_error("Right-hand side expanded Not Yet Implemented.")
			end
			run_time_set := left_type.live_type.run_time_set
			if run_time_set.count = 0 then
				right_side.compile_to_jvm(type)
				ca.opcode_pop
				ca.opcode_aconst_null
				left_side.jvm_assign(type)
			else
				right_side.compile_to_jvm(type)
				ca.opcode_dup
				point1 := ca.opcode_ifnull
				from
					branch_index := ca.get_branch_array_index
					i := run_time_set.count
				until
					i = 0
				loop
					ca.opcode_dup
					rc := run_time_set.item(i)
					rc.opcode_instanceof
					ca.add_branch(ca.opcode_ifne, branch_index)
					i := i - 1
				end
				ca.opcode_pop
				ca.opcode_aconst_null
				ca.resolve_u2_branch(point1)
				ca.resolve_branches(branch_index)
				ca.release_branch_array_index
				left_side.jvm_assign(type)
			end
		end

	start_position: POSITION is
		do
			Result := left_side.start_position
		end

	specialize_in (type: TYPE): like Current is
		local
			l, r: EXPRESSION
		do
			l := left_side.specialize_in(type)
			r := right_side.specialize_in(type)
			if r = right_side and then l = left_side then
				Result := Current
			else
				create Result.make(l, r)
			end
		end

	specialize_thru (parent_type: TYPE; inherit_item: INHERIT_TEXT_ITEM; new_type: TYPE): like Current is
		local
			l, r: EXPRESSION
		do
			l := left_side.specialize_thru(parent_type, inherit_item, new_type)
			r := right_side.specialize_thru(parent_type, inherit_item, new_type)
			if r = right_side and then l = left_side then
				Result := Current
			else
				create Result.make(l, r)
			end
		end

	specialize_2 (type: TYPE): like Current is
		local
			l, r: EXPRESSION; l_dt, r_dt: TYPE
		do
			l := left_side.specialize_2(type)
			r := right_side.specialize_2(type)
			if start_position.is_the_validity_check_site_of(type) then
				l_dt := l.declaration_type
				r_dt := r.declaration_type
				if r.is_void then
					error_handler.add_position(right_side.start_position)
					error_handler.append("Void must not be the right-hand-side of an assignment attempt (too weird).")
					error_handler.print_as_fatal_error
				elseif l_dt.is_expanded then
					error_handler.append("The left-hand side of an assignment attempt must not be expanded. (Actually, ")
					error_handler.add_expression(left_side)
					error_handler.append(" is of type ")
					error_handler.add_type(l_dt)
					error_handler.append(".) Invalid reverse assignment (VJRV).")
					error_handler.add_position(start_position)
					error_handler.print_as_fatal_error
				elseif not l_dt.is_a(r_dt, l.start_position, r.start_position) then
					error_handler.add_position(start_position)
					error_handler.append("Invalid assignment attempt. The left-hand side expression must conform with the right-hand %
		%side. Note: the previous constraint is stricter than ETL is, but, the SmartEiffel team %
		%is convinced that it is better for your safety to enforce this extra constraint. If you really %
		%cannot work around, feel free to post your problem in our mailing list. If you agree with %
		%us, feel free to help us to convince other people (Wed Mar 19 2003). D.Colnet.")
					error_handler.print_as_fatal_error
				elseif r_dt.is_a(l_dt, r.start_position, l.start_position) then
					if type.name.to_string = as_general then
						-- To skip this very special site :-)
					else
						error_handler.append("Type ")
						error_handler.append(r_dt.name.to_string)
						error_handler.append(" is a kind of type ")
						error_handler.append(l_dt.name.to_string)
						error_handler.append(", hence, simple assignment is allowed (%"?=%" is not necessary).")
						error_handler.add_position(start_position)
						error_handler.print_as_warning
					end
				else
					error_handler.cancel
				end
			end
			-- of validity checks.
			if r = right_side and then l = left_side then
				Result := Current
			else
				-- Note: we do not need to add an implicit cast here because both operands are reference types.
				-- *** Should we check that the `right_side' is actually a reference ?
				-- *** (Dom july 12th 2004) ***
				create Result.make(l, r)
			end
		end

	has_been_specialized: BOOLEAN is
		do
			Result := left_side.has_been_specialized
			Result := Result and right_side.has_been_specialized
		end

	safety_check is
		do
			right_side.safety_check
		end

	pretty (indent_level: INTEGER) is
		local
			semi_colon_flag: BOOLEAN
		do
			pretty_printer.set_indent_level(indent_level)
			semi_colon_flag := pretty_printer.semi_colon_flag
			left_side.pretty(indent_level)
			pretty_printer.put_string(once " ?= ")
			pretty_printer.set_semi_colon_flag(False)
			right_side.pretty(indent_level)
			pretty_printer.set_semi_colon_flag(semi_colon_flag)
			if semi_colon_flag then
				pretty_printer.put_character(';')
			end
			pretty_printer.set_indent_level(0)
		end

	verify_scoop (type: TYPE; allowed: FORMAL_ARG_LIST) is
		local
			dummy: BOOLEAN
		do
			dummy := left_side.verify_scoop(type, allowed)
			dummy := right_side.verify_scoop(type, allowed)
		end

	collect (t: TYPE) is
		local
			l, r: TYPE
		do
			l := left_side.collect(t)
			r := right_side.collect(t)
			assignment_handler.collect(right_side.start_position, r, l)
		end

	adapt_for (t: TYPE): like Current is
		local
			l: like left_side; r: like right_side
		do
			l := left_side.adapt_for(t)
			r := right_side.adapt_for(t)
			if l = left_side and then r = right_side then
				Result := Current
			else
				create Result.make(l, r)
			end
		end

	accept (visitor: ASSIGNMENT_ATTEMPT_VISITOR) is
		do
			visitor.visit_assignment_attempt(Current)
		end

feature {}
	make (ls: like left_side; rs: like right_side) is
		require
			ls /= Void
			rs /= Void
		do
			left_side := ls
			right_side := rs
		end

invariant
	left_side.is_writable

	right_side /= Void

end -- class ASSIGNMENT_ATTEMPT
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
