-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class CREATION_CLAUSE
	--
	-- The store the contents of one creation clause.
	--   example 1
	--                creation {ANY} make
	--   example 2
	--                creation make, foo
	--   example 3
	--                creation {NONE,ANY} make, foo
	--   example 4
	--                creation
	--
	-- Note : The original text of the source file can be stored
	--        for pretty pretty_printing to be fine.
	--

inherit
	VISITABLE
insert
	GLOBALS

creation {ANY}
	make

feature {ANY}
	start_position: POSITION
			-- Of the "creation" keyword.

	clients: CLIENT_LIST

	comment: COMMENT

feature {}
	make (sp: like start_position; c: like clients; cm: like comment; pl: like procedure_list) is
		require
			not sp.is_unknown
			c /= Void
		do
			start_position := sp
			clients := c
			comment := cm
			procedure_list := pl
		ensure
			clients = c
			comment = cm
			procedure_list = pl
		end

feature {ANY}
	pretty is
		do
			if not pretty_printer.zen_mode then
				pretty_printer.skip_one_line
			end
			pretty_printer.set_indent_level(0)
			pretty_printer.keyword(once "creation")
			if clients /= Void then
				clients.pretty(1)
			end
			if comment /= Void then
				comment.pretty(1)
			end
			pretty_printer.set_indent_level(1)
			if procedure_list /= Void then
				procedure_list.pretty(1)
			end
		end

	short (heading_done: BOOLEAN; client: CLASS_NAME): BOOLEAN is
			-- True when at least one creation list is printed.
		local
			i: INTEGER; fn: FEATURE_NAME
		do
			if clients.gives_permission_to(client) then
				if not heading_done then
					short_printer.hook_or("hook100", "creation%N")
				end
				if procedure_list /= Void then
					from
						i := 1
					until
						i > procedure_list.count
					loop
						fn := procedure_list.item(i)
						short_printer.put_feature_1(fn)
						i := i + 1
					end
				end
				short_printer.hook_or("hook101", "")
				Result := True
			else
				error_handler.cancel
			end
		end

	has (fn: FEATURE_NAME): BOOLEAN is
		require
			fn /= Void
		do
			if procedure_list /= Void then
				Result := procedure_list.has(fn)
			end
		end

feature {CREATION_CLAUSE_LIST, CREATION_CLAUSE_VISITOR}
	procedure_list: FEATURE_NAME_LIST

feature {ANY}
	accept (visitor: CREATION_CLAUSE_VISITOR) is
		do
			visitor.visit_creation_clause(Current)
		end

feature {CREATION_CLAUSE_LIST}
	default_root: STRING is
			-- Return the default creation procedure name to be used as the root
			-- procedure (the execution entry point of the system).
		do
			if procedure_list /= Void then
				if procedure_list.has_make then
					Result := as_make
				else
					Result := procedure_list.first.to_string
				end
			end
		end

	root_creation_search (a_name: STRING): FEATURE_NAME is
		do
			if procedure_list /= Void then
				Result := procedure_list.root_creation_search(a_name)
			end
		end

	check_expanded_with (t: TYPE_MARK) is
		require
			t.is_expanded
		local
			rf: RUN_FEATURE; rf3: RUN_FEATURE_3; rc: LIVE_TYPE; type: TYPE
		do
			if True then
				--*** Les tests du else sont a faire dans le specialize
				--*** le collect doit juste rendre vivante la procedure
				--de creation.
				type := t.type
				if smart_eiffel.collect(type, type.lookup(procedure_list.first), True) /= Void then
					check
						False
					end
				end
			else
				-- *** CODE QUI SUIT A DEPLACER (Dom. 12/9/2003)
				if procedure_list = Void then
					error_handler.add_position(start_position)
					error_handler.append("Cannot create a class with an empty creation list.")
					error_handler.print_as_fatal_error
				end
				if procedure_list.count > 1 then
					error_handler.add_type_mark(t)
					error_handler.add_position(t.start_position)
					error_handler.append(fz_cbe)
					error_handler.add_position(start_position)
					fatal_error_vtec_2
				end
				rc := t.live_type
				crash
				-- *** rf := rc.get_feature(procedure_list.first)
				if rf = Void then
					error_handler.add_position(start_position)
					error_handler.append("Creation procedure for ")
					error_handler.add_type_mark(t)
					error_handler.add_position(t.start_position)
					error_handler.append(" not found.")
					error_handler.print_as_fatal_error
				end
				rf3 ?= rf
				if rf3 = Void then
					error_handler.add_position(start_position)
					error_handler.add_position(rf.start_position)
					error_handler.append("Feature found is not a procedure.")
					error_handler.print_as_fatal_error
				end
				if rf3.arguments /= Void then
					error_handler.add_type_mark(t)
					error_handler.add_position(t.start_position)
					error_handler.append(fz_cbe)
					error_handler.add_position(start_position)
					error_handler.add_position(rf3.start_position)
					error_handler.append("Procedure found has arguments. ")
					fatal_error_vtec_2
				end
			end
		end

	explicit_create_in (names: SET[FEATURE_NAME]) is
		require
			names /= Void
		local
			i: INTEGER; fn: FEATURE_NAME
		do
			if procedure_list /= Void then
				-- *** Actually, this should be syntactically rejected
				-- and we should have in the class invariant:
				--     procedure_list /= Void
				--
				-- Sinon, cela ne veut rien dire (cf. ECMA !).
				-- ... a faire plus tard. (Dom. 4 dec 2003)
				-- ***
				from
					i := procedure_list.count
				until
					i <= 0
				loop
					fn := procedure_list.item(i)
					names.add(fn)
					i := i - 1
				end
			end
		end

invariant
	clients /= Void

end -- class CREATION_CLAUSE
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
