-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class EXPORT_LIST

inherit
	VISITABLE
insert
	GLOBALS

creation {ANY}
	make

feature {ANY}
	start_position: POSITION
			-- Of keyword "export".

	clients_for (fn: FEATURE_NAME): CLIENT_LIST is
		local
			i: INTEGER; ei: EXPORT_ITEM
		do
			from
				i := items.upper
			until
				i = 0
			loop
				ei := items.item(i)
				if ei.affect(fn) then
					if Result = Void then
						Result := ei.clients
					else
						Result := Result.merge_with(ei.clients)
					end
				end
				i := i - 1
			end
		end

	pretty is
		local
			i, rank: INTEGER
		do
			pretty_printer.set_indent_level(2)
			pretty_printer.keyword(once "export")
			from
				i := items.lower
			until
				i > items.upper
			loop
				rank := rank + 1
				items.item(i).pretty(rank)
				i := i + 1
				if i <= items.upper then
					pretty_printer.put_character(';')
				end
			end
			pretty_printer.set_indent_level(0)
		end

feature {ANY}
	accept (visitor: EXPORT_LIST_VISITOR) is
		do
			visitor.visit_export_list(Current)
		end

feature {EXPORT_LIST_VISITOR}
	items: ARRAY[EXPORT_ITEM]

feature {}
	make (sp: like start_position; i: like items) is
		require
			not sp.is_unknown
			i.lower = 1
			not i.is_empty
		do
			start_position := sp
			items := i -- *** ADD some validity checking...
		ensure
			start_position = sp
			items = i
		end

invariant
	items.lower = 1

	not items.is_empty

end -- class EXPORT_LIST
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
