-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class INHERIT_TEXT_ITEM
	--
	-- To store the inheritance options for one parent of a class.
	--

inherit
	VISITABLE
insert
	GLOBALS

creation {ANY}
	make

feature {ANY}
	start_position: POSITION is
		do
			Result := type.start_position
		end

	is_non_conforming: BOOLEAN
			-- This parent is inserted (then current type does not 
			-- conform to this parent via this inherit text item)

	type: TYPE_MARK
			-- Declaration type mark of the parent.

	class_text: CLASS_TEXT
			-- An alias for `type.class_text'.

	class_text_name: STRING
			-- An alias for `class_text.name.to_string'.

	inherit_text: INHERIT_TEXT
			-- The corresponding one

	accept (visitor: INHERIT_TEXT_ITEM_VISITOR) is
		do
			visitor.visit_inherit_text_item(Current)
		end

feature {TYPE, FEATURE_ACCUMULATOR, PRECURSOR_CALL, ANONYMOUS_FEATURE_MIXER}
	reverse_rename (final_fn: FEATURE_NAME): FEATURE_NAME is
		require
			final_fn /= Void
		do
			if rename_list = Void then
				Result := final_fn
			else
				Result := rename_list.name_in_parent(final_fn)
			end
		ensure
		--*** Result /= Void
		end

feature {FEATURE_ACCUMULATOR, ANONYMOUS_FEATURE_MIXER, INHERIT_TEXT} --**PH(31/03/04): INHERIT_TEXT should be removed
	get_undefine (a_final_fn: FEATURE_NAME): FEATURE_NAME is
			-- If the undefine list exists and if `a_final_fn.to_key' is member of the undefine list, this function returns the 
			-- actual feature name which is in the list. (This can be useful to print error messages with accurate
			-- POSITIONs and to know if `a_final_fn' is undefined.)
		require
			a_final_fn /= Void
		do
			if undefine_list /= Void then
				Result := undefine_list.feature_name(a_final_fn.to_key)
			end
		ensure
		--***	 (Result /= Void) = undefine_of(a_final_fn)
		end

	get_redefine (a_final_fn: FEATURE_NAME): FEATURE_NAME is
		require
			a_final_fn /= Void
		do
			if redefine_list /= Void then
				Result := redefine_list.feature_name(a_final_fn.to_key)
			end
		ensure
			Result /= Void = has_redefine(a_final_fn)
		end

feature {INHERIT_TEXT_ITEM, INHERIT_TEXT_ITEM_VISITOR} -- Optionnal list in syntaxical order:
	rename_list: RENAME_LIST

	export_list: EXPORT_LIST

	undefine_list: FEATURE_NAME_LIST

	redefine_list: FEATURE_NAME_LIST

	select_list: FEATURE_NAME_LIST

feature {INHERIT_TEXT, TYPE, FEATURE_STAMP}
	do_rename (fn: FEATURE_NAME): like fn is
			-- If some rename exists for `fn', return the new name, otherwise, give back `fn'.
			-- Do not use `do_rename' to find the name in a sub_class (because of select).
			-- Use FEATURE_STAMP.resolve_dynamic_binding_for or FEATURE_STAMP.fs_for_building_type.
		require
			fn /= Void
		do
			if rename_list = Void then
				Result := fn
			else
				Result := rename_list.name_in_child(fn)
				if Result = Void then
					Result := fn
				end
			end
		ensure
			Result /= Void
		end

feature {INHERIT_TEXT, FEATURE_ACCUMULATOR, ANONYMOUS_FEATURE_MIXER, TYPE}
	id_extra_information (tfw: TEXT_FILE_WRITE) is
		do
			tfw.put_integer(class_text.id)
		end

	has_redefine (fn: FEATURE_NAME): BOOLEAN is
		require
			fn /= Void
		do
			if redefine_list /= Void then
				Result := redefine_list.has(fn)
			end
		end

	runnable_type (ct: TYPE_MARK): TYPE_MARK is
		require
			ct.is_static
		do
			if type.is_generic then
				Result := type.to_static(ct.type)
			else
				Result := type
			end
		ensure
			Result.is_static
		end

	is_a_vncg (t1, t2: TYPE_MARK): BOOLEAN is
		require
			t1.is_static
			t2.is_static
			t2.generic_list /= Void
			error_handler.is_empty
		local
			rank, i: INTEGER; gl, gl1, gl2: ARRAY[TYPE_MARK]; tfg: FORMAL_GENERIC_TYPE_MARK; rt: TYPE_MARK
			t2_bc: CLASS_TEXT; t2_bcn: STRING
		do
			t2_bc := t2.class_text
			t2_bcn := t2_bc.name.to_string
			if class_text_name = t2_bcn then
				-- Here is a good parent:
				gl := type.generic_list
				gl2 := t2.generic_list
				if gl = Void or else gl.count /= gl2.count then
					error_handler.add_position(start_position)
					error_handler.add_position(t2.start_position)
					error_handler.append("Bad number of generic arguments.")
					error_handler.print_as_fatal_error
				end
				if t1.is_generic then
					gl1 := t1.generic_list
					from
						Result := True
						i := gl2.count
					until
						not Result or else i = 0
					loop
						if gl.item(i).is_formal_generic then
							tfg ?= gl.item(i)
							rank := tfg.rank
							Result := gl1.item(rank).is_a(gl2.item(i))
						else
							rt := gl.item(i).specialize_in(t1.type).to_static(t1.type)
							Result := rt.is_a(gl2.item(i))
						end
						i := i - 1
					end
				elseif type.is_static then
					Result := type.is_a(t2)
				else
					-- Don't know exactely what to do here :(
					error_handler.add_position(start_position)
					error_handler.append("SmartEiffel is not yet able to handle this parent type %
		  %mark (probably because of constrained genericity). You %
		  %should try to inherit something else, more accurate for %
		  %the compiler. Sorry.")
					error_handler.print_as_fatal_error
				end
				if not Result then
					error_handler.cancel
				end
			elseif class_text.is_subclass_of(t2_bc) then
				if t1.is_generic or else not type.is_static then
					rt := type.specialize_in(t1.type).to_static(t1.type)
					Result := class_text.is_a_vncg(rt, t2)
				else
					Result := class_text.is_a_vncg(type, t2)
				end
				if not Result then
					error_handler.cancel
				end
			end
		ensure
			error_handler.is_empty
		end

	graph_node_vncg_update (site: POSITION; t1, t2: TYPE): BOOLEAN is
		require
			t1 /= Void
			t2 /= Void
			t1.is_a(t2, site, site)
			t2.generic_list /= Void
		local
			rank, i: INTEGER; gl: ARRAY[TYPE_MARK]; gl1, gl2: ARRAY[TYPE]; tfg: FORMAL_GENERIC_TYPE_MARK
			rt: TYPE; t2_bc: CLASS_TEXT; t2_bcn: STRING
		do
			t2_bc := t2.class_text
			t2_bcn := t2_bc.name.to_string
			if class_text_name = t2_bcn then
				-- Here is a good parent:
				gl := type.generic_list
				gl2 := t2.generic_list
				if t1.is_generic then
					gl1 := t1.generic_list
					from
						Result := True
						i := gl2.count
					until
						not Result or else i = 0
					loop
						if gl.item(i).is_formal_generic then
							tfg ?= gl.item(i)
							rank := tfg.rank
							Result := graph_node_vncg_update_continue(site, gl1.item(rank), gl2.item(i))
						else
							rt := gl.item(i).specialize_in(t1).resolve_in(t1)
							Result := graph_node_vncg_update_continue(site, rt, gl2.item(i))
						end
						i := i - 1
					end
				elseif type.is_static then
					Result := graph_node_vncg_update_continue(site, type.type, t2)
				end
			elseif class_text.is_subclass_of(t2_bc) then
				if t1.is_generic or else not type.is_static then
					rt := type.specialize_in(t1).resolve_in(t1)
					Result := class_text.graph_node_vncg_update(site, rt, t2)
				else
					Result := class_text.graph_node_vncg_update(site, type.type, t2)
				end
				if not Result then
					error_handler.cancel
				end
			end
		end

	e_feature (fn: FEATURE_NAME): ANONYMOUS_FEATURE is
		local
			fn2: FEATURE_NAME
		do
			if rename_list = Void then
				Result := class_text.e_feature(fn)
			else
				fn2 := rename_list.name_in_parent(fn)
				if fn2 /= Void then
					Result := class_text.e_feature(fn2)
				end
			end
		end

	clients_for (fn: FEATURE_NAME): CLIENT_LIST is
		require
			fn /= Void
		local
			fn2: like fn
		do
			if rename_list = Void then
				Result := consider_export(fn)
			else
				fn2 := rename_list.name_in_parent(fn)
				if fn2 /= Void then
					if export_list /= Void then
						Result := export_list.clients_for(fn)
					end
					if Result = Void then
						Result := consider_export(fn2)
					end
				end
			end
		end

	exports_for (fn: FEATURE_NAME): CLIENT_LIST is
			-- Only looks for a possible explicit export clause for a given feature name. Contrarily to `clients_for', does not
			-- compute the export list. Hence, the result may be Void.
		require
			fn /= Void
		local
			fn2: like fn
		do
			if rename_list = Void then
				fn2 := fn
			else
				fn2 := rename_list.name_in_parent(fn)
			end
			if fn2 /= Void and then export_list /= Void then
				Result := export_list.clients_for(fn)
			end
		end

	pretty is
		local
			end_needed: BOOLEAN
		do
			pretty_printer.set_indent_level(1)
			pretty_printer.put_type_mark(type)
			pretty_printer.set_indent_level(2)
			if comment /= Void then
				comment.pretty(2)
			end
			pretty_printer.set_indent_level(2)
			if rename_list /= Void then
				end_needed := True
				rename_list.pretty
			end
			pretty_printer.set_indent_level(2)
			if export_list /= Void then
				end_needed := True
				export_list.pretty
			end
			pretty_printer.set_indent_level(2)
			if undefine_list /= Void then
				end_needed := True
				pretty_printer.keyword(once "undefine")
				undefine_list.pretty(3)
			end
			pretty_printer.set_indent_level(2)
			if redefine_list /= Void then
				end_needed := True
				pretty_printer.keyword(once "redefine")
				redefine_list.pretty(3)
			end
			pretty_printer.set_indent_level(2)
			if select_list /= Void then
				end_needed := True
				pretty_printer.keyword(once "select")
				select_list.pretty(3)
			end
			pretty_printer.set_indent_level(2)
			if end_needed then
				pretty_printer.keyword(once "end")
			end
			pretty_printer.set_indent_level(0)
		end

	get_started (pl: like inherit_text) is
			-- First step of initialization (see also `get_started_step2').
		require
			pl /= Void
		local
			i: INTEGER; wbc: CLASS_TEXT; fn, old_fn, new_fn: FEATURE_NAME; all_check: BOOLEAN
		do
			inherit_text := pl
			class_text := type.class_text
			class_text_name := class_text.name.to_string
			all_check := ace.all_check
			wbc := inherit_text.class_text
			if forbidden_parent_list.fast_has(class_text_name) and then wbc.name.to_string /= as_integer_32 then
				error_handler.add_position(start_position)
				error_handler.append("You cannot inherit %"")
				error_handler.append(class_text_name)
				error_handler.append("%" (forbidden or not yet implemented).")
				error_handler.print_as_fatal_error
			end
			if all_check then
				if class_text.formal_generic_list /= Void then
				elseif type.is_generic then
					error_handler.add_position(class_text.name.start_position)
					error_handler.add_position(start_position)
					error_handler.append("This class is not generic (VTUG.1).")
					error_handler.print_as_fatal_error
				end
			end
			if all_check and then rename_list /= Void then
				rename_list.get_started(class_text)
			end
			if all_check and then undefine_list /= Void then
				from
					i := undefine_list.count
				until
					i = 0
				loop
					fn := undefine_list.item(i)
					old_fn := get_old_name(fn)
					if old_fn = Void and then not class_text.has(fn) then
						error_handler.add_position(fn.start_position)
						error_handler.append("Cannot undefine non-existent feature (VDUS.1).")
						error_handler.print_as_fatal_error
					end
					i := i - 1
				end
			end
			if redefine_list /= Void then
				from
					i := redefine_list.count
				until
					i = 0
				loop
					fn := redefine_list.item(i)
					if not wbc.proper_has(fn) then
						error_handler.add_position(fn.start_position)
						error_handler.append("Redefinition not found.")
						error_handler.print_as_fatal_error
					end
					if all_check then
						old_fn := get_old_name(fn)
						if old_fn = Void and then not class_text.has(fn) then
							error_handler.add_position(fn.start_position)
							error_handler.append(Vdrs1)
							error_handler.print_as_fatal_error
						end
						new_fn := get_new_name(fn)
						if new_fn /= Void then
							error_handler.add_position(new_fn.start_position)
							error_handler.add_position(fn.start_position)
							error_handler.append(Vdrs1)
							error_handler.print_as_fatal_error
						end
					end
					i := i - 1
				end
			end
			if all_check and then select_list /= Void then
				from
					i := select_list.count
				until
					i = 0
				loop
					fn := select_list.item(i)
					old_fn := get_old_name(fn)
					if old_fn = Void and then not class_text.has(fn) then
						error_handler.add_position(fn.start_position)
						error_handler.append(Vmss)
						error_handler.print_as_fatal_error
					end
					new_fn := get_new_name(fn)
					if new_fn /= Void then
						if get_old_name(new_fn) = Void then
							error_handler.add_position(new_fn.start_position)
							error_handler.add_position(fn.start_position)
							error_handler.append(Vmss)
							error_handler.print_as_fatal_error
						end
					end
					i := i - 1
				end
			end
		ensure
			inherit_text = pl
			class_text /= Void
			class_text_name /= Void
		end

	has_select_for (fn: FEATURE_NAME): BOOLEAN is
		require
			fn /= Void
		do
			if select_list /= Void then
				Result := select_list.has(fn)
			end
		end

feature {EIFFEL_PARSER}
	set_comment (c: like comment) is
		do
			comment := c
		end

	add_rename (rp: RENAME_PAIR) is
		require
			rp /= Void
		do
			if rename_list = Void then
				create rename_list.make(rp)
			else
				rename_list.add_last(rp)
			end
		end

	set_export (el: EXPORT_LIST) is
		require
			el /= Void
		do
			export_list := el
		ensure
			export_list = el
		end

	set_undefine (ul: FEATURE_NAME_LIST) is
		do
			undefine_list := ul
		ensure
			undefine_list = ul
		end

	set_redefine (rl: FEATURE_NAME_LIST) is
		do
			redefine_list := rl
		ensure
			redefine_list = rl
		end

	set_select (sl: like select_list) is
		do
			select_list := sl
		ensure
			select_list = sl
		end

feature {}
	comment: COMMENT
			-- Associated heading comment.

	make (non_conforming: BOOLEAN; t: like type) is
		require
			not t.is_anchored
		do
			is_non_conforming := non_conforming
			type := t
		ensure
			is_non_conforming = non_conforming
			type = t
		end

	forbidden_parent_list: ARRAY[STRING] is
		once
			Result :=
			<<as_bit, as_boolean, as_character, as_double, as_integer_8, as_integer_16, as_integer_32, as_native_array,
				as_none, as_pointer, as_real, as_weak_reference>>
		end

	get_old_name (fn: FEATURE_NAME): like fn is
		do
			if rename_list /= Void then
				Result := rename_list.name_in_parent(fn)
				if Result = fn then
					Result := Void
				end
			end
		end

	get_new_name (fn: FEATURE_NAME): like fn is
		do
			if rename_list /= Void then
				Result := rename_list.name_in_child(fn)
				if Result = fn then
					Result := Void
				end
			end
		end

	select_conflict (fn1, fn2: FEATURE_NAME) is
			--***PH: restaurer ce message (faire le test dans accu?)
		do
			error_handler.add_position(fn1.start_position)
			error_handler.add_position(fn2.start_position)
			error_handler.append("Select conflict for these features.")
			error_handler.print_as_fatal_error
		end

	Vmss: STRING is "Cannot select non-existent feature (VMSS)."

	Vdrs1: STRING is "Cannot redefine non-existent feature (VDRS.1)."

	consider_export (fn: FEATURE_NAME): CLIENT_LIST is
		require
			fn /= Void
		do
			if export_list = Void then
				Result := class_text.clients_for(fn)
			else
				Result := export_list.clients_for(fn)
				if Result = Void then
					Result := class_text.clients_for(fn)
				end
			end
		end

	graph_node_vncg_update_continue (site: POSITION; t1, t2: TYPE): BOOLEAN is
		do
			Result := t1.is_a(t2, site, site)
			if Result then
				assignment_handler.vncg(site, t1, t2)
			else
				error_handler.cancel
			end
		end

invariant
	not type.is_anchored

end -- class INHERIT_TEXT_ITEM
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
