-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
deferred class PARSER
	--
	-- To share some features among parsers (actually the
	-- EIFFEL_PARSER, the ACE parser and the INI_PARSER).
	--

inherit
	ANY
insert
	GLOBALS

feature {}
	line, column: INTEGER
			-- Current `line' number and current `column' number.

	current_line: STRING
			-- Current line string of `text'.

	cc: CHARACTER
			-- Current character in the `current_line'.

	pos (l, c: INTEGER): POSITION is
			-- Initialize a new one at line `l' and column `c'.
		require
			l >= 1
			c >= 1
		deferred
		end

	end_of_text: CHARACTER is '%/0/'
			-- Flag of the end of the `text'.

	update_last_manifest_string (p: POSITION; once_flag, unicode_flag: BOOLEAN; string: STRING;
										  source_view: FAST_ARRAY[STRING]) is
		deferred
		end

	may_expand_var is
			-- Expand an environment variable in curly brackets into 
			-- buffer, if variable expansion is active. Else just append 
			-- '$' to buffer.
		deferred
		end

	last_manifest_string: MANIFEST_STRING

	a_manifest_string: BOOLEAN is
		local
			state, l, c: INTEGER; once_flag, unicode_flag, stop: BOOLEAN
			ascii_code, unicode, digit_count: INTEGER; source_view: FAST_ARRAY[STRING]; line_view: STRING
			multi_line_end: STRING; left_alignment, current_alignment: INTEGER
		do
			l := line
			c := column
			if a_keyword(fz_once) then
				once_flag := True
			end
			if cc = 'U' then
				unicode_flag := True
				next_char
			end
			if cc = '%"' then
				Result := True
				if smart_eiffel.short_or_class_check_flag or else smart_eiffel.pretty_flag then
					create source_view.with_capacity(4)
					create line_view.make(32)
					source_view.add_last(line_view)
				end
				from
					buffer.clear_count
				until
					state > 9
				loop
					if source_view /= Void then
						line_view.extend(cc)
						if cc = '%N' then
							create line_view.make(32)
							source_view.add_last(line_view)
						end
					end
					next_char
					inspect
						state
					when 0 then
						-- Somewhere inside the manifest string.
						inspect
							cc
						when '%N' then
							from
							until
								multi_line_end /= Void
							loop
								if buffer.is_empty then
									fcp(em8)
								else
									inspect
										buffer.last
									when ' ', '%T' then
										buffer.remove_last(1)
										wcp(em42)
									when '{', '[' then
										multi_line_end := once "]foo%""
										multi_line_end.clear_count
										if buffer.last = '{' then
											multi_line_end.extend('}')
											left_alignment := 1
											state := 8
										else
											multi_line_end.extend(']')
											state := 7
										end
										multi_line_end.append(buffer)
										current_alignment := 1
										multi_line_end.remove_last(1)
										multi_line_end.extend('%"')
										buffer.clear_count
									else
										fcp(em8)
									end
								end
							end
						when '%"' then
							state := 11
						when '%%' then
							state := 1
						when '$' then
							may_expand_var
						else
							buffer.extend(cc)
						end
					when 1 then
						-- Just after a '%%'.
						state := 0
						inspect
							cc
						when '%N' then
							state := 3
						when 'A' then
							buffer.extend('%A')
						when 'B' then
							buffer.extend('%B')
						when 'C' then
							buffer.extend('%C')
						when 'D' then
							buffer.extend('%D')
						when 'F' then
							buffer.extend('%F')
						when 'H' then
							buffer.extend('%H')
						when 'L' then
							buffer.extend('%L')
						when 'N' then
							buffer.extend('%N')
						when 'Q' then
							buffer.extend('%Q')
						when 'R' then
							buffer.extend('%R')
						when 'S' then
							buffer.extend('%S')
						when 'T' then
							buffer.extend('%T')
						when 'U' then
							buffer.extend('%U')
						when 'V' then
							buffer.extend('%V')
						when '%%' then
							buffer.extend('%%')
						when '%'' then
							buffer.extend('%'')
						when '%"' then
							buffer.extend('%"')
						when '(' then
							buffer.extend('%(')
						when ')' then
							buffer.extend('%)')
						when '<' then
							buffer.extend('%<')
						when '>' then
							buffer.extend('%>')
						when '/' then
							ascii_code := 0
							digit_count := 0
							state := 4
						when ' ', '%T' then
							state := 2
						else
							fcp(em37)
							state := 0
						end
					when 2 then
						-- Extended form. At end of line.
						inspect
							cc
						when '%N' then
							state := 3
						when ' ', '%T' then
							wcp(em42)
						else
							fcp("In extended form of manifest string.%
							    % Bad character after '%%'.")
						end
					when 3 then
						-- Extended form. At beginning of line.
						inspect
							cc
						when ' ', '%T' then
						when '%%' then
							state := 0
						when '%N' then
							fcp(em8)
						else
							fcp("In extended form of manifest string.%
							    % Bad character before '%%'.")
						end
					when 4 then
						-- Inside ascii code. After opening '/'.
						inspect
							cc
						when '0' .. '9' then
							digit_count := digit_count + 1
							ascii_code := ascii_code * 10 + cc.decimal_value
						when 'x' then
							if digit_count = 1 and ascii_code = 0 then
								digit_count := 0
								state := 5
							else
								fcp(em38)
							end
						when 'U' then
							if digit_count = 0 then
								next_char
								if cc = 'x' then
									state := 6
									unicode := 0
									digit_count := 0
								else
									fcp(em38)
								end
							else
								fcp(em38)
							end
						when '/' then
							buffer.extend(ascii_code.to_character)
							state := 0
							if digit_count = 0 then
								ecp(em39)
							elseif ascii_code > Maximum_character_code then
								ascii_code := 0
								ecp(em40)
							end
						else
							fcp(em38)
						end
					when 5 then
						-- Inside ascii code. After /0x
						if cc.is_hexadecimal_digit then
							ascii_code := ascii_code * 16 + cc.hexadecimal_value
							digit_count := digit_count + 1
							if digit_count.even then
								buffer.extend(ascii_code.to_character)
								ascii_code := 0
							end
						elseif cc = '/' then
							state := 0
							if digit_count = 0 then
								ecp(em45)
							elseif digit_count.odd then
								ecp("You must use an even number of hexadecimal %
								    %digits to denote a sequence of CHARACTERs.")
							end
						else
							fcp(em45)
						end
					when 6 then
						-- Inside unicode code. After /Ux
						if cc.is_hexadecimal_digit then
							unicode := unicode * 16 + cc.hexadecimal_value
							digit_count := digit_count + 1
						elseif cc = '/' then
							state := 0
							if digit_count = 0 then
								ecp(em46)
							elseif digit_count > 8 then
								ecp("Too long hexadecimal sequence %
								    %for a single unicode value.")
							else
								unicode_string_buffer.clear_count
								if unicode_string_buffer.valid_unicode(unicode) then
									unicode_string_buffer.add_last(unicode)
									unicode_string_buffer.utf8_encode_in(buffer)
								else
									ecp("Invalid unicode notation (see also http://%
										 %www.unicode.org as well as feature %
										 %{UNICODE_STRING}.valid_unicode).")
								end
							end
						else
							fcp(em46)
						end
					when 7 then
						-- Just after the multi-line opener, in left
						-- alignment mode.
						check
							left_alignment = 0
						end
						inspect
							cc
						when ' ', '%T' then
							current_alignment := current_alignment + 1
						when '%N' then
							buffer.extend('%N')
							current_alignment := 1
						when end_of_text then
							error_handler.add_position(pos(l, c))
							fcp(em41)
						else
							buffer.extend(cc)
							if left_alignment = 0 then
								left_alignment := current_alignment
							end
							state := 8
						end
					when 8 then
						-- Inside multi-line manifest string.
						current_alignment := current_alignment + 1
						inspect
							cc
						when ' ', '%T' then
							if current_alignment >= left_alignment then
								buffer.extend(cc)
							end
						when '%N' then
							buffer.extend(cc)
							current_alignment := 0
						when end_of_text then
							error_handler.add_position(pos(l, c))
							fcp(em41)
						when '%"' then
							buffer.extend(cc)
							if buffer.has_suffix(multi_line_end) then
								state := 10
							end
						else
							if current_alignment < left_alignment then
								if cc = multi_line_end.first then
									buffer.extend(cc)
									state := 9
								else
									error_handler.add_position(pos(l, c))
									fcp(em41)
								end
							else
								buffer.extend(cc)
							end
						end
					when 9 then
						-- Inside multi-line closer.
						inspect
							cc
						when end_of_text then
							error_handler.add_position(pos(l, c))
							fcp(em41)
						when '%"' then
							buffer.extend(cc)
							if buffer.has_suffix(multi_line_end) then
								state := 10
							end
						else
							buffer.extend(cc)
						end
					end
				end
				if state = 10 then
					-- Good verbatim string.
					buffer.remove_suffix(multi_line_end)
					from
					until
						stop
					loop
						if buffer.is_empty then
							stop := True
						else
							stop := buffer.last = '%N'
							buffer.remove_last(1)
						end
					end
				end
				if source_view /= Void then
					line_view.extend('%"')
				end
				next_char
				skip_comments
				update_last_manifest_string(pos(l, c), once_flag, unicode_flag, buffer, source_view)
			elseif once_flag or else unicode_flag then
				go_back_at(l, c)
			end
		end

	buffer: STRING is
		once
			create Result.make(80)
		end

	unicode_string_buffer: UNICODE_STRING is
		once
			create Result.make_empty
		end

	go_back_at (l, c: INTEGER) is
			-- Go back to some existing line `l', and column `c'.
		require
			l >= 1
			c >= 1
		local
			sp: POSITION
		do
			line := l
			column := c
			current_line := parser_buffer.item(l)
			cc := current_line.item(c)
			if last_comment /= Void then
				sp := last_comment.start_position
				if l < sp.line then
					last_comment := Void
				elseif l = sp.line then
					if c < sp.column then
						last_comment := Void
					end
				end
			end
		end

	last_comment: COMMENT
			-- Void or waiting comment.

	drop_comments: BOOLEAN
			-- When objects COMMENT are not necessary.

	skip_comments is
			-- Skip separators and comments if any. Unless `drop_comments',
			-- comments are stored in `last_comment'.
		local
			sp: POSITION; same_line, stop: BOOLEAN
		do
			from
				same_line := True
			until
				stop
			loop
				inspect
					cc
				when '%N' then
					same_line := False
					next_char
				when ' ', '%T' then
					next_char
				when '-' then
					next_char
					if cc = '-' then
						if drop_comments then
							if line = parser_buffer.count then
								cc := end_of_text
								stop := True
							else
								line := line + 1
								column := 1
								current_line := parser_buffer.item(line)
								if current_line.count = 0 then
									cc := '%N'
								else
									cc := current_line.first
								end
							end
						else
							from
								sp := pos(line, column - 1)
								next_char
								lcs.clear_count
							until
								cc = '%N'
							loop
								lcs.extend(cc)
								next_char
							end
							if last_comment = Void then
								create last_comment.make(sp, lcs.twin, same_line)
							else
								last_comment.add_last(lcs.twin)
							end
						end
					else
						cc := '-'
						column := column - 1
						stop := True
					end
				else
					stop := True
				end
			end
		end

	next_char is
		do
			if column < current_line.count then
				column := column + 1
				cc := current_line.item(column)
			elseif column = current_line.count then
				column := column + 1
				cc := '%N'
			elseif line = parser_buffer.count then
				cc := end_of_text
			else
				line := line + 1
				column := 1
				current_line := parser_buffer.item(line)
				if current_line.count = 0 then
					cc := '%N'
				else
					cc := current_line.first
				end
			end
		end

	start_line, start_column: INTEGER
			-- To store beginning position of : `a_keyword', `a_integer',
			-- `a_real', `skip1', `skip2' and `skip1unless2'.

	a_keyword (keyword: STRING): BOOLEAN is
			-- Look for a `keyword' beginning strictly at current position, then,
			-- `skip_comment' is automatically called. A keyword is never followed
			-- by a character of this set: {'A'..'Z','a'..'z','0'..'9','_'}.
		require
			keyword.count >= 1
			not keyword.has('%N')
		local
			c, i, j: INTEGER
		do
			i := column
			c := keyword.count
			if current_line.count - i + 1 >= c then
				from
					start_line := line
					start_column := i
					j := 1
				until
					c <= 0
				loop
					if current_line.item(i) = keyword.item(j) then
						i := i + 1
						j := j + 1
						c := c - 1
					else
						c := -1
					end
				end
			end
			if c = 0 then
				if i > current_line.count then
					Result := True
					cc := '%N'
					column := i
					skip_comments
				else
					inspect
						current_line.item(i)
					when ' ', '%T', '-' then
						Result := True
						cc := current_line.item(i)
						column := i
						skip_comments
					when 'a' .. 'z', 'A' .. 'Z', '0' .. '9', '_' then
					else
						Result := True
						cc := current_line.item(i)
						column := i
					end
				end
			end
		end

	skip1 (char: CHARACTER): BOOLEAN is
		do
			if char = cc then
				start_line := line
				start_column := column
				Result := True
				next_char
				skip_comments
			end
		end

	ecp (msg: STRING) is
			-- Error at current position.
		do
			error_handler.add_position(current_position)
			error_handler.append(msg)
			error_handler.print_as_error
		end

	fcp (msg: STRING) is
			-- Fatal error at current position.
		do
			error_handler.add_position(current_position)
			error_handler.append(msg)
			error_handler.print_as_fatal_error
		end

	wcp (msg: STRING) is
			-- Warning at current position.
		do
			warning(current_position, msg)
		end

	current_position: POSITION is
		do
			Result := pos(line, column)
		end

	lcs: STRING is
			-- Last Comment String.
		once
			create Result.make(80)
		end

	token_buffer: TOKEN_BUFFER
			-- The temporary buffer for some name.

	em1: STRING is "Underscore in fractionnal part must group 3 digits."

	em2: STRING is "Feature name expected."

	em3: STRING is "Index value expected (%"indexing ...%")."

	em4: STRING is "Error in inspect."

	em5: STRING is "Added %",%"."

	em6: STRING is "Added %";%"."

	em7: STRING is "Unexpected comma (deleted)."

	em8: STRING is "Unexpected new line in manifest string."

	em9: STRING is "Underscore in number must group 3 digits."

	em10: STRING is "Bad character constant."

	em11: STRING is "Bad clients list."

	em12: STRING is "Deleted extra coma."

	em13: STRING is "Deleted extra separator."

	em14: STRING is "Class name should use only uppercase letters."

	em15: STRING is "Name of the current class expected."

	em16: STRING is "Type mark expected."

	em17: STRING is "Unexpected character."

	em18: STRING is "Useless keyword deleted."

	em21: STRING is "A formal argument is not a writable variable."

	em22: STRING is "Bad creation/create (writable expected)."

	em23: STRING is "Bad creation/create (procedure name expected)."

	em24: STRING is "Deleted extra semi-colon."

	em25: STRING is "Identifier should use only lowercase letters."

	em26: STRING is "Same identifier appears twice (local/formal)."

	em27: STRING is "Added %"(%"."

	em28: STRING is "Added %")%"."

	em29: STRING is "Added %":%"."

	em30: STRING is "Expected %"[%" (to start generic argument list)."

	em31: STRING is "Expected %"]%" (to finish generic argument list)."

	em32: STRING is "Type mark expected."

	em34: STRING is "Bad agent (call expected)."

	em35: STRING is "Empty generic list (deleted)."

	em36: STRING is "Closing %"}%" expected."

	em37: STRING is "Unknown special character."

	em38: STRING is "Unexpected character in decimal ascii code."

	em39: STRING is "Bad (empty?) ascii code."

	em40: STRING is "Decimal CHARACTER code out of range."

	em41: STRING is "Error inside multi-line manifest string."

	em42: STRING is "Extra blank or tab character removed in multi-line %
		   %manifest string."

	em43: STRING is "Invalid free operator (the last character must be %
		   %a member of this +-*/\=<>@#|& character list.)."

	em44: STRING is "Invalid free operator. (This character cannot be used.)"

	em45: STRING is "Unexpected character in hexadecimal ascii code."

	em46: STRING is "Unexpected character in hexadecimal unicode."

end -- class PARSER
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
