-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class TYPE
	--
	-- For all kinds of TYPEs even when they are not actually created (see also LIVE_TYPE).
	--

inherit
	LOADED_HIERARCHIC_GRAPH_NODE[TYPE, INHERIT_TEXT_ITEM]
		rename make as node_init
		redefine hash_code
		end
	VISITABLE
insert
	GLOBALS

creation {SMART_EIFFEL}
	make

feature {}
	make (ctm: like canonical_type_mark) is
		require
			smart_eiffel.status.is_collecting
			ctm.is_static
		local
			inherit_text: INHERIT_TEXT; a_inherit_item: INHERIT_TEXT_ITEM; parent_tm: TYPE_MARK; i: INTEGER
			gl: ARRAY[TYPE_MARK]; generic_type: TYPE; user_generic_type_mark: USER_GENERIC_TYPE_MARK
			relative_type: TYPE; ref_type: TYPE_REFERENCE; exp_type: EXPANDED_TYPE_MARK
		do
			debug
				debug_info := ctm.written_mark.twin
			end
			node_init(Current)
			class_text := ctm.class_text
			is_deferred := class_text.is_deferred
			name := ctm.written_name
			canonical_type_mark := ctm
			if ctm.is_generic then
				from
					gl := canonical_type_mark.generic_list
					create private_generic_list.with_capacity(gl.count, 1)
					i := gl.lower
				until
					i > gl.upper
				loop
					private_generic_list.add_last(gl.item(i).type)
					i := i + 1
				end
			end
			class_text.formal_generic_list_count_check(canonical_type_mark.start_position, gl)
			smart_eiffel.register_type(Current)
			feature_accumulator.new_incomplete_type
			inherit_text := canonical_type_mark.class_text.inherit_text
			if inherit_text = Void then
				check
					canonical_type_mark.written_mark = as_general
                  or else canonical_type_mark.written_mark = as_none
				end
				-- The inherit link to ANY is automatically added by the `eiffel_parser' even when nothing is written in 
				-- the original source file. To avoid cycles, GENERAL is not modified.
			else
				-- Parent list is build in reverse order (non conforming first)
				if inherit_text.insert_list /= Void then
					from
						i := inherit_text.insert_list.upper
					until
						i < inherit_text.insert_list.lower
					loop
						a_inherit_item := inherit_text.insert_list.item(i)
						--             parent_tm := a_inherit_item.type.resolve_in(Current).canonical_type_mark
						parent_tm := a_inherit_item.type.specialize_in(Current)
						parent_tm := parent_tm.to_static(Current) --************** PB pour construction de TYPE
						check
							parent_tm.is_static
						end
						add_parent(parent_tm.type)
						parents_edge_load.put(a_inherit_item, parents_edge_load.upper)
						-- *** attention: info non posee sur edge reverse
						i := i - 1
					end
				end
				if inherit_text.inherit_list /= Void then
					from
						i := inherit_text.inherit_list.upper
					until
						i < inherit_text.inherit_list.lower
					loop
						a_inherit_item := inherit_text.inherit_list.item(i)
						--             parent_tm := a_inherit_item.type.resolve_in(Current).canonical_type_mark
						parent_tm := a_inherit_item.type.specialize_in(Current)
						parent_tm := parent_tm.to_static(Current) --************** PB pour construction de TYPE
						check
							parent_tm.is_static
						end
						add_parent(parent_tm.type)
						parents_edge_load.put(a_inherit_item, parents_edge_load.upper)
						-- *** attention: info non posee sur edge reverse
						i := i - 1
					end
				end
			end
			if is_generic then
				-- The declaration_type is a parent for the current type, but as there is no inherit clause for it 
				-- we force it's creation before current type:
				generic_type := ctm.declaration_type.type

				-- Performs some extra checks only for the `canonical_type_mark' (i.e. made only once):
				user_generic_type_mark ?= canonical_type_mark
				if user_generic_type_mark /= Void then
					user_generic_type_mark.special_weak_reference_extra_check
				end
			end
			-- The relative type, if it exists, has to be build before current one.
			ref_type ?= canonical_type_mark
			if ref_type /= Void then
				relative_type := ref_type.mapped.type
			end
			exp_type ?= canonical_type_mark
			if exp_type /= Void then
				relative_type := exp_type.mapped.type
			end
			feature_accumulator.register(Current)
		ensure
			canonical_type_mark = ctm
			class_text = ctm.class_text
			name = ctm.written_name
			ctm.is_generic implies is_generic
			--Invariant:
			class_text = canonical_type_mark.class_text
			canonical_type_mark.is_static
		end

feature {EFFECTIVE_ROUTINE}
	collect_default_rescue (caller: ANONYMOUS_FEATURE) is
		require
			smart_eiffel.status.is_collecting
		local
			fs_current: FEATURE_STAMP; af_general, af_current: ANONYMOUS_FEATURE
			effective_routine: EFFECTIVE_ROUTINE
		do
			--drc: like default_rescue_compound
			--call: PROC_CALL_0; target: IMPLICIT_CURRENT
			if not default_rescue_collected then
				if Current /= type_general.type then
					af_general := general_default_rescue_fs.anonymous_feature(type_general.type)
					fs_current := general_default_rescue_fs.resolve_dynamic_binding_for(Current)
					af_current := fs_current.anonymous_feature(Current)
					if af_current /= af_general and af_current /= caller then
						-- We have a *real* default_rescue (redefined between
						-- GENERAL and the Current type). Let's create the
						-- `default_rescue_compound'.
						--***PH(05/03/04) removed this code, I add the code
						--after this comment instead.
						--create target.make(af_current.start_position)
						--create call.make(target, get_feature_name(fs_current))
						--create drc.make_1(Void, call)
						--default_rescue_compound := drc
						--drc.collect(Current)
						effective_routine ?= af_current --*** could it be deferred? external?
						default_rescue_compound := effective_routine.routine_body
						af_current.collect(Current)
					end
				end
				default_rescue_collected := True
			end
		ensure
			default_rescue_collected
		end

	default_rescue_compound: INSTRUCTION
			-- Default "rescue" compound used when a feature does not have its own rescue clause. It contains a 
			-- call to the redefined default_rescue routine.
			-- Void when the would-be called feature would be the one defined in GENERAL.

feature {}
	default_rescue_collected: BOOLEAN

feature {FEATURE_ACCUMULATOR}

	do_collect is
		local
			gl: ARRAY[TYPE_MARK]
		do
			-- Validity check section done here because the newly created TYPE is now really ready for use:
			if is_generic then
				gl := canonical_type_mark.generic_list
				class_text.formal_generic_list.constraint_genericity_check(Current, gl, canonical_type_mark.start_position)
				-- Checking infinite generic with a brute force idea:
				if canonical_type_mark.written_name.to_string.count > 512 then
					error_handler.add_position(canonical_type_mark.start_position)
					error_handler.append("Probably infinite or too long generic derivation of this type mark (see next warnings %
				    %to find the cause of the problem... and good luck).")
					error_handler.print_as_warning
					error_handler.append("The huge generic derivation related to the previous warning is: ")
					error_handler.append(canonical_type_mark.written_name.to_string)
					if error_handler.warning_counter > 50 then
						error_handler.print_as_error
					else
						error_handler.print_as_warning
					end
				end
			end
			inheritance_cycle_check
			-- *** Ou donc faut-il le placer ...
			check_multiple_conformant_path
			collect_features
			if not ace.boost then
				-- Invariant may be inherited in a class who has invariant checking (even if this class does not
				-- check invariants).
				build_actual_invariant
			end
			check
				valid_feature_stamps
			end
		end

	specialize_2 is
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > feature_stamps.count
			loop
				feature_stamps.item(i).specialize_2(Current)
				i := i + 1
			end
			if not ace.boost and then class_invariant /= Void then
				--***PH(29/02/04) use class_text.invariant_check instead of ace.boost
				class_invariant := class_invariant.specialize_2(Current)
			end
		end

feature {CLASS_CHECKER, TYPE}
	up_to_any_in (parent_list: FAST_ARRAY[TYPE]) is
			-- Add in `parent_list', a path to the ANY class, whithout including the ANY class itself.
			--*** Wrong comment
		require
			not parent_list.is_empty
		local
			p: TYPE; i: INTEGER
		do
			from
				i := parents.lower
			until
				i > parents.upper
			loop
				p := parents.item(i)
				if not parent_list.fast_has(p) then
					if p.name.to_string = as_any then
						-- We do not add this class, because everybody knows what comes from ANY.
					else
						parent_list.add_last(p)
						p.up_to_any_in(parent_list)
					end
				end
				i := i + 1
			end
		end

feature {MANIFEST_ARRAY, TYPE}
	rewritten_smallest_ancestor (other: like Current): like Current is
			--*** keep or destroy?
		local
			i: INTEGER
		do
			--*** manque la gestion de la conversion (INTEGER->DOUBLE etc)
			--*** manque TUPLEs et autres esoteriques
			if other = Current or else other.is_conform_to(Current) then
				Result := other
			else
				from
					i := parents_count
				until
					Result /= Void or else i < 1
				loop
					if not parent_edge_load(i).is_non_conforming then
						Result := parent(i).smallest_ancestor(other)
					end
					i := i - 1
				end
				if Result = Void then
					Result := smart_eiffel.type_any
				end
			end
		end

feature {ANY}
	live_type: LIVE_TYPE
			-- The corresponding one if already created.

	canonical_type_mark: TYPE_MARK
			-- The one used to create `Current'. (As the `canonical_type_mark' is selected nearly randomly by the compilation
			-- process, the `start_position' of the `canonical_type_mark' itself is not meaningful.)

	name: HASHED_STRING
			-- Because `canonical_type_mark' is static, this is an alias for `canonical_type_mark.written_name'

	class_text: CLASS_TEXT

	private_generic_list: ARRAY[TYPE]

	hash_code: INTEGER is
		do
			--***         Result := to_pointer.hash_code
			--***restore definition
			if hash_code_memo = 0 then
				hash_code_memo := next_hash
			end
			Result := hash_code_memo
		end

	hash_code_memo: INTEGER

	hash_counter: COUNTER is
		once
			create Result
		end

	next_hash: INTEGER is
		do
			hash_counter.increment
			Result := hash_counter.value
		end

	valid_feature_name (fn: FEATURE_NAME): BOOLEAN is
			-- Is `fn' an existing final feature name of `Current'?
		require
			fn /= Void
		do
			Result := feature_stamps /= Void and then feature_stamps.has(fn) --*** could be Void?
		end

	registered_name (fn: FEATURE_NAME): FEATURE_NAME is
			-- Useful for declaration position and frozen status.
		require
			valid_feature_name(fn)
		do
			Result := feature_stamps.internal_key(fn)
		end

	lookup (fn: FEATURE_NAME): FEATURE_STAMP is
			-- Get the FEATURE_STAMP relative to `fn'. (As expressed in the require assertion, we assume that `fn' is really a
			-- final name in `Current'.)
		require
			not fn.start_position.is_unknown
			feature_collection_done
			valid_feature_name(fn)
		do
			Result := feature_stamps.reference_at(fn)
		ensure
			Result /= Void
			has_feature(Result)
		end

	search (fn: FEATURE_NAME): FEATURE_STAMP is
			-- To make in a single step a `valid_feature_name' test which may be followed by a `lookup' call. (The require
			-- assertion of `search' is more relaxed and a Void `Result' indicate that `fn' is not a valid final name in
			-- `Current'.)
		require
			not fn.start_position.is_unknown
			feature_collection_done
		do
			Result := feature_stamps.reference_at(fn)
		ensure
			valid_feature_name(fn) = (Result /= Void)
			valid_feature_name(fn) implies has_feature(Result)
			Result /= Void implies Result = lookup(fn)
		end

	has_feature (fs: FEATURE_STAMP): BOOLEAN is
		require
			fs /= Void
		do
			Result := feature_stamps.fast_occurrences(fs) > 0
		end

	get_feature_name (fs: FEATURE_STAMP): FEATURE_NAME is
		require
			has_feature(fs)
		do
			Result := feature_stamps.fast_key_at(fs)
		ensure
			registered_name(Result) = Result
		end

	has_feature_from_string (s: STRING): BOOLEAN is
		require
			s /= Void
			string_aliaser.registered_one(s)
		local
			i: INTEGER
		do
			from
				i := feature_stamps.lower
			until
				i > feature_stamps.upper or else Result
			loop
				Result := feature_stamps.key(i).to_string = s
				i := i + 1
			end
		end

	name_from_string (s: STRING): FEATURE_NAME is
		require
			has_feature_from_string(s)
		local
			i: INTEGER
		do
			from
				i := feature_stamps.lower
			until
				Result /= Void
			loop
				if feature_stamps.key(i).to_string = s then
					Result := feature_stamps.key(i)
				end
				i := i + 1
			end
		ensure
			Result /= Void
			registered_name(Result) = Result
		end

	feature_stamp_of (feature_name: STRING): FEATURE_STAMP is
			-- Used to get the good one knowing that `feature_name' is the final existing name.
		require
			has_feature_from_string(feature_name)
		local
			i: INTEGER
		do
			from
				i := feature_stamps.lower
			until
				Result /= Void
			loop
				if feature_stamps.key(i).to_string = feature_name then
					Result := feature_stamps.item(i)
				end
				i := i + 1
			end
		ensure
			Result /= Void
		end

	feature_collection_done: BOOLEAN is
		do
			Result := feature_stamps /= Void
		end

	class_invariant: CLASS_INVARIANT

	copy_stamp: FEATURE_STAMP is
			-- The one of the `copy' feature (which is magically called inside `twin').
		local
			tg: like Current; fn: FEATURE_NAME
		do
			-- First, going back to GENERAL because of possible rename:
			tg := type_general.type
			fn := tg.class_text.general_copy_feature.names.first
			Result := tg.lookup(fn) -- Finally:
			Result := Result.resolve_dynamic_binding_for(Current)
		ensure
			Result /= Void
		end

	accept (visitor: TYPE_VISITOR) is
		do
			visitor.visit_type(Current)
		end

feature {ANONYMOUS_FEATURE}
	name_of (af: ANONYMOUS_FEATURE): FEATURE_NAME is
			-- Warning: TIME CONSUMING TASK. Try to use `get_feature_name' instead.
			--*** This may be wrong if synonyms are implemented using shared AF.
			--*** See ANONYMOUS_FEATURE.add_into_shared.
		local
			i: INTEGER; fs: FEATURE_STAMP
		do
			from
				i := feature_stamps.lower
			until
				Result /= Void or else i > feature_stamps.upper
			loop
				fs := feature_stamps.item(i)
				check
					integrity: fs.has_anonymous_feature_for(Current)
				end
				if fs.anonymous_feature(Current) = af then
					Result := feature_stamps.key(i)
				end
				i := i + 1
			end
		end

feature {ANY} -- some property similar to TYPE_MARK *without* start_position and some other
	is_deferred: BOOLEAN

	is_generic: BOOLEAN is
		do
			Result := private_generic_list /= Void
		ensure
			is_generic and then is_array implies Result
		end

	is_expanded: BOOLEAN is
		do
			Result := canonical_type_mark.is_expanded
		ensure
			Result = not is_reference --***	 Result = (is_kernel_expanded xor is_user_expanded xor is_native_array)
		end

	is_separate: BOOLEAN is
		do
			Result := canonical_type_mark.is_separate
		ensure
			Result implies is_reference
			Result implies class_text.maybe_separate
		end

	is_reference: BOOLEAN is
		do
			Result := canonical_type_mark.is_reference
		ensure
			Result = not is_expanded
		end

	generic_list: ARRAY[TYPE] is
		require
			is_generic
		do
			Result := private_generic_list
		ensure
			Result.lower = 1
			not Result.is_empty
		end

	local_from_separate: TYPE is
			--*** use cache for this result
		require
			is_separate
		do
			Result := canonical_type_mark.local_from_separate.type
		ensure
			not Result.is_separate
		end

	is_kernel_expanded: BOOLEAN is
			-- True for BOOLEAN, CHARACTER, INTEGER, REAL, DOUBLE and POINTER.
		do
			Result := canonical_type_mark.is_kernel_expanded
		ensure
		--***	 Result = (is_expanded and not (is_user_expanded xor is_native_array))
		end

	is_kernel_reference: BOOLEAN is
			-- True for reference BOOLEAN, reference CHARACTER, reference INTEGER, reference REAL, reference DOUBLE and reference POINTER.
		local
			ref: TYPE_REFERENCE
		do
			ref ?= canonical_type_mark
			if ref /= Void then
				Result := ref.mapped.is_kernel_expanded
			end
		end

	is_kernel: BOOLEAN is
		do
			Result := is_kernel_expanded or else is_kernel_reference
		ensure
			Result = (is_kernel_expanded or else is_kernel_reference)
		end

	is_boolean: BOOLEAN is
		do
			Result := canonical_type_mark.is_boolean
		ensure
			Result implies is_kernel_expanded
		end

	is_character: BOOLEAN is
		do
			Result := canonical_type_mark.is_character
		ensure
			Result implies is_kernel_expanded
		end

	is_integer: BOOLEAN is
			-- Is it some INTEGER_* type?
		do
			Result := canonical_type_mark.is_integer
		ensure
			Result implies is_kernel_expanded
		end

	is_reference_integer: BOOLEAN is
			-- Is it some reference INTEGER_* type?
		local
			ref_type: TYPE_REFERENCE
		do
			ref_type ?= canonical_type_mark
			if ref_type /= Void then
				Result := ref_type.mapped.is_integer
			end
		end

	is_real: BOOLEAN is
		do
			Result := canonical_type_mark.is_real
		ensure
			Result implies is_kernel_expanded
		end

	is_double: BOOLEAN is
		do
			Result := canonical_type_mark.is_double
		ensure
			Result implies is_kernel_expanded
		end

	is_pointer: BOOLEAN is
		do
			Result := canonical_type_mark.is_pointer
		ensure
			Result implies is_kernel_expanded
		end

	is_string: BOOLEAN is
		do
			Result := canonical_type_mark.is_string
		end

	is_unicode_string: BOOLEAN is
		do
			Result := canonical_type_mark.written_mark = as_unicode_string
		end

	is_array: BOOLEAN is
		require
			is_generic
		do
			Result := canonical_type_mark.is_array
		ensure
			Result implies generic_list.count = 1
		end

	is_bit: BOOLEAN is
		do
			Result := canonical_type_mark.is_bit
		end

	is_any: BOOLEAN is
		do
			Result := canonical_type_mark.is_any
		end

	is_none: BOOLEAN is
		do
			Result := canonical_type_mark.is_none
		end

	is_native_array: BOOLEAN is
		do
			Result := canonical_type_mark.is_native_array
		ensure
		--***	 Result = (is_expanded and not (is_kernel_expanded xor is_user_expanded))
		end

	is_agent: BOOLEAN is
		do
			Result := canonical_type_mark.is_agent
		end

	is_tuple: BOOLEAN is
		do
			Result := canonical_type_mark.is_tuple
		end

	actual_reference (destination: TYPE): TYPE is
			-- If `destination' is not Void and if `destination' is the exact most acurate corresponding reference type of
			-- `Current', `destination' is returned without any modification (as an example, this occurs when `Current' represents
			-- the "INTEGER" type while `destination' represents the "reference INTEGER" type). Otherwise, when `destination' is
			-- Void or not specialized enough the actual concrete Result type is created (as an example, this occurs when `Current'
			-- represents the "INTEGER" type while destination represents the "ANY" type; a representation of "reference INTEGER"
			-- is thus created).
		require
			is_expanded or is_separate
			destination = Void or else destination.is_reference and is_a_canonical(destination)
		do
			if destination = Void then
				Result := canonical_type_mark.actual_reference(Void).type
			else
				Result := canonical_type_mark.actual_reference(destination.canonical_type_mark).type
			end
		ensure
			Result.is_reference
		end

	actual_separate (destination: TYPE): TYPE is
			-- If `destination' is the exact concrete corresponding separate type of `Current', `destination' is returned without
			-- any modification. Otherwise, when `destination' is not specialized enough the actual concrete Result type is
			-- created.
		require
			is_reference
			destination.is_separate
			is_a_canonical(destination)
		do
			Result := canonical_type_mark.actual_separate(destination.canonical_type_mark).type
		ensure
			Result.is_separate
		end

	convertible_to (other: TYPE): BOOLEAN is
		require
			error_handler.is_empty
		do
			Result := canonical_type_mark.convertible_to(other.canonical_type_mark)
		ensure
			error_handler.is_empty
		end

	is_a (other: TYPE; current_position, other_position: POSITION): BOOLEAN is
			-- Type conformance checking: Is it a kind of `other'?  Assume that the context is identical.
			--
			-- When False, `error_handler' is filled if needed with the corresponding bad types, but the error report is not
			-- printed for the caller to add some comments or for the caller to cancel `error_handler'.
			--
		require
		--***not smart_eiffel.status.is_collecting -- you should not have such questions when the collect is over!
			error_handler.is_empty
		local
			s: TYPE_SEPARATE
		do
			if Current = other then
				Result := True
			elseif other.is_separate then
				s ?= canonical_type_mark
				Result := s.mapped.can_be_assigned_to(other.canonical_type_mark, Current, s.mapped.type, current_position
				--*** perf: type lookup
				, other_position)
			elseif is_separate then
				error_handler.add_position(current_position)
				error_handler.add_position(other_position)
				fatal_error(fz_sc1)
			else
				Result := canonical_type_mark.can_be_assigned_to(other.canonical_type_mark, Current, other, current_position, other_position)
			end
		ensure
			nb_errors = old nb_errors
			Result = error_handler.is_empty --; (not Result) implies (not error_handler.is_empty)
		end

	is_a_test (other: TYPE): BOOLEAN is
			-- Same behavior as plain `is_a', except that the `error_handler' is not filled in case
			-- of a False `Result'.
		local
			unknown_position: POSITION
		do
			Result := Current.is_a(other, unknown_position, unknown_position)
			error_handler.cancel
		end

	is_a_canonical (other: TYPE): BOOLEAN is
			-- only useful in assertions
		local
			unknown_position: POSITION
		do
			Result := is_a(other, unknown_position, unknown_position)
			if not Result then
				error_handler.cancel
			end
		end

	smallest_ancestor (other: TYPE): TYPE is
			-- Return the smallest common ancestor.
		local
			type_mark: TYPE_MARK
		do
			type_mark := canonical_type_mark.smallest_ancestor(other.canonical_type_mark)
			if type_mark /= Void then
				Result := type_mark.type
			end
		ensure
			Result /= Void implies (Current.is_a_canonical(Result) and other.is_a_canonical(Result))
		end

	is_dummy_expanded: BOOLEAN is
			-- True when is it a user's expanded type with no attribute.
		require
			smart_eiffel.status.collecting_done
		do
			if is_user_expanded then
				Result := live_type.is_dummy_expanded
			end
		end

	is_user_expanded: BOOLEAN is
			-- Is it really a user expanded type ?
		do
			Result := canonical_type_mark.is_user_expanded
		ensure
		--***	 Result = (is_expanded and not (is_kernel_expanded xor is_native_array))
		end

	c_type_for_target_in (str: STRING) is
			-- Append in `str' the C type to use when current Eiffel type is
			-- used for the target of a feature.
		require
			smart_eiffel.is_ready
			str /= Void
		do
			canonical_type_mark.c_type_for_target_in(str)
		end

	c_type_for_result_in (str: STRING) is
			-- Append in `str' the C type to use when current Eiffel
			-- type is used as a result type of a C function.
		require
			smart_eiffel.is_ready
			str /= Void
		do
			canonical_type_mark.c_type_for_result_in(str)
		end

	mapping_cast is
			-- Produce a C cast for conversion into current C type.
			-- For example: (T27*)
		require
			live_type.at_run_time
		do
			canonical_type_mark.mapping_cast
		end

	direct_non_void_call_flag: BOOLEAN is
		require
			smart_eiffel.status.collecting_done
		local
			run_time_set: RUN_TIME_SET
		do
			if live_type /= Void then
				run_time_set := live_type.run_time_set
				Result := run_time_set.count = 1 and then run_time_set.first = live_type
			end
		end

feature {TYPE}
	feature_stamps: DICTIONARY[FEATURE_STAMP, FEATURE_NAME]

feature {AGENT_ARGS, AGENT_CREATION, AGENT_LAUNCHER, AGENT_POOL, AGENT_TYPE_MARK, TYPE_VISITOR}
	open_arguments: ARRAY[TYPE] is
			-- (The `Result' can be Void when open arguments is the empty TUPLE.)
		require
			is_generic
			canonical_type_mark.is_agent
		do
			Result := private_generic_list.item(2).private_generic_list
		end

	agent_result: TYPE is
		require
			is_generic
			canonical_type_mark.is_agent
		do
			if private_generic_list.upper > 2 then
				Result := private_generic_list.item(3)
			end
		end

feature {LIVE_TYPE}
	forget_previous_collect is
			-- *** Do not know yet if this feature must be better called directly by `smart_eiffel'?
			-- *** (Dom july 14th 2004) ***
		do
			default_rescue_collected := False
		end

	adapt_class_invariant is
		require
			smart_eiffel.status.is_adapting
		do
			if class_text.invariant_check and then class_invariant /= Void then
				class_invariant := class_invariant.adapt_for(Current)
			end
		end

	set_live_type (lt: like live_type) is
		require
			lt /= Void
			must_be_done_once: live_type = Void
		do
			live_type := lt
		ensure
			lt.type = Current
			lt.name = name
			live_type = lt
		end

feature {TYPE_LIKE_FEATURE, PRECURSOR_CALL, E_RESULT, FEATURE_NAME, CALL, ARGUMENT_NAME2, SMART_EIFFEL}
	feature_accumulator: FEATURE_ACCUMULATOR is
		once
			create Result.make
		end

feature {CALL_0}
	find_manifest_expression_for (fs: FEATURE_STAMP; fn: FEATURE_NAME): MANIFEST_EXPRESSION is
		require
			fn /= Void
		local
			af: ANONYMOUS_FEATURE; cst_att: CST_ATT
		do
			af := find_anonymous_feature_for(fs, fn)
			cst_att ?= af
			if cst_att = Void then
				error_handler.append("This call should be some constant feature call (i.e. a statically computable value). %
                                 %(See the definition found in the next error message.)")
				error_handler.add_position(fn.start_position)
				error_handler.print_as_error
				error_handler.append("This is not a constant feature.")
				error_handler.add_position(af.start_position)
				error_handler.print_as_fatal_error
			end
			Result ?= cst_att.value --    ****
			check
				Result /= Void -- *** Actually, we should change the type in CST_ATT I think. (Dom 22 0ct 2003)
			end
		ensure
			Result /= Void
		end

feature {AGENT_CREATION, TYPE_BIT_2, CALL_1}
	find_anonymous_feature_for (fs: FEATURE_STAMP; fn: FEATURE_NAME): ANONYMOUS_FEATURE is
		require
			fs /= Void
			fn /= Void
		do
			if feature_accumulator.context_type /= Current then
				Result := fs.anonymous_feature(Current)
			else
				Result := feature_accumulator.anonymous_feature_for(fn)
			end
			if Result = Void then
				error_handler.append("No feature found for this call.")
				error_handler.add_position(fn.start_position)
				error_handler.print_as_fatal_error
			end
		ensure
			Result /= Void
		end

feature {FEATURE_ACCUMULATOR}
	add_feature (fn: FEATURE_NAME; fs: FEATURE_STAMP) is
		require
			fn /= Void
			fs /= Void
			not valid_feature_name(fn)
			class_text.proper_has(fn) implies fn.start_position.class_text = class_text
		do
			debug
				if debug_known_features = Void then
					debug_known_features := "["
				else
					debug_known_features.append(once " [")
				end
				debug_known_features.append(fn.to_string)
				debug_known_features.extend(']')
			end
			feature_stamps.add(fs, fn)
		end

	get_shared_generic_fs (fn: FEATURE_NAME): FEATURE_STAMP is
			-- The generic types of the same family (based on the same class) have to share their feature_stamps for identical
			-- feature_names. This is needed because we can put an ARRAY[APPLE] in an ARRAY[FRUIT]. This function is not useful
			-- for inherited methods because the parent's feature stamp is used. It's only for local definitions.
		require
			is_generic
		do
			Result := class_text.get_shared_generic_fs(Current, fn)
		end

feature {TYPE, TYPE_MARK}
	has_conforming_parent (other: like Current): BOOLEAN is
			-- True if `other' is a parent of Current, with a conforming inheritance, from the raw inheritance graph. The `is_a'
			-- niceties are not considered.
		local
			i: INTEGER
		do
			from
				if other = Current then
					Result := True
				end
				i := parents_count
			until
				Result or else i <= 0
			loop
				if parent(i) = other then
					Result := smart_eiffel.is_checking_signature or else parent_edge_load(i) = Void or else not parent_edge_load(i).is_non_conforming
				else
					Result := parent(i).has_conforming_parent(other)
				end
				i := i - 1
			end
		end

	is_conform_to (other: like Current): BOOLEAN is
			-- Conformance is used to check covariant redefinitions for
			-- example. This is not the same as `can_be_assigned_to'.
			-- Both types can be expanded for example.
		local
			i: INTEGER; unknown_position: POSITION
		do
			-- The general case first:
			from
				Result := Current = other
				i := parents_count
			until
				Result or else i <= 0
			loop
				if smart_eiffel.is_checking_signature or else parent_edge_load(i) = Void or else not parent_edge_load(i).is_non_conforming then
					Result := parent(i).is_a(other, unknown_position, unknown_position)
					if not Result then
						--*** automatic error_handler filling should be avoided.
						error_handler.cancel
					end
				end
				i := i - 1
			end
			-- Special case and exception now:
			if not Result then
				if is_integer then
					if canonical_type_mark.is_a(other.canonical_type_mark) then
						Result := True
					else
						--*** automatic error_handler filling should be avoided.
						error_handler.cancel
					end
				end
			end
		end

	is_conform_to_vncg (other: like Current): BOOLEAN is
		require
			other.generic_list /= Void
		local
			i: INTEGER; p: like Current; other_generic_list: like generic_list; unknown_position: POSITION
		do
			if class_text = other.class_text then
				--*** expanded/reference should be checked...
				other_generic_list := other.generic_list
				if generic_list = Void or else generic_list.count /= other_generic_list.count then
					error_handler.append("Bad number of generic arguments.")
				else
					from
						Result := True
						i := generic_list.count
					until
						not Result or else i = 0
					loop
						if generic_list.item(i).is_a(other_generic_list.item(i), unknown_position, unknown_position) then
							i := i - 1
						else
							Result := False
							error_handler.append(fz_bga)
							error_handler.extend(' ')
						end
					end
				end
			else
				from
					i := parents_count
				until
					Result or else i = 0
				loop
					p := parent(i)
					if parent_edge_load(i) = Void or else not parent_edge_load(i).is_non_conforming then
						--***add signature checking test?
						Result := p = other or else p.is_conform_to_vncg(other)
					end
					i := i - 1
				end
			end
		ensure
			Result implies is_a_canonical(other)
		end

feature {SMART_EIFFEL}
	root_procedure_check (procedure_name: FEATURE_NAME) is
			-- Look for the root procedure to start execution here.
			-- Do some checking on the root class (not deferred, not generic,
			-- really has `procedure_name' as a creation procedure etc.).
		local
			fs: FEATURE_STAMP; af: ANONYMOUS_FEATURE
		do
			if is_deferred then
				error_handler.append(name.to_string)
				error_handler.append(" cannot be a root class since it is a deferred class.")
				error_handler.print_as_warning
			end
			fs := search(procedure_name)
			if fs /= Void then
				af := fs.anonymous_feature_if_exist(Current)
			end
			if af = Void then
				error_handler.add_position(procedure_name.start_position)
				error_handler.append("Root procedure not found.")
				error_handler.print_as_fatal_error
			end
			if af.result_type /= Void then
				error_handler.add_position(af.start_position)
				error_handler.append("Invalid Root. Only procedure are allowed (VGCC.6).")
				error_handler.print_as_fatal_error
			end
			if af.arguments /= Void then
				error_handler.add_position(af.start_position)
				error_handler.append("The root procedure must not have arguments.")
				error_handler.print_as_fatal_error
			end
		end

feature {TYPE}
	print_graph_cycle (first: TYPE) is
		require
			has_parent_cycle
		local
			i: INTEGER
		do
			error_handler.append(name.to_string)
			if Current = first then
				error_handler.append(", ...")
				error_handler.print_as_fatal_error
			else
				error_handler.append(", ")
				from
					i := parents.upper
				until
					i < parents.lower
				loop
					if parents.item(i).feature_stamps = Void then
						error_handler.add_position(parents_edge_load.item(i).start_position)
						parents.item(i).print_graph_cycle(first)
					end
					i := i - 1
				end
			end
		end

	check_multiple_conformant_path is
		local
			first: TYPE
			i: INTEGER
			need_clean: BOOLEAN
		do
			from
				i := parents_count
			until
				i < 1 or first /= Void
			loop
				if not parent_edge_load(i).is_non_conforming then
					first := parent(i)
				end
				i := i - 1
			end
			from
			until
				i < 1
			loop
				if not parent_edge_load(i).is_non_conforming then
					if first /= Void then
						set_mark
						first.mark_conforming_parents
						first := Void
						need_clean := True
					end
					parent(i).mark_conforming_parents
				end
				i := i - 1
			end
			if need_clean then
				deep_unmark_connected -- *** non optimal
			end
		end

	mark_conforming_parents is
		local
			i: INTEGER
		do
			if mark then
				error_handler.append("Multiple conforming paths found. In order to get a non-conforming %
                                 %path, use 'insert' keyword for one of the arrows %
                                 %below instead of inherit.")
				error_handler.append("%NFirst path is: ")
				print_conforming_marked_path
				error_handler.append("%NHint: a conforming path is a path you get using the %"inherit%" keyword while %
											 %a non-conforming path is defined by using the %"insert%" keyword instead.%NNone of the features comming from the common ancestor may be redefined on any path. NOTE: THIS MAY CRASH THE COMPILER, USE AT YOUR OWN RISKS.")
				error_handler.print_as_warning
			else
				set_mark
				from
					i := parents_count
				until
					i < 1
				loop
					if parent_edge_load(i) /= Void and then not parent_edge_load(i).is_non_conforming then
						parent(i).mark_conforming_parents
					end
					i := i - 1
				end
			end
		end

	print_conforming_marked_path is
		local
			i: INTEGER
			path_found: BOOLEAN
		do
			from
				i := children_count
			until
				i < 1
			loop
				if child(i).mark and then child(i).has_direct_conforming_parent(Current) then
					if path_found then
						error_handler.append("%NOther path is: ")
					end
					child(i).print_conforming_marked_path
					error_handler.append(" -> ")
					error_handler.append(name.to_string)
					path_found := True
				end
				i := i - 1
			end
			if not path_found then
				error_handler.append(name.to_string)
			end
		end

	has_direct_conforming_parent(p: TYPE): BOOLEAN is
		local
			i: INTEGER
		do
			from
				i := parents_count
			until
				Result or else i <= 0
			loop
				if parent(i) = p then
					Result := not parent_edge_load(i).is_non_conforming
				end
				i := i - 1
			end
		end

feature {}
	inheritance_cycle_check is
		local
			i: INTEGER
		do
			from
				i := parents.upper
			until
				i < parents.lower
			loop
				if parents.item(i).feature_stamps = Void then
					check
						has_parent_cycle
					end
					error_handler.append("Cyclic inheritance graph: ")
					error_handler.append(name.to_string)
					error_handler.append(", ")
					error_handler.add_position(parents_edge_load.item(i).start_position)
					parents.item(i).print_graph_cycle(Current)
				end
				i := i - 1
			end
		end

	collect_features is
		require
			not has_parent_cycle
		local
			i, k: INTEGER; a_parent: TYPE; upper: INTEGER; pfs: like feature_stamps
			fn, final_name: FEATURE_NAME; iti: INHERIT_TEXT_ITEM; accu: FEATURE_ACCUMULATOR
		do
			if feature_stamps /= Void then
				not_yet_implemented
				--*** cycle dans le graphe d'heritage
			else
				accu := feature_accumulator
				accu.start(Current)
				if ace.high_memory_compiler then
					create {DOUBLE_HASHED_DICTIONARY[FEATURE_STAMP, FEATURE_NAME]} feature_stamps.make
				else
					create {HASHED_DICTIONARY[FEATURE_STAMP, FEATURE_NAME]} feature_stamps.make
				end
				from
					i := parents.upper
				until
					i < parents.lower
				loop
					a_parent := parents.item(i)
					iti := parents_edge_load.item(i)
					from
						pfs := a_parent.feature_stamps
						k := pfs.lower
						upper := pfs.upper
					until
						k > upper
					loop
						fn := pfs.key(k)
						if iti /= Void then
							final_name := iti.do_rename(fn)
						else
							final_name := fn
						end
						if not accu.is_known(final_name) then
							if class_text.proper_has(final_name) then
								final_name := class_text.proper_name_get(final_name)
							end
							collect_one_feature(final_name, i, pfs.item(k), fn)
						end
						k := k + 1
					end
					i := i - 1
				end
				collect_local_features
				accu.finalize
			end
		ensure
			feature_collection_done
		end

	collect_one_feature (final_fn: FEATURE_NAME; inherit_index: INTEGER; parent_fs: FEATURE_STAMP; parent_fn: FEATURE_NAME) is
			-- The loop is done in reverse order, so conforming types will be analyzed first.
			-- Types after `inherit_index' are ignored because it has
			-- already been used and we know `final_fn' not to be known  into.
		require
			parents.item(inherit_index).lookup(parent_fn) = parent_fs
		local
			accu: FEATURE_ACCUMULATOR; i: INTEGER; a_type: TYPE; a_fn: FEATURE_NAME; a_fs: FEATURE_STAMP
			a_inherit_item: INHERIT_TEXT_ITEM; an_af: ANONYMOUS_FEATURE
		do
			from
				accu := feature_accumulator
				accu.new_feature(final_fn)
				accu.add_inherited_definition(parents.item(inherit_index), parents_edge_load.item(inherit_index), parent_fs, parent_fn)
				i := inherit_index - 1
			until
				i < 0
			loop
				a_type := parents.item(i)
				a_inherit_item := parents_edge_load.item(i)
				a_fn := a_inherit_item.reverse_rename(final_fn)
				if a_fn /= Void then
					if a_type.valid_feature_name(a_fn) then
						a_fn := a_type.registered_name(a_fn)
					end
					a_fs := a_type.feature_stamps.reference_at(a_fn)
					if a_fs /= Void then
						accu.add_inherited_definition(a_type, a_inherit_item, a_fs, a_fn)
					end
				end
				i := i - 1
			end
			an_af := class_text.proper_get(final_fn)
			if an_af /= Void then
				accu.add_local_definition(an_af)
			end
		ensure
			feature_accumulator.is_known(final_fn)
		end

	collect_local_features is
		local
			i: INTEGER; d: DICTIONARY[ANONYMOUS_FEATURE, FEATURE_NAME]; fn: FEATURE_NAME
			accu: FEATURE_ACCUMULATOR
		do
			from
				accu := feature_accumulator
				d := class_text.feature_dictionary
				i := d.lower
			until
				i > d.upper
			loop
				fn := d.key(i)
				if not accu.is_known(fn) then
					accu.new_feature(fn)
					accu.add_local_definition(d.item(i))
				end
				i := i + 1
			end
		end

	build_actual_invariant is
			-- Called only once per type to gather inherited class invariant and local invariant.
		require
			not ace.boost
		local
			i: INTEGER; a_parent: TYPE; ci: CLASS_INVARIANT
		do
			class_invariant := class_text.class_invariant
			if class_invariant /= Void then
				class_invariant := class_invariant.specialize_in(Current)
			end
			from
				i := parents.upper
			until
				i < parents.lower
			loop
				a_parent := parents.item(i)
				ci := a_parent.class_invariant
				if ci /= Void then
					ci := ci.specialize_thru(a_parent, parents_edge_load.item(i), Current)
					if class_invariant = Void then
						class_invariant := ci
					else
						class_invariant := class_invariant.add_parent_class_invariant(ci)
					end
				end
				i := i - 1
			end
		end

	valid_feature_stamps: BOOLEAN is
			-- All our feature_stamps know us.
		local
			i: INTEGER
		do
			Result := True
			if feature_stamps /= Void then
				from
					i := feature_stamps.lower
				until
					i > feature_stamps.upper
				loop
					Result := Result and feature_stamps.item(i).has_anonymous_feature_for(Current)
					i := i + 1
				end
			end
		end

	debug_info: STRING

	debug_known_features: STRING

feature {ANY}
	direct_thru_step (parent_type: TYPE; inherit_item: INHERIT_TEXT_ITEM): BOOLEAN is
			-- Check that we are actually doing an atomic-valid direct step into the inheritance hierarchy (i.e. we are making
			-- a direct move move from `parent_type' using the `inherit_item' path to reach `Current'.
		require
			parent_type /= Void
			inherit_item /= Void
		local
			parent_type_tm: TYPE_MARK; parent_type_2: TYPE
		do
			-- The `parent_type' must be a direct parent of `Current':
			Result := has_parent(parent_type)
			if not Result then
				-- For easy debug:
				sedb_breakpoint
			else
				-- The `inherit_item' is actually a path to `parent_type':
				parent_type_tm := inherit_item.type
				if parent_type_tm.is_static then
					-- No need to specialize the type mark, but the True reason is that this `inherit_item' can
					-- be an implicit link to ANY.
				else
					parent_type_tm := parent_type_tm.specialize_in(Current)
				end
				parent_type_2 := parent_type_tm.resolve_in(Current)
				Result := parent_type_2 = parent_type -- For easy debug:
				if not Result then
					sedb_breakpoint
				end
			end
		ensure
			assertion_check_only: Result
		end

invariant
	-- This invariant is correct, but time consuming:
	--	class_text = canonical_type_mark.class_text
	--	canonical_type_mark.is_static

end -- class TYPE
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
