-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
deferred class KERNEL_EXPANDED_TYPE_MARK
	--
	-- Root of TYPE_BOOLEAN, TYPE_CHARACTER, TYPE_INTEGER, TYPE_REAL,
	-- TYPE_DOUBLE and TYPE_POINTER.
	--

inherit
	STATIC_TYPE_MARK

insert
	NON_GENERIC_TYPE_MARK

feature {ANY}
	is_expanded: BOOLEAN is True

	is_reference, is_dummy_expanded, is_user_expanded, need_c_struct, is_separate: BOOLEAN is False

	need_gc_mark_function: BOOLEAN is False

	class_text_name: CLASS_NAME

	jvm_method_flags: INTEGER is 9

	frozen actual_reference (destination: TYPE_MARK): TYPE_REFERENCE is
		local
			type_reference: TYPE_REFERENCE; sp: POSITION
		do
			if destination /= Void then
				type_reference ?= destination
			end
			if type_reference /= Void and then type_reference.mapped.written_name = written_name then
				Result := type_reference
			else
				sp := start_position
				if sp.is_unknown then
					if destination /= Void then
						sp := destination.start_position
					end
				end
				create Result.make(sp, Current)
			end
		end

	frozen actual_separate (destination: TYPE_MARK): TYPE_MARK is
		do
		end

	frozen jvm_target_descriptor_in (str: STRING) is
		do
			jvm_descriptor_in(str)
		end

	frozen jvm_check_class_invariant is
		do
		end

	frozen jvm_standard_is_equal is
		local
			ca: like code_attribute; point1, point2: INTEGER
		do
			ca := code_attribute
			point1 := jvm_if_x_eq
			ca.opcode_iconst_0
			point2 := ca.opcode_goto
			ca.resolve_u2_branch(point1)
			ca.opcode_iconst_1
			ca.resolve_u2_branch(point2)
		end

	frozen start_position: POSITION is
		do
			Result := class_text_name.start_position
		end

	frozen local_from_separate: TYPE_MARK is
		do
			check
				False
			end
		end

	frozen run_type: TYPE_MARK is
		do
			Result := Current
		end

	frozen c_header_pass1 is
		do
		end

	frozen c_header_pass2 is
		do
		end

	frozen c_header_pass3 is
		do
		end

	frozen c_header_pass4 is
		do
		end

	frozen c_initialize is
		do
			if is_pointer then
				cpp.put_string(fz_null)
			else
				cpp.put_character('0')
			end
		end

	frozen c_initialize_in (str: STRING) is
		do
			if is_pointer then
				str.append(fz_null)
			else
				str.extend('0')
			end
		end

	frozen c_type_for_target_in (str: STRING) is
		do
			c_type_for_argument_in(str)
		end

	frozen c_type_for_result_in (str: STRING) is
		do
			c_type_for_argument_in(str)
		end

feature {LIVE_TYPE, TYPE_MARK}
	frozen just_before_gc_mark_in (str: STRING) is
		do
		end

	frozen gc_info_in (str: STRING) is
		do
		end

	frozen gc_define1 is
		do
		end

	frozen gc_define2 is
		do
		end

feature {TYPE_MARK}
	frozen short_ (shorted_type: TYPE) is
		do
			short_printer.put_class_name(class_text_name)
		end

feature {}
	is_a_end_hook (other: TYPE_MARK): BOOLEAN is
		require
			other.is_static
		local
			ref_type: TYPE_REFERENCE
		do
			ref_type ?= other
			if ref_type /= Void then
				if Current.is_a(ref_type.mapped) then
					Result := True
				else
					error_handler.cancel
				end
			else
				Result := class_text.is_subclass_of(other.class_text)
			end
		end

	can_be_assigned_to_end_hook (other: TYPE_MARK; t, o: TYPE; tp, op: POSITION): BOOLEAN is
		require
			other.is_static
		local
			ref_type: TYPE_REFERENCE
		do
			ref_type ?= other
			if ref_type /= Void then
				if can_be_assigned_to(ref_type.mapped, t, ref_type.mapped.type, tp, op) then
					--** perf: type lookup
					Result := True
				else
					error_handler.cancel
				end
			else
				Result := t.is_conform_to(o)
			end
		end

	smallest_ancestor_end_hook (other: TYPE_MARK): TYPE_MARK is
		require
			other.is_static
		local
			ref_type: TYPE_REFERENCE
		do
			ref_type ?= other
			if ref_type /= Void then
				if ref_type.mapped.is_a(Current) then
					Result := other
				else
					error_handler.cancel
					Result := smart_eiffel.type_any.canonical_type_mark
				end
			else
				Result := smart_eiffel.type_any.canonical_type_mark
			end
		end

end -- class KERNEL_EXPANDED_TYPE_MARK
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
