-- This file is part of SmartEiffel The GNU Eiffel Compiler Tools and Libraries.
-- See the Copyright notice at the end of this file.
--
class NON_EMPTY_TUPLE_TYPE_MARK
	--
	-- For non empty TUPLE declaration type_mark like: "TUPLE[BAR]" or "TUPLE[BAR, ZOO]".
	--

inherit
	TUPLE_TYPE_MARK
	GENERIC_TYPE_MARK
		undefine is_tuple
		redefine pretty_in
		end

creation {ANY}
	make

feature {ANY}
	as_type_mark: TYPE_MARK is
		do
			Result := Current
		end
	
	pretty_in (buffer: STRING) is
		local
			i: INTEGER
		do
			buffer.append(once "TUPLE")
			buffer.extend('[')
			from
				i := generic_list.lower
			until
				i > generic_list.upper
			loop
				generic_list.item(i).pretty_in(buffer)
				i := i + 1
				if i <= generic_list.upper then
					buffer.append(once ", ")
				end
			end
			buffer.extend(']')
		end

	count: INTEGER is
		do
			Result := generic_list.count
		end

	is_a (other: TYPE_MARK): BOOLEAN is
		local
			other_non_empty_tuple: like Current; other_run_types: ARRAY[TYPE_MARK]; i: INTEGER; t1, t2: TYPE_MARK
		do
			Result := True
			if other.is_tuple then
				other_non_empty_tuple ?= other
				if other_non_empty_tuple = Void then
				elseif other_non_empty_tuple.count > count then
					Result := False
				else
					from
						other_run_types := other_non_empty_tuple.generic_list
						if other_run_types /= Void then
							i := other_run_types.upper
						end
					until
						not Result or else i <= 0
					loop
						t1 := generic_list.item(i)
						t2 := other_run_types.item(i)
						if t1.is_a(t2) then
							i := i - 1
						else
							Result := False
							error_handler.add_position(t1.start_position)
							error_handler.add_position(t2.start_position)
						end
					end
				end
			elseif other.is_any or else other.class_text_name.to_string = as_general then
			else
				Result := False
			end
			if not Result then
				error_handler.type_error(Current, other)
			end
		end

	smallest_ancestor (other: TYPE_MARK): TYPE_MARK is
		local
			rto: TYPE_MARK; unknown_position: POSITION; i, c: INTEGER; t1, t2: TYPE_MARK
			other_tuple: like Current; common, run_types, other_run_types: ARRAY[TYPE_MARK]
		do
			run_types := generic_list
			rto := other
			other_tuple ?= rto
			if other_tuple /= Void then
				if other_tuple.count = 0 then
					Result := other
				else
					other_run_types := other_tuple.generic_list
					c := count.min(other_tuple.count)
					create common.make(1, c)
					from
						i := common.lower
					until
						i > common.upper
					loop
						t1 := run_types.item(i)
						t2 := other_run_types.item(i)
						common.put(t1.smallest_ancestor(t2), i)
						i := i + 1
					end
					create {NON_EMPTY_TUPLE_TYPE_MARK} Result.make(unknown_position, common)
				end
			else
				Result := rto.smallest_ancestor(Current)
			end
		end

	accept (visitor: NON_EMPTY_TUPLE_TYPE_MARK_VISITOR) is
		do
			visitor.visit_non_empty_tuple_type_mark(Current)
		end

feature {TYPE_MARK}
	short_ (shorted_type: TYPE) is
		do
			short_generic(shorted_type, create {CLASS_NAME}.make(string_aliaser.tuple_name, start_position))
		end

feature {}
	make (sp: like start_position; gl: like generic_list) is
		require
			gl.lower = 1 and gl.count > 0
		local
			buffer: STRING
		do
			generic_list := gl
			buffer := once "TUPLE..."
			buffer.copy(as_tuple)
			if gl /= Void then
				buffer.extend(' ')
				gl.count.append_in(buffer)
			end
			create class_text_name.make(string_aliaser.hashed_string(buffer), sp)
		ensure
			start_position = sp
			generic_list = gl
		end

end -- class NON_EMPTY_TUPLE_TYPE_MARK
--
-- ------------------------------------------------------------------------------------------------------------------------------
-- Copyright notice below. Please read.
--
-- SmartEiffel is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License,
-- as published by the Free Software Foundation; either version 2, or (at your option) any later version.
-- SmartEiffel is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
-- received a copy of the GNU General Public License along with SmartEiffel; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--
-- Copyright(C) 1994-2002: INRIA - LORIA (INRIA Lorraine) - ESIAL U.H.P.       - University of Nancy 1 - FRANCE
-- Copyright(C) 2003-2004: INRIA - LORIA (INRIA Lorraine) - I.U.T. Charlemagne - University of Nancy 2 - FRANCE
--
-- Authors: Dominique COLNET, Philippe RIBET, Cyril ADRIAN, Vincent CROIZIER, Frederic MERIZEN
--
-- http://SmartEiffel.loria.fr - SmartEiffel@loria.fr
-- ------------------------------------------------------------------------------------------------------------------------------
