class MANIFEST_EXPRESSION
--
-- To know more about manifest expression syntax and semantic, you should read this file.
--

feature {}

   very_common_manifest_expression_examples is
	 -- Here is the corresponding unique default type for all common manifest expression.
      do
	 -- INTEGER_8 range:
	 integer_8 := -128
	 integer_8 :=  127

	 -- INTEGER_16 range:
	 integer_16 := -32768
	 integer_16 :=  32767

	 -- INTEGER_32 range (synonym of INTEGER):
	 integer_32 := -2147483648
	 integer_32 :=  2147483647

	 -- INTEGER_64 range:
	 integer_64 := -9223372036854775808
	 integer_64 :=  9223372036854775807

	 -- REAL_64 examples:
	 real_64 := 3.5
	 real_64 := 3.5e32

	 -- Other common manifest expressions:
	 --    - STRING is not described here, because it is really 
	 --    common and well defined in ETL.
	 --    - To know more about hexadecimal manifest notation for INTEGERs, see also 
	 --    our tutorial hexadecimal.e file.
	 --    - To know more about UNICODE_STRING notation, see also 
	 --    examples in tutorial/unicode directory.
	 --
      end
   
   non_common_manifest_expression_examples is
      do
	 -- NATURAL_8 range:
	 natural_8 := {NATURAL_8 0 }
	 natural_8 := {NATURAL_8 255 }

	 -- NATURAL_16 range:
	 natural_16 := {NATURAL_16 0 }
	 natural_16 := {NATURAL_16 65535 }

	 -- NATURAL_32 range (synonym of NATURAL):
	 natural_32 := {NATURAL_32 0 }
	 natural_32 := {NATURAL_32 4194967295 }

	 -- NATURAL_64 range:
	 natural_64 := {NATURAL_64 0 }
	 natural_64 := {NATURAL_64 184466744073709551615 }

	 -- REAL_32 examples:
	 real_32 := {REAL_32 3.5 }
	 real_32 := {REAL_32 3.5e32 }

	 -- Manifest ARRAY example:
	 array_of_integer := {ARRAY[INTEGER] << 1, 2, 3, 4 >> }
	 
	 -- Manifest ARRAY example:
	 array_of_character := {ARRAY[CHARACTER] << 'a', 'b', 'c', 'd', 'e' >> }

	 -- Manifest ARRAY example (to show that values are computed):
	 array_of_integer_8 := {ARRAY[INTEGER_8] << 1, 1 + 1, 2 + 1, 2 * 2 >> }
	 
	 -- Set example:
	 set_of_character := {SET[CHARACTER] << 'a' , 'e', 'i', 'o', 'u' >> }


	 array_of_natural_8 := {ARRAY[NATURAL_8] << 1, x + y, 3, 4 >> }
	 
      end

   --
   -- Grammar for the non common manifest expressions:
   --
   -- Manifest_expression = Atomic_manifest | Multiple_values_manifest
   -- Atomic_manifest = "{" Atomic_type Atomic_value "}"
   -- Atomic_value = Integer_constant | Real_constant
   -- Multiple_values_manifest = "{" Generic_collection_type "<<" { Expression "," ... }* ">>" "}"
   -- Generic_collection_type = Class_name "[" Element_type "]"
   -- Element_type = Type
   --

   --
   -- Semantic (*)
   --
   --  For <Atomic_manifest>:
   --     - The <Atomic_value> must fit in the corresponding <Atomic_type>.
   --     - For all NATURAL* types, this is checked with a pure compiler-support. For 
   --     other types, it is possible to standardize the compiler-support interface to 
   --     be able to handle user-defined classes with the same syntax (not detailed here).  
   --
   -- For <Multiple_values_manifest>:
   --     - The <Generic_collection_type> must be a generic type with 
   --     only one generic argument, furthermore, the <Generic_collection_type> must have at least 
   --     one exported creation procedure which is exported to ANY.
   --     - The type of each <Expression> must conforms with <Element_type>.
   --     - The compiler-support interface is not detailed here (as an example, see routines `manifest_allocation'
   --       and `manifest_set_item' of class ARRAY). The compiler-support interface may be normalized too.
   --
   -- (*) This is just a first draft that we will continue if the ECMA commitee agree with this proposal.
   --
   
feature {} -- Name to corresponding TYPE-name declaration list:

   integer_8: INTEGER_8
   
   integer_16: INTEGER_16
   
   integer_32: INTEGER_32
   
   integer_64: INTEGER_64
   
   natural_8: NATURAL_8
   
   natural_16: NATURAL_16
   
   natural_32: NATURAL_32
   
   natural_64: NATURAL_64
   
   real_32: REAL_32
   
   real_64: REAL_64
   
   array_of_integer: ARRAY[INTEGER]
	 
   array_of_character: ARRAY[CHARACTER]

   array_of_integer_8: ARRAY[INTEGER_8]

   set_of_character: SET[CHARACTER]

end
